@@ -9,6 +9,13 @@ __license__     = "MIT (http://opensource.org/licenses/MIT)"
 import logging
 log = logging.getLogger(__name__)
 
+class Urispace(object):
+    """
+    Placeholder class for URI values in namespace.
+    """
+    def __init__(self):
+        return
+
 class Curiespace(object):
     """
     Placeholder class for CURIE values in namespace.
@@ -45,6 +52,7 @@ class Namespace(object):
         """
         self._prefix  = prefix
         self._baseUri = baseUri
+        self.URI      = Urispace()
         self.CURIE    = Curiespace()
         return
 
@@ -69,17 +77,25 @@ class Namespace(object):
             return self.mk_uri(parts[1])
         return curie
 
+    def __getattr__(self, name):
+        """
+        Return value for <namespace>.<name>
+        """
+        # Called for attributes that aren't defined.
+        # Placeholder should generate error
+        return self.__dict__[name]
+
 def makeNamespace(prefix, baseUri, names):
     """
     Create a namespace with given prefix, base URI and set of local names.
 
-    Returns the namespace value.  Attributes of the namespace value are URIs
-    for the corresponding identifier (e.g. ANNAL.Site, cf. below).  Attributes of
+    Returns the namespace value.  Attributes of the URI attribute are URIs for
+    the corresponding identifier (e.g. ANNAL.URI.Site, cf. below).  Attributes of
     the CURIE attribute are CURIES (e.g. ANNAL.CURIE.Site).
     """
     ns = Namespace(prefix, baseUri)
     for name in names:
-        setattr(ns, name, ns.mk_uri(name))
+        setattr(ns.URI,   name, ns.mk_uri(name))
         setattr(ns.CURIE, name, ns.mk_curie(name))
     return ns
 
@@ -112,39 +128,60 @@ OWL = makeNamespace("owl", "http://www.w3.org/2002/07/owl#",
 Annalist namespace terms
 """
 ANNAL = makeNamespace("annal", "http://purl.org/annalist/2014/#",
-    [ "EntityRoot", "Entity"
-    , "Site", "SiteData", "Collection", "Entity", "EntityRoot"
-    , "Collection_Types", "Collection_Views", "Collection_Lists"
-    , "Type_Data", "EntityData", "Metadata"
+    [ "EntityRoot", "Entity", "EntityData"
+    , "Site", "SiteData", "Collection"
+    # Internal IDs for MIME types (see resourcetypes.py)
+    , "Metadata", "Type_Data"
     # Entity types
-    , "User", "Type", "List", "View", "Field_group", "Field", "Enum"
-    , "Text", "Longtext", "Richtext", "Slug", "Identifier"
-    , "Placement", "Image", "Audio", "User", "Vocabulary"
-    , "Import", "Upload"
-    , "Default_type", "unknown_type"
-    # Properties
-    , "software_version", "comment", "inherit_from"
+    , "User", "Type", "List", "View", "Field_group", "Field", "Field_list", "Vocabulary", "Enum"
+    , "Enum_field_placement", "Enum_list_type", "Enum_render_type", "Enum_value_mode", "Enum_value_type"
+    # Group value types
+    , "View_field", "List_field", "Group_field"
+    # Value types
+    , "Text", "Longtext", "Richtext", "EntityRef", "Identifier"
+    , "Placement", "Image", "Audio", "Video"
+    , "Resource"
+    , "Default_type", "Unknown_type"
+    # Repeat/list group types
+    , "Type_supertype_uri" # @@deprecated?
+    # Properties in list JSON
+    , "entity_list"
+    # Properties in internal entities
     , "id", "type_id", "type"
-    , "label", "help", "url", "uri", "record_type"
-    , "supertype_uris", "supertype_uri"
+    , "url", "uri", "record_type"
+    # Types, Views, lists and field groups
+    , "default_type", "default_view"
+    , "supertype_uri"
     , "display_type", "type_list", "type_view"
     , "field_aliases", "alias_target", "alias_source"
-    , "user_uri", "user_permissions"
+    , "open_view", "view_fields"
+    , "edit_task_buttons", "view_task_buttons"
+    , "button_id", "button_label", "button_help"
+    , "list_entity_selector", "list_fields"
     , "group_fields"
-    , "view_fields"
-    , "list_entity_selector", "open_view"
-    , "list_entities", "list_fields"
-    , "placeholder", "default_value", "property_uri", "options_valkey"
+    , "field_id"
+    # Enumerated value terms
+    # , "enum_uri"
+    # User permissions
+    , "user_uri", "user_permission"
+    # Field definitions
+    , "field_render_type", "field_value_type", "field_value_mode"
+    , "field_entity_type"
+    , "placeholder", "tooltip", "default_value", "property_uri"
+    , "superproperty_uri"
     , "field_ref_type", "field_ref_restriction", "field_ref_field"
-    , "repeat", "repeat_id", "repeat_label", "repeat_label_add", "repeat_label_delete"
-    , "default_type", "default_view" , "default_list"
-    , "field_id", "field_name", "field_placement"
-    , "field_render_type", "field_value_mode", "field_entity_type"
-    , "field_value_type", "field_target_type"
-    , "group_ref", "repeat_label_add", "repeat_label_delete"
-    , "task_buttons", "button_id", "button_label"
-    # Deprecated properties - in migration tables
+    , "field_fields", "group_ref", "repeat_label_add", "repeat_label_delete"
+    , "field_name", "field_placement"
+    # Collection metadata
+    , "software_version", "meta_comment", "inherit_from"
+    , "default_list"
+    , "default_view_id", "default_view_type", "default_view_entity"
+    # Deprecated properties - used in entity migration
+    , "Slug", "RepeatGroup", "RepeatGroupRow" 
     , "options_typeref", "restrict_values", "target_field"
+    , "field_target_type", "comment"
+    , "supertype_uris"
+    , "user_permissions"
     ])
 
 # End.
