@@ -14,10 +14,12 @@ import re
 from pyparsing import Word, QuotedString, Literal, Group, Empty, StringEnd, ParseException
 from pyparsing import alphas, alphanums
 
-from annalist.util                  import extract_entity_id
+from annalist                       import layout
+from annalist.util                  import valid_id, extract_entity_id
 
+from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
-from annalist.models.entitytypeinfo import EntityTypeInfo, get_built_in_type_ids
+from annalist.models.entitytypeinfo import EntityTypeInfo
 
 #   -------------------------------------------------------------------
 #   Auxilliary functions
@@ -36,7 +38,6 @@ def order_entity_key(entity):
     key = ( 0 if type_id.startswith('_')   else 1, type_id, 
             0 if entity_id.startswith('_') else 1, entity_id
           )
-    # log.info(key)
     return key
 
 #   -------------------------------------------------------------------
@@ -59,44 +60,35 @@ class EntityFinder(object):
         # self._subtypes = None
         return
 
-    def get_collection_type_ids(self):
+    def get_collection_type_ids(self, altscope):
         """
         Returns iterator over possible type ids in current collection.
 
         Each type is returned as a candidate type identifier string
         """
-        for t in get_built_in_type_ids():
-            yield t
-        for t in self._coll._children(RecordTypeData):
-            yield t
-        return
+        return self._coll.cache_get_all_type_ids(altscope=altscope)
 
-    def get_collection_subtypes(self, type_id):
+    def get_collection_subtype_ids(self, supertype_id, altscope):
         """
-        Returns a iterator of `entitytypeinfo` objects for all subtypes
-        of the supplied type in the current collection, including the 
-        identified type itself.
+        Returns a iterator of type ids for all subtypes of the supplied type 
+        accessible in the indicated scope from the current collection, including 
+        the identified type itself.
         """
-        supertypeinfo = EntityTypeInfo(self._site, self._coll, type_id)
-        supertypeuri  = supertypeinfo.get_type_uri()
-        return self.get_collection_uri_subtypes(supertypeuri)
-
-    def get_collection_uri_subtypes(self, type_uri):
-        """
-        Returns a iterator of `entitytypeinfo` objects for all subtypes
-        of the supplied type in the current collection, including the 
-        identified type itself.
-        """
-        if type_uri is not None:
-            for tid in self.get_collection_type_ids():
-                tinfo = EntityTypeInfo(self._site, self._coll, tid)
-                if tinfo and (type_uri in tinfo.get_all_type_uris()):
-                    # log.info(
-                    #     "supertype %s, yield %s: %s"%
-                    #     (type_uri, tinfo.get_type_id(), tinfo.get_type_uri())
-                    #     )
-                    yield tinfo
-        return 
+        if not valid_id(supertype_id):
+            log.warning("EntityFinder.get_collection_subtype_ids: invalid type_id %s"%(supertype_id,))
+            return
+        supertype_info = EntityTypeInfo(self._coll, supertype_id)
+        supertype_uri  = supertype_info.get_type_uri()
+        if supertype_uri is not None:
+            for try_subtype_id in self.get_collection_type_ids(altscope):
+                try_subtype = self._coll.cache_get_type(try_subtype_id)
+                if try_subtype:
+                    try_subtype_uri = try_subtype.get_uri()
+                    if ( ( supertype_uri == try_subtype_uri ) or
+                         ( supertype_uri in self._coll.cache_get_supertype_uris(try_subtype_uri) ) ):
+                        yield try_subtype_id
+        else:
+            log.warning("EntityFinder.get_collection_subtype_ids: no type_uri for %s"%(supertype_id,))
 
     def get_type_entities(self, type_id, user_permissions, altscope):
         """
@@ -107,13 +99,16 @@ class EntityFinder(object):
         Otherwise only collection entities are included.        
         """
         #@@
-        # log.info("get_type_entities: type_id %s"%type_id)
+        # log.info("get_type_entities: type_id %s, user_permissions %r"%(type_id,user_permissions))
         #@@
-        entitytypeinfo = EntityTypeInfo(self._site, self._coll, type_id)
-        for e in entitytypeinfo.enum_entities_with_inferred_values(
+        entitytypeinfo = EntityTypeInfo(self._coll, type_id)
+        for e in entitytypeinfo.enum_entities_with_implied_values(
                 user_permissions, altscope=altscope
                 ):
-            if e.get_id() != "_initial_values":
+            if e.get_id() != layout.INITIAL_VALUES_ID:
+                #@@
+                # log.info("  yield: %s"%(e.get_id(),))
+                #@@
                 yield e
         return
 
@@ -126,26 +121,32 @@ class EntityFinder(object):
         a value of 'all' means that site-wide entities are included in the listing.
         Otherwise only collection entities are included.        
         """
-        # log.info("get_subtype_entities: type_id %s"%type_id)
-        for entitytypeinfo in self.get_collection_subtypes(type_id):
-            for e in entitytypeinfo.enum_entities_with_inferred_values(
+        for subtype_id in self.get_collection_subtype_ids(type_id, "all"):
+            subtype_info = EntityTypeInfo(self._coll, subtype_id)
+            es = subtype_info.enum_entities_with_implied_values(
                     user_permissions, altscope=altscope
-                    ):
-                # log.info(
-                #     "get_subtype_entities type_id %s, yield %s/%s"%
-                #     (type_id, e.get_type_id(), e.get_id())
-                #     )
-                if e.get_id() != "_initial_values":
+                    )
+            #@@
+            # es = list(es) #@@ Force strict eval
+            # log.info("get_subtype_entities: %r"%([e.get_id() for e in es],))
+            #@@
+            for e in es:
+                if e.get_id() != layout.INITIAL_VALUES_ID:
                     yield e
         return
 
     def get_all_types_entities(self, types, user_permissions, altscope):
         """
-        Iterate mover all entities of all type ids from a supplied type iterator
+        Iterate over all entities of all types from a supplied type iterator
         """
-        assert user_permissions is not None
+        #@@
+        # log.info("@@@@ get_all_types_entities")
+        #@@
         for t in types:
             for e in self.get_type_entities(t, user_permissions, altscope):
+                #@@
+                # log.info("get_all_types_entities: type %s/%s"%(t,e.get_id()))
+                #@@
                 yield e
         return
 
@@ -155,20 +156,19 @@ class EntityFinder(object):
 
         If a type_id is supplied, site data values are included.
         """
+        entities = None
         if type_id:
-            #@@
-            # log.info("get_base_entities: type_id %s"%type_id)
-            #@@
-            return self.get_subtype_entities(type_id, user_permissions, altscope)
+            entities = self.get_subtype_entities(type_id, user_permissions, altscope)
             # return self.get_type_entities(type_id, user_permissions, scope)
         else:
-            #@@
-            # log.info("get_base_entities: all types")
-            #@@
-            return self.get_all_types_entities(
-                self.get_collection_type_ids(), user_permissions, altscope
+            entities = self.get_all_types_entities(
+                self.get_collection_type_ids(altscope="all"), user_permissions, altscope
                 )
-        return
+        #@@
+        # entities = list(entities)  #@@ Force strict eval
+        # log.info("get_base_entities: %r"%([(e.get_type_id(), e.get_id()) for e in entities],))
+        #@@
+        return entities
 
     def search_entities(self, entities, search):
         """
@@ -180,30 +180,34 @@ class EntityFinder(object):
         return
 
     def get_entities(self, 
-        user_permissions=None, type_id=None, scope=None, context=None, search=None
+        user_permissions=None, type_id=None, altscope=None, context=None, search=None
         ):
         """
-        Get list of entities of the specified type, matching search term and visible to 
+        Iterates over entities of the specified type, matching search term and visible to 
         supplied user permissions.
         """
         entities = self._selector.filter(
-            self.get_base_entities(type_id, user_permissions, scope), context=context
+            self.get_base_entities(type_id, user_permissions, altscope), context=context
             )
         if search:
             entities = self.search_entities(entities, search)
         return entities
 
     def get_entities_sorted(self, 
-        user_permissions=None, type_id=None, scope=None, context={}, search=None
+        user_permissions=None, type_id=None, altscope=None, context={}, search=None
         ):
         """
         Get sorted list of entities of the specified type, matching search term and 
         visible to supplied user permissions.
         """
         entities = self.get_entities(
-            user_permissions, type_id=type_id, scope=scope, 
+            user_permissions, type_id=type_id, altscope=altscope, 
             context=context, search=search
             )
+        #@@
+        # entities = list(entities)  #@@ Force strict eval
+        # log.info("get_entities_sorted: %r"%([e.get_id() for e in entities],))
+        #@@
         return sorted(entities, key=order_entity_key)
 
     @classmethod
@@ -212,23 +216,55 @@ class EntityFinder(object):
         Returns True if entity contains/matches search term, else False.
         Search term None (or blank) matches all entities.
 
-        >>> e  = { 'p:a': '1', 'p:b': '2', 'p:c': '3' }
-        >>> EntityFinder.entity_contains(e, "1")
+        >>> e1  = { 'p:a': '1', 'p:b': '2', 'p:c': '3', 'annal:property_uri': 'annal:member' }
+        >>> EntityFinder.entity_contains(e1, "1")
         True
-        >>> EntityFinder.entity_contains(e, "3")
+        >>> EntityFinder.entity_contains(e1, "3")
         True
-        >>> EntityFinder.entity_contains(e, "nothere")
+        >>> EntityFinder.entity_contains(e1, "nothere")
+        False
+        >>> EntityFinder.entity_contains(e1, "annal:member")
+        True
+        >>> e2 = { 'list': ['l1', 'l2', 'l3'] \
+                 , 'dict': {'p:a': 'd1', 'p:b': 'd2', 'p:c': 'd3'} \
+                 }
+        >>> EntityFinder.entity_contains(e2, "l1")
+        True
+        >>> EntityFinder.entity_contains(e2, "d3")
+        True
+        >>> EntityFinder.entity_contains(e2, "nothere")
         False
         """
         if search:
+            # Entity is not a dict, so scan entity keys for search
             for key in e:
                 val = e[key]
-                if isinstance(val, (str, unicode)):
-                    if search in val:
-                        return True
+                if cls.value_contains(val, search):
+                    return True
+                # if isinstance(val, (str, unicode)):
+                #     if search in val:
+                #         return True
             return False
         return True
 
+    @classmethod
+    def value_contains(cls, val, search):
+        """
+        Helper function tests for search term in dictionary, list or string values.
+        Other values are not searched.
+        """
+        if isinstance(val, dict):
+            for k in val:
+                if cls.value_contains(val[k], search):
+                    return True
+        elif isinstance(val, list):
+            for e in val:
+                if cls.value_contains(e, search):
+                    return True
+        elif isinstance(val, (str, unicode)):
+            return search in val
+        return False
+
 #   -------------------------------------------------------------------
 #   EntitySelector
 #   -------------------------------------------------------------------
@@ -416,7 +452,8 @@ class EntitySelector(object):
             elif selval['type'] == "literal":
                 return get_literal(selval['value'])
             else:
-                raise ValueError("Unrecognized value type from selector (%s)"%selval['type'])
+                msg = "Unrecognized value type from selector (%s)"%selval['type']
+                raise ValueError(msg)
                 assert False, "Unrecognized value type from selector"
         #
         def match_eq(v1f, v2f):
@@ -443,7 +480,8 @@ class EntitySelector(object):
             return None
         sel = self.parse_selector(selector)
         if not sel:
-            raise ValueError("Unrecognized selector syntax (%s)"%selector)
+            msg = "Unrecognized selector syntax (%s)"%selector
+            raise ValueError(msg)
         v1f = get_val_f(sel['val1'])
         v2f = get_val_f(sel['val2'])
         if sel['comp'] == "==":
@@ -453,7 +491,8 @@ class EntitySelector(object):
         if sel['comp'] == "subtype":
             return match_subtype(v1f, v2f)
         # Drop through: raise error
-        raise ValueError("Unrecognized entity selector (%s)"%selector)
+        msg = "Unrecognized entity selector (%s)"%selector
+        raise ValueError(msg)
 
 #   -------------------------------------------------------------------
 #   FieldComparison
@@ -475,8 +514,8 @@ class FieldComparison(object):
         """
         Return typeinfo corresponding to the supplied type URI
         """
-        t     = self._coll.get_uri_type(type_uri)
-        return t and EntityTypeInfo(self._site, self._coll, t.get_id())
+        t = self._coll.get_uri_type(type_uri)
+        return t and EntityTypeInfo(self._coll, t.get_id())
 
     def subtype(self, type1_uri, type2_uri):
         """
