@@ -46,21 +46,26 @@ class EntityRoot(object):
     An entity presents at least the following interface:
         cls._entitytype     type of entity (CURIE or URI)
         cls._entitytypeid   local type id (slug) used in local URI construction
-        cls._entityfile     relative path to file where entity body is stored
+        cls._entityfile     name of file where the entity data is stored
         cls._entityref      relative reference to entity from body file
         cls._contextref     relative reference to context from body file
+
         self._entityid      ID of entity; may be None for "root" entities (e.g. site?)
         self._entityurl     URI at which entity is accessed
         self._entitydir     directory where entity is stored
         self._entitybasedir base directory where all data is stored
         self._values        dictionary of values in entity body
+
+    See also 'Entity'
     """
 
     _entitytype     = ANNAL.CURIE.EntityRoot
     _entitytypeid   = None          # To be overridden
+    _entitybase     = None          # To be overridden
     _entityfile     = None          # To be overriden by entity subclasses..
-    _entityref      = None          # Relative reference to entity from body file
-    _contextref     = None          # Relative reference to context file from body file
+    _entityref      = None          # Relative ref to entity from body file
+    _contextbase    = None          # Relative ref to collection base URI from body file
+    _contextref     = None          # Relative ref to context file from body file
 
     def __init__(self, entityurl, entityviewurl, entitydir, entitybasedir):
         """
@@ -75,14 +80,16 @@ class EntityRoot(object):
                         associated with this entity or any possible descendents.  The
                         value is used as a safety check to ensure that data is not 
                         created or deleted outside an area that is known to contain 
-                        only annalist data.
+                        only Annalist data.
         """
         self._entityid      = None
+        self._ancestorid    = None
         self._entityurl     = make_entity_base_url(entityurl)
         self._entityviewurl = make_entity_base_url(entityviewurl)
         self._entitydir     = make_entity_base_url(entitydir)
         self._entitybasedir = entitybasedir
         self._values        = None
+        self._errors        = []
         # log.debug("EntityRoot.__init__: entity URI %s, entity dir %s"%(self._entityurl, self._entitydir))
         return
 
@@ -133,7 +140,7 @@ class EntityRoot(object):
         presented as belonging to a collection.  This allows for collection-specific
         specializations to be created without changing the URI used.
         """
-        log.debug("EntityRoot.get_url: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
+        # log.debug("EntityRoot.get_url: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
         return urlparse.urljoin(baseurl, self._entityurl)
 
     def get_view_url(self, baseurl=""):
@@ -147,7 +154,7 @@ class EntityRoot(object):
         collection the URL is mapped via the web application to the underlying storage
         location.
         """
-        log.debug("EntityRoot.get_view_url: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
+        # log.debug("EntityRoot.get_view_url: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
         return urlparse.urljoin(baseurl, self._entityviewurl)
 
     def get_view_url_path(self, baseurl=""):
@@ -155,7 +162,7 @@ class EntityRoot(object):
         Return URL path used to view entity data.  This is the URI-path of the URL
         returned by get_view_url (above)
         """
-        log.debug("EntityRoot.get_view_url_path: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
+        # log.debug("EntityRoot.get_view_url_path: baseurl %s, _entityurl %s"%(baseurl, self._entityurl))
         return util.entity_url_path(self.get_view_url(), "")
 
     def get_alt_entities(self, altscope=None):
@@ -184,6 +191,38 @@ class EntityRoot(object):
         """
         return self._values
 
+    def get_save_values(self):
+        """
+        Return values that are or will be recorded when entity is saved.
+
+        Also used for cache values.
+        """
+        values = self._values.copy()
+        values['@id']      = self._entityref
+        values['@type']    = self._get_types(values.get('@type', None))
+        values['@context'] = (
+            [ { '@base': self._contextbase }
+            , self._contextref
+            ])
+        # @TODO: is this next needed?  Put logic in set_values?
+        if self._entityid:
+            values[ANNAL.CURIE.id] = self._entityid
+        values.pop(ANNAL.CURIE.url, None)
+        return values
+
+    def set_error(self, msg):
+        """
+        Records error/diagnostic information in an entity
+        """
+        self._errors.append(msg)
+        return
+
+    def get_errors(self):
+        """
+        Returns a list of error/diagnostic entries associated with an entity, or an empty list
+        """
+        return self._errors
+
     def get_uri(self):
         """
         Return URI for current entity, which may be set explicitly or derived from
@@ -265,11 +304,25 @@ class EntityRoot(object):
         """
         Return directory and path for current entity body file
         """
-        if not self._entityfile:
-            raise ValueError("Entity._dir_path without defined entity file path")
+        if self._entitybase is None:
+            msg = (
+                "EntityRoot._dir_path without defined entity base reference (%s/%s)"%
+                (self._entitytypeid, self._entityid,)
+                )
+            log.error(msg)
+            raise ValueError(msg)
+        if self._entityfile is None:
+            msg = (
+                "EntityRoot._dir_path without defined entity file path (%s/%s)"%
+                (self._entitytypeid, self._entityid,)
+                )
+            log.error(msg)
+            raise ValueError(msg)
         # log.info("    _ EntityRoot._dir_path _entitydir  %s"%(self._entitydir,))
+        # log.info("    _ EntityRoot._dir_path _entitybase %s"%(self._entitybase,))
         # log.info("    _ EntityRoot._dir_path _entityfile %s"%(self._entityfile,))
-        (basedir, filepath) = util.entity_dir_path(self._entitydir, [], self._entityfile)
+        (basedir, filepath) = util.entity_dir_path(self._entitydir, [self._entitybase], self._entityfile)
+        # log.info("    _ EntityRoot._dir_path basedir, filepath %s, %s"%(basedir, filepath))
         return (basedir, filepath)
 
     def _dir_path_uri(self):
@@ -288,9 +341,14 @@ class EntityRoot(object):
         # log.debug("EntityRoot._exists_path %s"%(p))
         if d and os.path.isdir(d):
             if p and os.path.isfile(p):
-                log.debug("EntityRoot._exists_path %s: OK"%(p))
+                # log.debug("EntityRoot._exists_path %s: OK"%(p))
                 return p
-        log.debug("EntityRoot._exists_path %s: not present"%(p))
+            mp = self._migrate_path()
+            if mp and os.path.isfile(mp):
+                assert mp == p, "EntityRoot._exists_path: Migrated filename %s, expected %s"%(mp, p)
+                # log.info("EntityRoot._exists_path %s: Migrated from %s"%(mp, p))
+                return mp
+        # log.debug("EntityRoot._exists_path %s: not present"%(p))
         return None
 
     def _exists(self):
@@ -320,53 +378,53 @@ class EntityRoot(object):
             types.append(self._entitytype)
         return types
 
-    def _save(self):
+    def _save(self, post_update_flags=None):
         """
         Save current entity to Annalist storage
         """
         # @@TODO: think about capturing provenance metadata too.
         if not self._entityref:
-            raise ValueError("Entity._save without defined entity reference")
+            msg = "Entity._save without defined entity reference"
+            log.error(msg)
+            raise ValueError(msg)
+        if not self._contextbase:
+            msg = "Entity._save without defined context base"
+            log.error(msg)
+            raise ValueError(msg)
         if not self._contextref:
-            raise ValueError("Entity._save without defined context reference")
+            msg = "Entity._save without defined context reference"
+            log.error(msg)
+            raise ValueError(msg)
         if not self._values:
-            raise ValueError("Entity._save without defined entity values")
+            msg = "Entity._save without defined entity values"
+            log.error(msg)
+            raise ValueError(msg)
         (body_dir, body_file) = self._dir_path()
-        log.debug("EntityRoot._save: dir %s, file %s"%(body_dir, body_file))
+        # log.debug("EntityRoot._save: dir %s, file %s"%(body_dir, body_file))
         fullpath = os.path.join(settings.BASE_DATA_DIR, "annalist_site", body_file)
         # Next is partial protection against code errors
         if not fullpath.startswith(os.path.join(settings.BASE_DATA_DIR, "annalist_site")):
             log.error("EntityRoot._save: Failing to create entity at %s"%(fullpath,))
             log.info("EntityRoot._save: dir %s, file %s"%(body_dir, body_file))
             log.info("EntityRoot._save: settings.BASE_DATA_DIR %s"%(settings.BASE_DATA_DIR,))
-            raise ValueError(
+            msg = (
                 "Attempt to create entity file outside Annalist site tree (%s)"%
                 fullpath
                 )
+            log.error(msg)
+            raise ValueError(msg)
         # Create directory (if needed) and save data
         util.ensure_dir(body_dir)
-        values = self._values.copy()
-        values['@id']      = self._entityref
-        values['@type']    = self._get_types(values.get('@type', None))
-        values['@context'] = (
-            [ self._contextref
-            # layout.COLL_CONTEXT_FILE
-            # layout.ENTITY_CONTEXT_PATH + "/" + layout.COLL_CONTEXT_FILE
-            # , { '@base': layout.ENTITY_CONTEXT_PATH }
-            ])
-        # @TODO: is this next needed?  Put logic in set_values?
-        if self._entityid:
-            values[ANNAL.CURIE.id] = self._entityid
-        values.pop(ANNAL.CURIE.url, None)
+        values = self.get_save_values()
         with open(fullpath, "wt") as entity_io:
-            json.dump(values, entity_io, indent=2, separators=(',', ': '))
-        self._post_update_processing(values)
+            json.dump(values, entity_io, indent=2, separators=(',', ': '), sort_keys=True)
+        self._post_update_processing(values, post_update_flags)
         return
 
-    def _remove(self, type_uri):
+    def _remove(self, type_uri, post_remove_flags=None):
         """
-        Remove current entity from Annalist storage.  Requires typ_uri supplied as a 
-        double-check that the expected enytity is being removed.
+        Remove current entity from Annalist storage.  Requires type_uri supplied as a 
+        double-check that the expected entity is being removed.
         """
         d = self._entitydir
         # Extra check to guard against accidentally deleting wrong thing
@@ -376,6 +434,7 @@ class EntityRoot(object):
             log.error("Expected type_uri: %r, got %r"%(type_uri, e[ANNAL.CURIE.type]))
             log.error("Expected dirbase:  %r, got %r"%(parent._entitydir, d))
             raise Annalist_Error("Entity %s unexpected type %s or path %s"%(entityid, e[ANNAL.CURIE.type_id], d))
+        self._post_remove_processing(post_remove_flags)
         return
 
     def _load_values(self):
@@ -384,9 +443,7 @@ class EntityRoot(object):
 
         Adds value for 'annal:url' to the entity data returned.
         """
-        log.debug("EntityRoot._load_values %s/%s"%(self.get_type_id(), self.get_id()))
-        #@@TODO: rework logic to that _migrate_path is called only when expected filename is not found.
-        self._migrate_path()
+        # log.debug("EntityRoot._load_values %s/%s"%(self.get_type_id(), self.get_id()))
         body_file = self._exists_path()
         if body_file:
             # log.debug("EntityRoot._load_values body_file %r"%(body_file,))
@@ -426,27 +483,28 @@ class EntityRoot(object):
     def _migrate_path(self):
         """
         Migrate entity data filenames from those used in older software versions.
+
+        Returns name of migrated file if migration performed, otherwise None
         """
         # log.debug("EntityRoot._migrate_path (%r)"%(self._migrate_filenames(),))
         if self._migrate_filenames() is None:
             # log.debug("EntityRoot._migrate_path (skip)")
             return
-        if not self._exists():
-            for old_data_filename in self._migrate_filenames():
-                # This logic migrates data from previous filenames
-                (basedir, old_data_filepath) = util.entity_dir_path(self._entitydir, [], old_data_filename)
-                if basedir and os.path.isdir(basedir):
-                    if old_data_filepath and os.path.isfile(old_data_filepath):
-                        # Old body file found here
-                        (d, new_data_filepath) = self._dir_path()
-                        log.info(
-                            "EntityRoot._migrate_path: Migrate file %s to %s"%
-                            (old_data_filepath, new_data_filepath)
-                            )
-                        os.rename(old_data_filepath, new_data_filepath)
-                        break
-        log.debug("EntityRoot._migrate_path (done)")
-        return
+        for old_data_filename in self._migrate_filenames():
+            # This logic migrates data from previous filenames
+            (basedir, old_data_filepath) = util.entity_dir_path(self._entitydir, [], old_data_filename)
+            if basedir and os.path.isdir(basedir):
+                if old_data_filepath and os.path.isfile(old_data_filepath):
+                    # Old body file found here
+                    (d, new_data_filepath) = self._dir_path()
+                    log.info(
+                        "EntityRoot._migrate_path: Migrate file %s to %s"%
+                        (old_data_filepath, new_data_filepath)
+                        )
+                    os.rename(old_data_filepath, new_data_filepath)
+                    return new_data_filepath
+        # log.debug("EntityRoot._migrate_path (not found)")
+        return None
 
     def _migrate_filenames(self):
         """
@@ -481,18 +539,44 @@ class EntityRoot(object):
         """
         return entitydata
 
-    def _post_update_processing(self, entitydata):
+    def _migrate_values_map_field_names(self, migration_map, entitydata):
+        """
+        Support function to map field names using a supplied map.
+
+        The map is a list of pairs (old_uri, new_uri), where occurrences of the 
+        old property URI are replaced with the same value using the new URI.
+
+        The migrations are applied in-place, and the resulting updated entity 
+        data is returned.
+        """
+        for old_key, new_key in migration_map:
+            if old_key in entitydata:
+                entitydata[new_key] = entitydata.pop(old_key)
+        # Return result
+        return entitydata
+
+    def _post_update_processing(self, entitydata, post_update_flags):
         """
         Default method for post-update processing.
 
-        This method is called when an entity has been updated.  
+        This method is called when an entity has been ceated or updated.  
 
-        Individual entity classes may provide their own override methiods for this.  
+        Individual entity classes may provide their own override methods for this.  
         (e.g. to trigger regeneration of context data when groups, views, fields or 
         vocabulary descriptions are updated.)
         """
         return entitydata
 
+    def _post_remove_processing(self, post_update_flags):
+        """
+        Default method for post-remove processing.
+
+        This method is called when an entity has been removed.  
+
+        Individual entity classes may provide their own override methods for this.
+        """
+        return
+
     def _base_children(self, cls):
         """
         Iterates over candidate child identifiers that are possible instances of an 
@@ -502,8 +586,8 @@ class EntityRoot(object):
         cls         is a subclass of Entity indicating the type of children to
                     iterate over.
         """
-        parent_dir = os.path.dirname(os.path.join(self._entitydir, cls._entitypath or ""))
-        assert "%" not in parent_dir, "_entitypath template variable interpolation may be in filename part only"
+        parent_dir = os.path.dirname(os.path.join(self._entitydir, cls._entityroot or ""))
+        assert "%" not in parent_dir, "_entityroot template variable interpolation may be in filename part only"
         child_files = []
         if os.path.isdir(parent_dir):
             child_files = os.listdir(parent_dir)
@@ -520,16 +604,19 @@ class EntityRoot(object):
 
         cls         is a subclass of Entity indicating the type of children to
                     iterate over.
-        altscope    if supplied value is not "all" or "site", returns empty iterator,
+        altscope    if the supplied value is "site", returns an empty iterator,
                     otherwise an iterator over child entities.
 
         NOTE: `Site` class overrides this.
         """
+        # log.info("@@ EntityRoot._children: parent %s, altscope %s"%(self.get_id(), altscope))
         if altscope != "site":
             return self._base_children(cls)
         return iter(())     # Empty iterator
 
     def _entity_files(self):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entitytypeinfo'
         """
         Iterates over files/resources (not subdirectories) that are part of the current entity.
 
@@ -542,26 +629,45 @@ class EntityRoot(object):
                 yield (p, f)
         return
 
-    def _entity_files_dirs(self):
+    def _copy_entity_files(self, src_entity):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entityedit', 'am_managecollections'
         """
-        Iterates over files/resources that are part of the current entity.
+        Copy metadata abnd attached resources from the supplied `src_entity` 
+        to the current entity.
 
-        Returns pairs (p,f), where 'p' is a full path name, and 'f' is a filename within the 
-        current entity directory. 
+        Resources that already exist for the current entty are not copied.
+
+        returns     list of error messages; an empty list indicates success.
         """
-        for f in os.listdir(self._entitydir):
-            p = os.path.join(self._entitydir, f)
-            if os.path.exists(p):
-                yield (p, f)
-        return
+        msgs = []
+        for p, f in src_entity._entity_files():
+            if not self._exists_file(f):
+                p_new = self._copy_file(p, f)
+                if not p_new:
+                    msg_vals = (
+                        { 'id':     self.get_id()
+                        , 'src_id': src_entity.get_id()
+                        , 'file':   f
+                        })
+                    log.warning(
+                        "EntityRoot._copy_entity_files: error copying file %(file)s from %(src_id)s to %(id)s"%
+                        msg_vals
+                        )
+                    msgs.APPEND(message.ENTITY_COPY_FILE_ERROR%msg_vals)
+        return msgs
 
     def _exists_file(self, f):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entitytypeinfo'
         """
         Test if a file named 'f' exists in the current entity directory
         """
         return os.path.isfile(os.path.join(self._entitydir, f))
 
     def _copy_file(self, p, f):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entitytypeinfo'
         """
         Copy file with path 'p' to a new file 'f' in the current entity directory
         """
@@ -577,6 +683,8 @@ class EntityRoot(object):
         return new_p
 
     def _rename_files(self, old_entity):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entitytypeinfo'
         """
         Rename old entity files to path of current entity (which must not exist),
         and return path to resulting entity, otherwise None.
@@ -603,6 +711,9 @@ class EntityRoot(object):
         return new_p
 
     def _fileobj(self, localname, filetypeuri, mimetype, mode):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'entity', 'entitytypeinfo', 'site', 'entityedit', 'util',
+        #                'test_import_resource', 'test_render_ref_multifields', 'test_upload_file'
         """
         Returns a file object for accessing a blob associated with the current entity.
 
@@ -624,6 +735,8 @@ class EntityRoot(object):
         return open(file_name, mode)
 
     def _metaobj(self, localpath, localname, mode):
+        #@@TODO: abstract logic to work with non-file storage
+        #        Used by 'collection', 'site'
         """
         Returns a file object for accessing a metadata resource associated with 
         the current entity.
@@ -639,9 +752,9 @@ class EntityRoot(object):
         local_dir = os.path.join(body_dir, localpath)
         util.ensure_dir(local_dir)
         filename = os.path.join(local_dir, localname)
-        log.debug("entityroot._metaobj: self._entitydir %s"%(self._entitydir,))
-        log.debug("entityroot._metaobj: body_dir %s, body_file %s"%(body_dir, body_file))
-        log.debug("entityroot._metaobj: filename %s"%(filename,))
+        # log.debug("entityroot._metaobj: self._entitydir %s"%(self._entitydir,))
+        # log.debug("entityroot._metaobj: body_dir %s, body_file %s"%(body_dir, body_file))
+        # log.debug("entityroot._metaobj: filename %s"%(filename,))
         return open(filename, mode)
 
     def _read_stream(self):
