@@ -6,6 +6,7 @@ __author__      = "Graham Klyne (GK@ACM.ORG)"
 __copyright__   = "Copyright 2014, G. Klyne"
 __license__     = "MIT (http://opensource.org/licenses/MIT)"
 
+import traceback
 import logging
 log = logging.getLogger(__name__)
 
@@ -18,26 +19,26 @@ from annalist.util                  import valid_id, extract_entity_id
 from annalist.identifiers           import ANNAL, RDF, RDFS
 
 from annalist.models.collection     import Collection
-from annalist.models.annalistuser   import AnnalistUser
+from annalist.models.annalistuser   import AnnalistUser, site_default_user_id, default_user_id, unknown_user_id
 from annalist.models.recordtype     import RecordType
 from annalist.models.recordlist     import RecordList
 from annalist.models.recordview     import RecordView
-from annalist.models.recordgroup    import RecordGroup
+from annalist.models.recordgroup    import RecordGroup, RecordGroup_migration
 from annalist.models.recordfield    import RecordField
 from annalist.models.recordvocab    import RecordVocab
 from annalist.models.recordenum     import RecordEnumFactory
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-COLL_ID     = '_coll'
-USER_ID     = '_user'
-TYPE_ID     = '_type'
-LIST_ID     = '_list'
-VIEW_ID     = '_view'
-GROUP_ID    = '_group'
-FIELD_ID    = '_field'
-VOCAB_ID    = '_vocab'
-TASK_ID     = '_task'
+COLL_ID     = layout.COLL_TYPEID
+USER_ID     = layout.USER_TYPEID
+TYPE_ID     = layout.TYPE_TYPEID
+LIST_ID     = layout.LIST_TYPEID
+VIEW_ID     = layout.VIEW_TYPEID
+GROUP_ID    = layout.GROUP_TYPEID
+FIELD_ID    = layout.FIELD_TYPEID
+VOCAB_ID    = layout.VOCAB_TYPEID
+TASK_ID     = layout.TASK_TYPEID
 
 COLL_MESSAGES = (
     { 'parent_heading':         "(@@ COLL_MESSAGES.parent_heading - unused message @@)"
@@ -208,65 +209,69 @@ ENTITY_PERMISSIONS = (
     })
 
 TYPE_CLASS_MAP = (
-    { COLL_ID:              Collection
-    , USER_ID:              AnnalistUser
-    , TYPE_ID:              RecordType
-    , LIST_ID:              RecordList
-    , VIEW_ID:              RecordView
-    , GROUP_ID:             RecordGroup
-    , FIELD_ID:             RecordField
-    , VOCAB_ID:             RecordVocab
-    , 'Enum_list_type':     RecordEnumFactory('Enum_list_type',   'Enum_list_type')
-    , 'Enum_render_type':   RecordEnumFactory('Enum_render_type', 'Enum_render_type')
-    , 'Enum_value_mode':    RecordEnumFactory('Enum_value_mode',  'Enum_value_mode')
-    , 'Enum_bib_type':      RecordEnumFactory('Enum_bib_type',    'Enum_bib_type')
+    { COLL_ID:                  Collection
+    , USER_ID:                  AnnalistUser
+    , TYPE_ID:                  RecordType
+    , LIST_ID:                  RecordList
+    , VIEW_ID:                  RecordView
+    , GROUP_ID:                 RecordGroup_migration
+    , FIELD_ID:                 RecordField
+    , VOCAB_ID:                 RecordVocab
+    , '_enum_field_placement':  RecordEnumFactory('_enum_field_placement',  '_enum_field_placement')
+    , '_enum_list_type':        RecordEnumFactory('_enum_list_type',        '_enum_list_type')
+    , '_enum_render_type':      RecordEnumFactory('_enum_render_type',      '_enum_render_type')
+    , '_enum_value_mode':       RecordEnumFactory('_enum_value_mode',       '_enum_value_mode')
+    , '_enum_value_type':       RecordEnumFactory('_enum_value_type',       '_enum_value_type')
     })
 
 TYPE_MESSAGE_MAP = (
-    { COLL_ID:              COLL_MESSAGES
-    , USER_ID:              USER_MESSAGES
-    , TYPE_ID:              TYPE_MESSAGES
-    , LIST_ID:              LIST_MESSAGES
-    , VIEW_ID:              VIEW_MESSAGES
-    , GROUP_ID:             GROUP_MESSAGES
-    , FIELD_ID:             FIELD_MESSAGES
-    , VOCAB_ID:             VOCAB_MESSAGES
-    , 'Enum_list_type':     ENUM_MESSAGES
-    , 'Enum_render_type':   ENUM_MESSAGES
-    , 'Enum_value_mode':    ENUM_MESSAGES
-    , 'Enum_bib_type':      ENUM_MESSAGES
+    { COLL_ID:                  COLL_MESSAGES
+    , USER_ID:                  USER_MESSAGES
+    , TYPE_ID:                  TYPE_MESSAGES
+    , LIST_ID:                  LIST_MESSAGES
+    , VIEW_ID:                  VIEW_MESSAGES
+    , GROUP_ID:                 GROUP_MESSAGES
+    , FIELD_ID:                 FIELD_MESSAGES
+    , VOCAB_ID:                 VOCAB_MESSAGES
+    , '_enum_field_placement':  ENUM_MESSAGES  
+    , '_enum_list_type':        ENUM_MESSAGES
+    , '_enum_render_type':      ENUM_MESSAGES
+    , '_enum_value_mode':       ENUM_MESSAGES
+    , '_enum_value_type':       ENUM_MESSAGES
     })
 
 SITE_PERMISSIONS_MAP = (
-    { COLL_ID:              SITE_PERMISSIONS
-    , USER_ID:              ADMIN_PERMISSIONS
-    , TYPE_ID:              SITE_PERMISSIONS
-    , LIST_ID:              SITE_PERMISSIONS
-    , VIEW_ID:              SITE_PERMISSIONS
-    , GROUP_ID:             SITE_PERMISSIONS
-    , FIELD_ID:             SITE_PERMISSIONS
-    , VOCAB_ID:             ADMIN_PERMISSIONS  #@@ CONFIG??
-    , 'Enum_list_type':     SITE_PERMISSIONS
-    , 'Enum_render_type':   SITE_PERMISSIONS
-    , 'Enum_value_mode':    SITE_PERMISSIONS
-    , 'Enum_bib_type':      SITE_PERMISSIONS
-    , 'EntityData':         SITE_PERMISSIONS
+    { COLL_ID:                  SITE_PERMISSIONS
+    , USER_ID:                  ADMIN_PERMISSIONS
+    , TYPE_ID:                  SITE_PERMISSIONS
+    , LIST_ID:                  SITE_PERMISSIONS
+    , VIEW_ID:                  SITE_PERMISSIONS
+    , GROUP_ID:                 SITE_PERMISSIONS
+    , FIELD_ID:                 SITE_PERMISSIONS
+    , VOCAB_ID:                 SITE_PERMISSIONS
+    , '_enum_field_placement':  SITE_PERMISSIONS  
+    , '_enum_list_type':        SITE_PERMISSIONS
+    , '_enum_render_type':      SITE_PERMISSIONS
+    , '_enum_value_mode':       SITE_PERMISSIONS
+    , '_enum_value_type':       SITE_PERMISSIONS
+    , 'EntityData':             SITE_PERMISSIONS
     })
 
 TYPE_PERMISSIONS_MAP = (
-    { COLL_ID:              CONFIG_PERMISSIONS
-    , USER_ID:              ADMIN_PERMISSIONS
-    , TYPE_ID:              CONFIG_PERMISSIONS
-    , LIST_ID:              CONFIG_PERMISSIONS
-    , VIEW_ID:              CONFIG_PERMISSIONS
-    , GROUP_ID:             CONFIG_PERMISSIONS
-    , FIELD_ID:             CONFIG_PERMISSIONS
-    , VOCAB_ID:             CONFIG_PERMISSIONS
-    , 'Enum_list_type':     CONFIG_PERMISSIONS
-    , 'Enum_render_type':   CONFIG_PERMISSIONS
-    , 'Enum_value_mode':    CONFIG_PERMISSIONS
-    , 'Enum_bib_type':      CONFIG_PERMISSIONS
-    , 'EntityData':         ENTITY_PERMISSIONS
+    { COLL_ID:                  CONFIG_PERMISSIONS
+    , USER_ID:                  ADMIN_PERMISSIONS
+    , TYPE_ID:                  CONFIG_PERMISSIONS
+    , LIST_ID:                  CONFIG_PERMISSIONS
+    , VIEW_ID:                  CONFIG_PERMISSIONS
+    , GROUP_ID:                 CONFIG_PERMISSIONS
+    , FIELD_ID:                 CONFIG_PERMISSIONS
+    , VOCAB_ID:                 CONFIG_PERMISSIONS
+    , '_enum_field_placement':  SITE_PERMISSIONS  
+    , '_enum_list_type':        SITE_PERMISSIONS
+    , '_enum_render_type':      SITE_PERMISSIONS
+    , '_enum_value_mode':       SITE_PERMISSIONS
+    , '_enum_value_type':       SITE_PERMISSIONS
+    , 'EntityData':             ENTITY_PERMISSIONS
     })
 
 def get_built_in_type_ids():
@@ -284,11 +289,10 @@ class EntityTypeInfo(object):
         Type-dependent messages
     """
 
-    def __init__(self, site, coll, type_id, create_typedata=False):
+    def __init__(self, coll, type_id, create_typedata=False):
         """
         Set up type attribute values.
 
-        site            current site object
         coll            collection object in which type is used
         type_id         entity type id, which is a collection-defined value,
                         or one of a number of special site-wide built-in types.
@@ -301,7 +305,7 @@ class EntityTypeInfo(object):
         Attributes of type information object are:
 
         recordtype      type object describing the identified type
-        entityparent    Parent enbtity for entities of this type, or None if 
+        entityparent    Parent entity for entities of this type, or None if 
                         the type is not defined for the collection
         entityaltparent Alternative (site-wide) parent entity for built-in types, 
                         or None
@@ -311,15 +315,13 @@ class EntityTypeInfo(object):
 
         and other values as initialized here.
         """
-        #@@TODO: remove site param and access through coll
-        self.entitysite      = site
         self.entitycoll      = coll
         self.recordtype      = None
         self.entityparent    = None
         self.coll_id         = coll.get_id()
         self.type_id         = type_id
         self.permissions_map = None
-        if type_id == "_coll":
+        if type_id == layout.COLL_TYPEID: # "_coll"
             # NOTE: 
             #
             # This setup defaults to using site permissions for collection operations.
@@ -328,8 +330,8 @@ class EntityTypeInfo(object):
             #
             # (See use of attribute DisplayInfo.coll_perms.)
             #
-            self.recordtype      = site.site_data_collection().get_type(type_id)
-            self.entityparent    = site
+            self.recordtype      = coll.get_site().site_data_collection().get_type(type_id)
+            self.entityparent    = coll.get_site()
             self.entityaltparent = None
             self.entityclass     = Collection
             self.entitymessages  = COLL_MESSAGES
@@ -337,7 +339,7 @@ class EntityTypeInfo(object):
         elif type_id in TYPE_CLASS_MAP:
             self.recordtype      = coll.get_type(type_id)
             self.entityparent    = coll
-            self.entityaltparent = site
+            self.entityaltparent = coll.get_site()
             self.entityclass     = TYPE_CLASS_MAP[type_id]
             self.entitymessages  = TYPE_MESSAGE_MAP[type_id]
             if self.coll_id == layout.SITEDATA_ID:
@@ -345,8 +347,14 @@ class EntityTypeInfo(object):
             else:
                 self.permissions_map = TYPE_PERMISSIONS_MAP[type_id]
         else:
+            if not valid_id(type_id):
+                raise ValueError("EntityTypeInfo invalid type_id (%s)"%(type_id,))
             if RecordType.exists(coll, type_id, altscope="all"):
+                # log.info("@@ EntityTypeInfo: Type %s exists"%type_id)
                 self.recordtype     = coll.get_type(type_id)
+            else:
+                # log.info("@@ EntityTypeInfo: Type %s does not exist for collection %s"%(type_id,coll.get_id()))
+                pass
             if create_typedata and not RecordTypeData.exists(coll, type_id):
                 self.entityparent   = RecordTypeData.create(coll, type_id, {})
             else:
@@ -358,12 +366,10 @@ class EntityTypeInfo(object):
         if not self.recordtype:
             # .recordtype is used by views.displayinfo to locate the default
             # view and/or list id for examining records of a particular type.
-            #
             # Also used in entityedit for getting @type URI/CURIE values.
-            #
-            # Used in render_utils to get link to type record
-            if type_id not in get_built_in_type_ids():
-                log.warning("EntityTypeInfo.__init__: RecordType %s not found"%type_id)
+            # Used in bound_field to get link to type record
+            log.warning("EntityTypeInfo.__init__: RecordType %s not found"%type_id)
+            # log.info("".join(traceback.format_stack()))
             # raise ValueError("Trace")
         return
 
@@ -391,16 +397,53 @@ class EntityTypeInfo(object):
         """
         Return list of all type URIs for this type
         """
-        types = [self.get_type_uri()]
+        type_uris = None
+        type_uri  = self.get_type_uri()
+        if type_uri:
+            type_uris = [type_uri] + list(self.entitycoll.cache_get_supertype_uris(type_uri))
+        return type_uris
+
+    def get_default_view_id(self):
+        """
+        Returns the default view id for the current record type
+        """
+        view_id = None
         if self.recordtype:
-            supertypes = self.recordtype.get(ANNAL.CURIE.supertype_uris, None)
-            if supertypes:
-                for st in supertypes:
-                    # supertype_uris is list of objects { 'annal:supertype_uri': uri }
-                    t = st.get(ANNAL.CURIE.supertype_uri, None)
-                    if t:
-                        types.append(t)
-        return types
+            view_id = extract_entity_id(self.recordtype.get(ANNAL.CURIE.type_view, None))
+        else:
+            log.warning("EntityTypeInfo.get_default_view_id: no type data for %s"%(self.type_id))
+        return view_id or "Default_view"
+
+    # Entity-specific methods
+
+    def get_entity_permissions_map(self, entity_id=None):
+        """
+        Returns an entity-specific permission map that takes account of special 
+        access permissions applied to specific individual entities.
+
+        Thge permission maop is a map from an action name ("view", "new", etc) to 
+        a permission token that must be granted to a requester for the action 
+        to be allowed.
+
+        If `entity_id` is not specified, or is None, returns a default permission map 
+        applicable to entities of the current type in the absence of more specific 
+        permission requirements.
+        """
+        entity_perms_map = self.permissions_map
+        # Check for entity-specific permissions
+        #
+        # The logic here is currently ad-hoc, but in due course could be replaced
+        # by something more generic
+        # log.info(
+        #     "@@ get_entity_permissions_map: type_id %s, entity_id %s"%(self.type_id, entity_id)
+        #     )
+        if self.type_id == USER_ID:
+            # Relax view access requirements for default and unknown user id
+            # (Real users require admin rights to view)
+            if entity_id in [site_default_user_id, default_user_id, unknown_user_id]:
+                entity_perms_map = dict(entity_perms_map)
+                entity_perms_map["view"] = CONFIG_PERMISSIONS["view"]
+        return entity_perms_map
 
     def _new_entity(self, entity_id):
         """
@@ -424,13 +467,11 @@ class EntityTypeInfo(object):
         """
         Creates and returns an entity for the current type, with the supplied values.
         """
-        log.debug(
-            "create_entity id %s, parent %s, values %r"%
-            (entity_id, self.entityparent, entity_values)
-            )
+        log.debug("create_entity: id %s, parent id %s"%(entity_id, self.entityparent.get_id()))
+        # log.debug("create_entity: values %r"%(entity_values,))
         # Set type URI for entity; previous types are not carried forwards
         # Don't save entity URI if same as URL
-        if entity_values.get(ANNAL.CURIE.uri) == entity_values.get(ANNAL.CURIE.url):
+        if entity_values.get(ANNAL.CURIE.uri, "") == entity_values.get(ANNAL.CURIE.url, ""):
             entity_values.pop(ANNAL.CURIE.uri, None)
         entity_values['@type']          = self.get_all_type_uris() # NOTE: previous types not carried forward
         entity_values[ANNAL.CURIE.type] = self.get_type_uri()
@@ -455,10 +496,10 @@ class EntityTypeInfo(object):
 
         If `action` is "new" then a new entity is initialized (but not saved).
         """
-        log.debug(
-            "get_entity id %s, parent %s, altparent %s, action %s"%
-            (entity_id, self.entityparent, self.entityaltparent, action)
-            )
+        # log.debug(
+        #     "EntityTypeInfo.get_entity id %s, parent %s, altparent %s, action %s"%
+        #     (entity_id, self.entityparent, self.entityaltparent, action)
+        #     )
         entity = None
         entity_id = extract_entity_id(entity_id)
         if valid_id(entity_id):
@@ -471,7 +512,7 @@ class EntityTypeInfo(object):
                     self.entityparent, entity_id, altscope="all"
                     )
             else:
-                log.info(
+                log.debug(
                     "EntityTypeInfo.get_entity %s/%s at %s not found"%
                     (self.type_id, entity_id, self.entityparent._entitydir)
                     )
@@ -489,57 +530,46 @@ class EntityTypeInfo(object):
             entity = self.get_entity(entity_id, action="new")
         return entity
 
-    # @@TODO: rename inferred -> implied
-    def get_entity_inferred_values(self, entity):
+    def get_copy_entity(self, entity_id, copy_entity_id):
         """
-        Adds inferrable values to the supplied entity value (e.g. aliases),
+        Read or create an entity with the indicated entity_id.
+
+        If the identified entity does not already exist, a new entity is created 
+        but not (yet) saved.
+
+        The newly created entity is a copy of 'copy_entity_id'.        
+        """
+        entity_id = extract_entity_id(entity_id)
+        entity    = self.get_entity(entity_id)
+        if entity is None:
+            entity = self._new_entity(entity_id)
+            entity.set_values(
+                self.get_initial_entity_values(entity_id, copy_entity_id=copy_entity_id)
+                )
+        return entity
+
+    def get_entity_implied_values(self, entity):
+        """
+        Adds implied values to the supplied entity value (e.g. aliases),
         and returns a new value with the additional values
 
-        Inferred values are determined by the type of the entity, and if type
-        information is not present this fundtion generates a failure.
+        Implied values are determined by the type of the entity, and if type
+        information is not present this function generates a failure.
         """
         if not self.recordtype: 
             raise AssertionError(
-                "add_inferred_values_to_entity called with no type information available.  "+
+                "EntityTypeInfo.get_entity_implied_values called with no type information available.  "+
                 "entity_id %s/%s, type_id %s"%(entity.get_type_id(), entity.get_id(), self.type_id)
                 )
-        inferred_entity = entity
-        if inferred_entity and ANNAL.CURIE.field_aliases in self.recordtype:
-            inferred_entity = copy.deepcopy(entity)
+        implied_entity = entity
+        if implied_entity and ANNAL.CURIE.field_aliases in self.recordtype:
+            implied_entity = copy.deepcopy(entity)
             for alias in self.recordtype[ANNAL.CURIE.field_aliases]:
                 tgt = alias[ANNAL.CURIE.alias_target]
                 src = alias[ANNAL.CURIE.alias_source]
-                if inferred_entity.get(tgt, None) in [None, ""]:
-                    inferred_entity[tgt] = inferred_entity.get(src, "")
-        return inferred_entity
-
-    def copy_data_files(self, new_entity_id, old_typeinfo, old_entity_id):
-        """
-        Copy associated data files from specified entity to new.
-        """
-        if self.entity_exists(new_entity_id):
-            if old_typeinfo.entity_exists(old_entity_id):
-                new_entity = self._new_entity(new_entity_id)
-                old_entity = old_typeinfo._new_entity(old_entity_id)
-                for p, f in old_entity._entity_files():
-                    if not new_entity._exists_file(f):
-                        p_new = new_entity._copy_file(p, f)
-                        if not p_new:
-                            log.warning(
-                                "EntityTypeInfo.copy_data_files: error copying file %s from %s to %s"%
-                                (f, old_entity_id, new_entity_id)
-                                )
-            else:
-                log.warning(
-                    "EntityTypeInfo.copy_data_files: source entity not found %s/%s"%
-                    (old_typeinfo.type_id, old_entity_id)
-                    )
-        else:
-            log.warning(
-                "EntityTypeInfo.copy_data_files: target entity not found %s/%s"
-                %(self.type_id, entity_id)
-                )
-        return
+                if implied_entity.get(tgt, None) in [None, ""]:
+                    implied_entity[tgt] = implied_entity.get(src, "")
+        return implied_entity
 
     def rename_entity(self, new_entity_id, old_typeinfo, old_entity_id):
         """
@@ -582,18 +612,17 @@ class EntityTypeInfo(object):
         Iterate over entities in collection with current type.
         """
         if (not user_perms or 
-            self.permissions_map['list'] in user_perms[ANNAL.CURIE.user_permissions]):
-            if self.entityparent:
+            self.permissions_map['list'] in user_perms[ANNAL.CURIE.user_permission]):
+            if not self.entityparent:
+                log.warning("EntityTypeInfo.enum_entities: missing entityparent; type_id %s"%(self.type_id))
+            else:
                 for eid in self.entityparent.child_entity_ids(
                         self.entityclass, 
                         altscope=altscope):
                     yield self.get_entity(eid)
-            else:
-                log.warning("EntityTypeInfo.enum_entities: missing entityparent; type_id %s"%(self.type_id))
         return
 
-    # @@TODO: rename inferred -> implied
-    def enum_entities_with_inferred_values(self, user_perms=None, altscope=None):
+    def enum_entities_with_implied_values(self, user_perms=None, altscope=None):
         """
         Iterate over entities in collection with current type.
         Returns entities with alias and inferred fields instantiated.
@@ -603,28 +632,41 @@ class EntityTypeInfo(object):
         """
         #@@
         # log.info(
-        #     "enum_entities_with_inferred_values: parent %s, altscope %s"%
+        #     "@@ EntityTypeInfo.enum_entities_with_implied_values: parent %s, altscope %s"%
         #     (self.entityparent.get_id(), altscope)
         #     )
         #@@
         if (not user_perms or 
-            self.permissions_map['list'] in user_perms[ANNAL.CURIE.user_permissions]):
-            if self.entityparent:
+            self.permissions_map['list'] in user_perms[ANNAL.CURIE.user_permission]):
+            if not self.entityparent:
+                log.warning(
+                    "EntityTypeInfo.enum_entities_with_implied_values: missing entityparent; type_id %s"%
+                    (self.type_id)
+                    )
+            elif not self.recordtype:
+                log.warning(
+                    "EntityTypeInfo.enum_entities_with_implied_values: missing recordtype; type_id %s"%
+                    (self.type_id)
+                    )
+                # No record type info: return base entity without implied values
+                for eid in self.entityparent.child_entity_ids(
+                        self.entityclass, 
+                        altscope=altscope):
+                    yield self.get_entity(eid)
+            else:
+                #@@
+                # log.info(
+                #     "@@ enum_entities_with_implied_values: parent %s, altscope %s"%
+                #     (self.entityparent.get_id(), altscope)
+                #     )
                 #@@
                 for eid in self.entityparent.child_entity_ids(
                         self.entityclass, 
                         altscope=altscope):
-                    yield self.get_entity_inferred_values(self.get_entity(eid))
-                #@@
-                # for eid in self.entityparent._children(self.entityclass, altscope=altscope):
-                #     if self.entityclass.exists(self.entityparent, eid, altscope=altscope):
-                #         yield self.get_entity_inferred_values(self.get_entity(eid))
-                #@@
-            else:
-                log.warning("EntityTypeInfo.enum_entities: missing entityparent; type_id %s"%(self.type_id))
+                    yield self.get_entity_implied_values(self.get_entity(eid))
         return
 
-    def get_initial_entity_values(self, entity_id):
+    def get_initial_entity_values(self, entity_id, copy_entity_id=layout.INITIAL_VALUES_ID):
         """
         Returns an initial value dictionary for the indicated entity.
 
@@ -637,26 +679,17 @@ class EntityTypeInfo(object):
             , RDFS.CURIE.label:     ""
             , RDFS.CURIE.comment:   ""
             })
-        init_entity = self.get_entity("_initial_values")
+        init_entity = self.get_entity(copy_entity_id)
         if init_entity:
             values = init_entity.get_values()
-            values.pop("@id", None)
-            values.pop(ANNAL.CURIE.id,  None)
-            values.pop(ANNAL.CURIE.url, None)
-        values[ANNAL.CURIE.id] = entity_id
+            values.pop("@id",              None)
+            values.pop(ANNAL.CURIE.id,     None)
+            values.pop(ANNAL.CURIE.url,    None)
+        values[ANNAL.CURIE.id]      = entity_id
+        values[RDFS.CURIE.label]    = ""
+        values[RDFS.CURIE.comment]  = ""
         return values
 
-    def get_default_view_id(self):
-        """
-        Returns the default view id for the current record type
-        """
-        view_id = None
-        if self.recordtype:
-            view_id = extract_entity_id(self.recordtype.get(ANNAL.CURIE.type_view, None))
-        else:
-            log.warning("EntityTypeInfo.get_default_view_id: no type data for %s"%(self.type_id))
-        return view_id or "Default_view"
-
     def get_fileobj(self, entity_id, name, typeuri, mimetype, mode):
         """
         Returns a file object to access a file stored with the named entity 
@@ -673,4 +706,13 @@ class EntityTypeInfo(object):
             log.warning("EntityTypeInfo.get_fileobj: missing entityparent; type_id %s"%(self.type_id))
         return fileobj
 
+    def get_ancestor_id(self, entity):
+        """
+        Returns the ancestor collection id for the supplied entity
+        (which is assumed to be of the current type).
+        """
+        if self.type_id == COLL_ID:
+            return layout.SITEDATA_ID
+        return entity._parent._ancestorid
+
 # End.
