@@ -25,15 +25,15 @@ from annalist.models.entitydata import EntityData
 
 class RecordEnumBase(EntityData):
 
+    _entitytypeid   = "_enum_base_id"
     _entitytype     = ANNAL.CURIE.Enum
-    _entitytypeid   = "_enum"
+    _entityroot     = layout.COLL_ENUM_PATH
     _entityview     = layout.COLL_ENUM_VIEW
-    _entitypath     = layout.COLL_ENUM_PATH
     _entityfile     = layout.ENUM_META_FILE
     _entityprov     = layout.ENUM_PROV_FILE
-    _entityref      = layout.META_ENUM_REF
 
     def __init__(self, parent, entity_id, type_id):
+        # print("@@ RecordEnumBase.__init__ parentid %s, entityid %s"%(parent.get_id(), entity_id))
         self._entitytypeid = type_id
         super(RecordEnumBase, self).__init__(parent, entity_id)
         return
@@ -49,12 +49,14 @@ def RecordEnumFactory(name, type_id):
     Returns a dynamically-subclassed instance of RecordEnumBase using the supplied 
     class name and type_id for all created instances.
     """
+    # print("@@ RecordEnumFactory name %s, type_id %s"%(name, type_id))
     def RecordEnumInit(self, parent, entity_id):
+        # print("@@ RecordEnumInit parentid %s, entityid %s"%(parent.get_id(), entity_id))
         super(RecordEnumBase, self).__init__(parent, entity_id)
         return
     return type(name, (RecordEnumBase,), 
         { '_entitytypeid':  type_id
-        , '_entitypath':    layout.COLL_ENUM_PATH%{'id': "%(id)s", 'type_id': type_id}
+        , '_entityroot':    layout.COLL_ENUM_PATH%{'id': "%(id)s", 'type_id': type_id}
         , '__init__': RecordEnumInit}
         )
 
