@@ -1,5 +1,5 @@
 """
-Analist site-related facilities
+Annalist site-related facilities
 """
 
 __author__      = "Graham Klyne (GK@ACM.ORG)"
@@ -26,9 +26,11 @@ from django.core.urlresolvers       import resolve, reverse
 import annalist
 from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist.exceptions            import Annalist_Error, EntityNotFound_Error
-from annalist.util                  import valid_id, extract_entity_id, replacetree, updatetree
 from annalist                       import layout
 from annalist                       import message
+from annalist.util                  import (
+    valid_id, extract_entity_id, replacetree, updatetree
+    )
 
 from annalist.models.annalistuser   import AnnalistUser
 from annalist.models.entityroot     import EntityRoot
@@ -36,7 +38,6 @@ from annalist.models.sitedata       import SiteData
 from annalist.models.collection     import Collection
 from annalist.models.recordvocab    import RecordVocab
 from annalist.models.recordview     import RecordView
-from annalist.models.recordgroup    import RecordGroup
 from annalist.models.recordfield    import RecordField
 from annalist.models.rendertypeinfo import (
     is_render_type_literal,
@@ -49,7 +50,8 @@ from annalist.models.rendertypeinfo import (
 class Site(EntityRoot):
 
     _entitytype     = ANNAL.CURIE.Site
-    _entitytypeid   = "_site"
+    _entitytypeid   = layout.SITE_TYPEID
+    _entitybase     = layout.SITE_META_PATH
     _entityfile     = layout.SITE_META_FILE
     _entityref      = layout.META_SITE_REF
 
@@ -81,7 +83,7 @@ class Site(EntityRoot):
         """
         Iterates over candidate child identifiers that are possible instances of an 
         indicated class.  The supplied class is used to determine a subdirectory to 
-        be scanned.  As a spoecial case, the children are iterated only in a special 
+        be scanned.  As a special case, the children are iterated only in a special 
         `altscope` called "site".
 
         cls         is a subclass of Entity indicating the type of children to
@@ -227,19 +229,18 @@ class Site(EntityRoot):
                 returns a context dictionary for the field thus described.
         """
         # Build context data
-        #@@ context = self.get_site_jsonld_context()
         context = self.site_data_collection().get_coll_jsonld_context()
         # Assemble and write out context description
         datetime_now = datetime.datetime.today().replace(microsecond=0)
         datetime_str = datetime_now.isoformat(' ')
         with self._metaobj(
-            layout.SITEDATA_CONTEXT_PATH, layout.COLL_CONTEXT_FILE, "wt"
+            layout.SITEDATA_CONTEXT_PATH, layout.SITE_CONTEXT_FILE, "wt"
             ) as context_io:
             json.dump(
                 { "_comment": "Generated by generate_site_jsonld_context on %s"%datetime_str
                 , "@context": context 
                 }, 
-                context_io, indent=2, separators=(',', ': ')
+                context_io, indent=2, separators=(',', ': '), sort_keys=True
                 )
         return
 
@@ -248,13 +249,16 @@ class Site(EntityRoot):
     # These methods are used by test_createsitedata and annalist-manager to initialize
     # or update Annalist site data.  Tests are run using data copied from sampledata/init
     # to sampledata/data, allowing for additional test fixture files to be included.
-
+    #
     @staticmethod
-    def create_empty_site_data(site_base_uri, site_base_dir, 
+    def create_site_metadata(site_base_uri, site_base_dir, 
         label=None, description=None):
         """
-        Create empty directory structure for a new site, and returns the
-        Site object.
+        Create new site metadata record for a new site, and 
+        return the Site object.
+
+        This resets the site label and description that 
+        may have been updated by a sirte administrator.
         """
         datetime_now = datetime.datetime.today().replace(microsecond=0)
         if label is None:
@@ -262,14 +266,14 @@ class Site(EntityRoot):
         if description is None:
             description = "Annalist site metadata and site-wide values."
         annal_comment = (
-            "Initialized by annalist.models.site.create_empty_site_data at "+
+            "Initialized by annalist.models.site.create_site_metadata at "+
             datetime_now.isoformat(' ')
             )
         site = Site(site_base_uri, site_base_dir)
         sitedata_values = (
             { RDFS.CURIE.label:             label
             , RDFS.CURIE.comment:           description
-            , ANNAL.CURIE.comment:          annal_comment                                    
+            , ANNAL.CURIE.meta_comment:     annal_comment                                    
             , ANNAL.CURIE.software_version: annalist.__version_data__
             })
         sitedata = SiteData.create_sitedata(site, sitedata_values)
@@ -290,63 +294,76 @@ class Site(EntityRoot):
             """\n"""+
             """    %(site_base_dir)s\n"""+
             """      c/\n"""+
-            """        _annalist_site/\n"""+
-            """          _annalist_collection/         (site-wide definitions)\n"""+
+            """        _annalist_site/                 (site-wide definitions)\n"""+
+            """          d/\n"""+
             """            coll_meta.jsonld            (site metadata)\n"""+
             """            coll_context.jsonld         (JSON-LD context for site definitions)\n"""+
-            """            enums/\n"""+
-            """              (enumerated type values)\n"""+
+            """            %(enum_field_placement_dir)s/\n"""+
+            """              (field-placement-value)/\n"""+
+            """                enum_meta.jsonld\n"""+
             """               :\n"""+
-            """            fields/\n"""+
+            """            %(enum_list_type_dir)s/\n"""+
+            """              (list-type-id)/\n"""+
+            """                enum_meta.jsonld\n"""+
+            """               :\n"""+
+            """            %(enum_render_type_dir)s/\n"""+
+            """              (render-type-id)/\n"""+
+            """                enum_meta.jsonld\n"""+
+            """               :\n"""+
+            """            %(enum_value_type_dir)s/\n"""+
+            """              (value-type-id)/\n"""+
+            """                enum_meta.jsonld\n"""+
+            """               :\n"""+
+            """            %(enum_value_mode_dir)s/\n"""+
+            """              (value-mode-id)/\n"""+
+            """                enum_meta.jsonld\n"""+
+            """               :\n"""+
+            """            %(field_dir)s/\n"""+
             """              (view-field definitions)\n"""+
             """               :\n"""+
-            """            groups/\n"""+
-            """              (field group definitions)\n"""+
-            """               :\n"""+
-            """            lists/\n"""+
+            """            %(list_dir)s/\n"""+
             """              (entity list definitions)\n"""+
             """               :\n"""+
-            """            types/\n"""+
+            """            %(type_dir)s/\n"""+
             """              (type definitions)\n"""+
             """               :\n"""+
-            """            users/\n"""+
+            """            %(user_dir)s/\n"""+
             """              (user permissions)\n"""+
             """               :\n"""+
-            """            views/\n"""+
+            """            %(view_dir)s/\n"""+
             """              (entity view definitions)\n"""+
             """               :\n"""+
-            """            vocabs/\n"""+
+            """            %(vocab_dir)s/\n"""+
             """              (vocabulary namespace definitions)\n"""+
             """               :\n"""+
             """        (collection-id)/                (user-created data collection)\n"""+
-            """          _annalist_collection/         (collection definitions)\n"""+
+            """          d/\n"""+
             """            coll_meta.jsonld            (collection metadata)\n"""+
-            """            coll_context.jsonld         (JSON-LD context for collection definitions)\n"""+
-            """            types/                      (collection type definitions\n"""+
+            """            coll_context.jsonld         (JSON-LD context for collection data)\n"""+
+            """            %(type_dir)s/                      (collection type definitions)\n"""+
             """              (type-id)/\n"""+
             """                type_meta.jsonld\n"""+
             """               :\n"""+
-            """            lists/                      (collection list definitions\n"""+
+            """            %(list_dir)s/                      (collection list definitions)\n"""+
             """              (list-id)/\n"""+
             """                list_meta.jsonld\n"""+
             """               :\n"""+
-            """            views/                      (collection view definitions\n"""+
+            """            %(view_dir)s/                      (collection view definitions)\n"""+
             """              (view-id)/\n"""+
             """                view_meta.jsonld\n"""+
             """               :\n"""+
-            """            fields/                     (collection field definitions\n"""+
+            """            %(field_dir)s/                     (collection field definitions)\n"""+
             """              (field-id)/\n"""+
             """                field_meta.jsonld\n"""+
             """               :\n"""+
-            """            groups/                     (collection field group definitions\n"""+
+            """            %(group_dir)s/                     (collection field group definitions)\n"""+
             """              (group-id)/\n"""+
             """                group_meta.jsonld\n"""+
             """               :\n"""+
-            """            users/                      (collection user permissions\n"""+
+            """            %(user_dir)s/                      (collection user permissions)\n"""+
             """              (user-id)/\n"""+
             """                user_meta.jsonld\n"""+
             """               :\n"""+
-            """          d/\n"""+
             """            (type-id)/                  (contains all entity data for identified type)\n"""+
             """              (entity-id)/              (contains data for identified type/entity)\n"""+
             """                entity_data.jsonld      (entity data)\n"""+
@@ -363,10 +380,22 @@ class Site(EntityRoot):
             """for Annalist %(version)s at %(datetime)s\n"""+
             """\n"""+
             """\n""")%
-                { 'site_base_dir': site._entitydir
-                , 'site_base_uri': site._entityurl
-                , 'datetime':      datetime_now.isoformat(' ')
-                , 'version':       annalist.__version__
+                { 'site_base_dir':              site._entitydir
+                , 'site_base_uri':              site._entityurl
+                , 'datetime':                   datetime_now.isoformat(' ')
+                , 'version':                    annalist.__version__
+                , 'enum_field_placement_dir':   layout.ENUM_FIELD_PLACEMENT_DIR
+                , 'enum_list_type_dir':         layout.ENUM_LIST_TYPE_DIR
+                , 'enum_render_type_dir':       layout.ENUM_RENDER_TYPE_DIR
+                , 'enum_value_type_dir':        layout.ENUM_VALUE_TYPE_DIR
+                , 'enum_value_mode_dir':        layout.ENUM_VALUE_MODE_DIR
+                , 'field_dir':                  layout.FIELD_DIR
+                , 'group_dir':                  layout.GROUP_DIR
+                , 'list_dir':                   layout.LIST_DIR
+                , 'type_dir':                   layout.TYPE_DIR
+                , 'user_dir':                   layout.USER_DIR
+                , 'view_dir':                   layout.VIEW_DIR
+                , 'vocab_dir':                  layout.VOCAB_DIR
                 }
             )
         with site._fileobj("README", ANNAL.CURIE.Richtext, "text/markdown", "wt") as readme:
@@ -392,7 +421,7 @@ class Site(EntityRoot):
         coll_values = (
             { RDFS.CURIE.label:             label
             , RDFS.CURIE.comment:           description
-            , ANNAL.CURIE.comment:          annal_comment                                    
+            , ANNAL.CURIE.meta_comment:     annal_comment                                    
             , ANNAL.CURIE.software_version: annalist.__version_data__
             })
         coll = Collection.create(site, coll_id, coll_values)
@@ -407,19 +436,21 @@ class Site(EntityRoot):
         site_data_tgt, site_data_file = sitedata._dir_path()
         s = os.path.join(site_data_src, sdir)
         d = os.path.join(site_data_tgt, sdir)
-        replacetree(s, d)
+        if os.path.isdir(s):
+            replacetree(s, d)
         return
 
     @staticmethod
     def update_site_data_dir(sitedata, sdir, site_data_src):
         """
         Update indicated sitedata directory data from source: 
-        old data for the directory thgat ios not updated is left as-is.
+        old data for the directory that is not updated is left as-is.
         """
         site_data_tgt, site_data_file = sitedata._dir_path()
         s = os.path.join(site_data_src, sdir)
         d = os.path.join(site_data_tgt, sdir)
-        updatetree(s, d)
+        if os.path.isdir(s):
+            updatetree(s, d)
         return
 
     @staticmethod
@@ -432,23 +463,22 @@ class Site(EntityRoot):
         Creates a README.md file in the site base directory, and creates a
         collection _annalist_site containing built-in types, views, etc.
         """
-        site = Site.create_empty_site_data(
+        site = Site.create_site_metadata(
             site_base_uri, site_base_dir, label=label, description=description
             )
         sitedata = site.site_data_collection()
         Site.create_site_readme(site)
         site_data_tgt, site_data_file = sitedata._dir_path()
         log.info("Copy Annalist site data from %s to %s"%(site_data_src, site_data_tgt))
-        for sdir in ("types", "lists", "views", "groups", "fields", "vocabs", "users", "enums"):
+        for sdir in layout.COLL_DIRS:
             log.info("- %s -> %s"%(sdir, site_data_tgt))
             Site.replace_site_data_dir(sitedata, sdir, site_data_src)
         sitedata.generate_coll_jsonld_context()
         return site
 
     @staticmethod
-    def initialize_bib_data(
-        site, bib_data_src, 
-        label=None, description=None):
+    def initialize_bib_data(site, bib_data_src ):
+        # label=None, description=None
         """
         Initializes bibliography definitions data for a new site for testing.
         """
@@ -460,10 +490,29 @@ class Site(EntityRoot):
             )
         bib_data_tgt, bib_data_file = bibdatacoll._dir_path()
         log.info("Copy Annalist bibliographic definitions data from %s to %s"%(bib_data_src, bib_data_tgt))
-        for sdir in ("types", "lists", "views", "groups", "fields", "enums"):
+        for sdir in layout.DATA_DIRS:
             log.info("- %s -> %s"%(sdir, bib_data_tgt))
             Site.replace_site_data_dir(bibdatacoll, sdir, bib_data_src)
         bibdatacoll.generate_coll_jsonld_context()
         return bibdatacoll
 
+    # @staticmethod
+    # def initialize_named_coll(
+    #     site, coll_id, coll_data_src,
+    #     label=None, description=None):
+    #     """
+    #     Initializes bibliography definitions data for a new site for testing.
+    #     """
+    #     namedcoll = site.create_empty_coll_data(
+    #         site, coll_id, 
+    #         label=label, description=description
+    #         )
+    #     coll_data_tgt, coll_data_file = namedcoll._dir_path()
+    #     log.info("Copy Annalist %s definitions data from %s to %s"%(coll_id, coll_data_src, coll_data_tgt))
+    #     for sdir in layout.DATA_DIRS:
+    #         log.info("- %s -> %s"%(sdir, coll_data_tgt))
+    #         Site.replace_site_data_dir(namedcoll, sdir, coll_data_src)
+    #     namedcoll.generate_coll_jsonld_context()
+    #     return namedcoll
+
 # End.
