@@ -22,16 +22,21 @@ from annalist.identifiers       import RDF, RDFS, ANNAL
 from annalist                   import layout
 from annalist                   import message
 
+from annalist.views.uri_builder import uri_base, uri_params, uri_with_params
+
 from annalist.views.fields.render_placement import (
     get_placement_classes
     )
 
 from entity_testutils import (
     collection_dir, 
+    entitydata_list_url_query,
     site_view_url,
+    collection_view_url,
     collection_edit_url,
     collection_entity_view_url,
-    site_title
+    site_title,
+    context_field_row
     )
 from tests import (
     TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
@@ -74,7 +79,7 @@ def entity_resource_url(
     resource_ref=layout.ENTITY_DATA_FILE
     ):
     """
-    URI for entity data; also view using default entity view
+    URI for entity resource data
     """
     base = entity_url(coll_id, type_id, entity_id)
     return urlparse.urljoin(base, resource_ref)
@@ -82,36 +87,53 @@ def entity_resource_url(
 def entitydata_edit_url(action=None, coll_id="testcoll", type_id=None, entity_id=None, view_id="Default_view"):
     viewname = ( 
         'AnnalistEntityNewView'             if action == "new" else
-        'AnnalistEntityEditView'
+        'AnnalistEntityEditView'            if action is not None else
+        'AnnalistEntityDataView'
         )
-    kwargs = {'action': action, 'coll_id': coll_id, 'view_id': view_id}
+    kwargs = {'coll_id': coll_id, 'view_id': view_id}
+    if action:
+        kwargs.update({'action': action})
     if type_id:
         kwargs.update({'type_id': type_id})
     if entity_id:
         kwargs.update({'entity_id': entity_id})
     return reverse(viewname, kwargs=kwargs)
 
-def entitydata_list_all_url(coll_id="testcoll", list_id=None, scope=None):
+def entitydata_list_all_url(
+        coll_id="testcoll", list_id=None, 
+        scope=None, continuation_url=None, query_params=None):
     if list_id:
         viewname = "AnnalistEntityGenericList"
         kwargs   = {'list_id': list_id, 'coll_id': coll_id}
     else:
         viewname = "AnnalistEntityDefaultListAll"
         kwargs   = {'coll_id': coll_id}
-    if scope is not None:
-        kwargs['scope'] = scope
-    return reverse(viewname, kwargs=kwargs)
+    return entitydata_list_url_query(viewname, kwargs, 
+        query_params,
+        { 'scope': scope
+        , 'continuation_url': continuation_url
+        })
+    # if scope is not None:
+    #     kwargs['scope'] = scope
+    # return reverse(viewname, kwargs=kwargs)
 
-def entitydata_list_type_url(coll_id="testcoll", type_id="testtype", list_id=None, scope=None):
+def entitydata_list_type_url(
+        coll_id="testcoll", type_id="testtype", list_id=None, 
+        scope=None, continuation_url=None, query_params=None):
     if list_id:
         viewname = "AnnalistEntityGenericList"
         kwargs   = {'list_id': list_id, 'coll_id': coll_id, 'type_id': type_id}
     else:
         viewname = "AnnalistEntityDefaultListType"
         kwargs   = {'coll_id': coll_id, 'type_id': type_id}
-    if scope is not None:
-        kwargs['scope'] = scope
-    return reverse(viewname, kwargs=kwargs)
+    return entitydata_list_url_query(viewname, kwargs, 
+        query_params,
+        { 'scope': scope
+        , 'continuation_url': continuation_url
+        })
+    # if scope is not None:
+    #     kwargs['scope'] = scope
+    # return reverse(viewname, kwargs=kwargs)
 
 def entitydata_delete_confirm_url(coll_id="testcoll", type_id="testtype"):
     kwargs = {'coll_id': coll_id, 'type_id': type_id}
@@ -165,13 +187,16 @@ def entitydata_load_keys(entity_uri=False):
 
 def entitydata_create_values(
         entity_id, update="Entity", coll_id="testcoll", type_id="testtype", 
-        entity_uri=None, type_uri=None, hosturi=TestHostUri,
+        entity_uri=None, type_uri=None, supertype_uris=None,
+        hosturi=TestHostUri,
         extra_fields=None):
     """
     Data used when creating entity test data
     """
     if type_uri is not None:
-        types = [type_uri, type_uri+"/super1", type_uri+"/super2", entitydata_type(type_id)]
+        types = [type_uri, entitydata_type(type_id)]
+        if supertype_uris is not None:
+            types = types + supertype_uris
     else:
         types = [entity_url(coll_id, "_type", type_id), entitydata_type(type_id)]
     # log.info('entitydata_create_values: types %r'%(types,)) 
@@ -204,18 +229,19 @@ def entitydata_values(
         entity_id, update="Entity", 
         coll_id="testcoll", type_id="testtype", 
         entity_uri=None,
-        type_uri=None, hosturi=TestHostUri
+        type_uri=None, supertype_uris=None, 
+        hosturi=TestHostUri
         ):
     # type_uri = entity_url(coll_id, "_type", type_id)
     dataurl = entity_url(coll_id, type_id, entity_id)
     d = entitydata_create_values(
         entity_id, update=update, coll_id=coll_id, type_id=type_id, 
-        entity_uri=entity_uri, type_uri=type_uri, hosturi=hosturi
+        entity_uri=entity_uri, type_uri=type_uri, supertype_uris=supertype_uris,
+        hosturi=hosturi
         ).copy() #@@ copy needed here?
     d.update(
-        { '@id':            './'
-        # , '@base':          "../.."
-        , '@context':       ["../../coll_context.jsonld"]
+        { '@id':            "%s/%s"%(type_id, entity_id)
+        , '@context':       [{"@base": "../../"}, "../../coll_context.jsonld"]
         , 'annal:id':       entity_id
         , 'annal:type_id':  type_id
         , 'annal:url':      dataurl
@@ -224,63 +250,71 @@ def entitydata_values(
     return d
 
 def entitydata_context_data(
-        entity_id=None, orig_id=None, type_id="testtype", type_ids=[],
-        action=None, update="Entity"
+        entity_id=None, orig_id=None, coll_id="testcoll", type_id="testtype", type_ids=[],
+        action=None, update="Entity", view_label="Default record view"
     ):
+    if entity_id:
+        entity_label = '%s %s/%s/%s'%(update, coll_id, type_id, entity_id)
+        entity_descr = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
+    else:
+        entity_label = '%s data ... (%s/%s)'%(update, coll_id, type_id)
+        entity_descr = '%s description ... (%s/%s)'%(update, coll_id, type_id)
     context_dict = (
-        { 'title':              "Collection testcoll"
+        { 'title':              "%s - %s - Collection %s"%(entity_label, view_label, coll_id)
+        , 'heading':            view_label
         , 'coll_id':            'testcoll'
         , 'type_id':            'testtype'
         , 'orig_id':            'orig_entity_id'
         , 'fields':
-          [ { 'field_label':        'Id'
-            , 'field_id':           'Entity_id'
-            , 'field_name':         'entity_id'
-            , 'field_render_type':  'EntityId'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  "annal:Slug"
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_label':        'Type'
-            , 'field_id':           'Entity_type'
-            , 'field_name':         'entity_type'
-            , 'field_render_type':  'EntityTypeId'
-            , 'field_target_type':  "annal:Slug"
-            , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
-            , 'field_value_mode':   'Value_direct'
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_label':        'Label'
-            , 'field_id':           'Entity_label'
-            , 'field_name':         'Entity_label'
-            , 'field_render_type':  'Text'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  "annal:Text"
-            , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
-          , { 'field_label':        'Comment'
-            , 'field_id':           'Entity_comment'
-            , 'field_name':         'Entity_comment'
-            , 'field_render_type':  'Markdown'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  "annal:Richtext"
-            , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
+          [ context_field_row(
+              { 'field_label':        'Id'
+              , 'field_id':           'Entity_id'
+              , 'field_name':         'entity_id'
+              , 'field_render_type':  'EntityId'
+              , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   "annal:EntityRef"
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              },
+              { 'field_label':        'Type'
+              , 'field_id':           'Entity_type'
+              , 'field_name':         'entity_type'
+              , 'field_render_type':  'EntityTypeId'
+              , 'field_value_type':   "annal:EntityRef"
+              , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
+              , 'field_value_mode':   'Value_direct'
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_label':        'Label'
+              , 'field_id':           'Entity_label'
+              , 'field_name':         'Entity_label'
+              , 'field_render_type':  'Text'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   "annal:Text"
+              , 'field_value':        entity_label
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_label':        'Comment'
+              , 'field_id':           'Entity_comment'
+              , 'field_name':         'Entity_comment'
+              , 'field_render_type':  'Markdown'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   "annal:Richtext"
+              , 'field_value':        entity_descr
+              , 'options':            []
+              })
           ]
         , 'continuation_url':   entitydata_list_type_url("testcoll", type_id)
         })
     if entity_id:
-        context_dict['fields'][0]['field_value'] = entity_id
-        context_dict['fields'][1]['field_value'] = "_type/"+type_id if valid_id(type_id) else None
-        context_dict['fields'][2]['field_value'] = '%s testcoll/testtype/%s'%(update,entity_id)
-        context_dict['fields'][3]['field_value'] = '%s coll testcoll, type testtype, entity %s'%(update,entity_id)
+        context_dict['fields'][0]['row_field_descs'][0]['field_value'] = entity_id
+        context_dict['fields'][0]['row_field_descs'][1]['field_value'] = "_type/"+type_id if valid_id(type_id) else None
         context_dict['orig_id']     = entity_id
     if orig_id:
         context_dict['orig_id']     = orig_id
@@ -317,16 +351,16 @@ def entitydata_context_add_field(
         , 'field_label':        field_label
         , 'field_render_type':  field_render_type
         , 'field_value_mode':   field_value_mode
-        , 'field_target_type':  field_value_type
+        , 'field_value_type':   field_value_type
         , 'field_placement':    get_placement_classes(field_placement)
         , 'options':            field_options
         })
     return context_dict
 
 def entitydata_form_data(
-        entity_id=None, orig_id=None, 
+        entity_id="", orig_id=None, 
         type_id="testtype", orig_type=None,
-        coll_id="testcoll", 
+        coll_id="testcoll", orig_coll=None,
         action=None, cancel=None, close=None, edit=None, copy=None, 
         update="Entity"
         ):
@@ -336,6 +370,8 @@ def entitydata_form_data(
         , 'orig_id':            'orig_entity_id'
         , 'continuation_url':   entitydata_list_type_url(coll_id, orig_type or type_id)
         })
+    if entity_id is not None:
+        form_data_dict['entity_id']         = entity_id
     if entity_id:
         form_data_dict['entity_id']         = entity_id
         form_data_dict['entity_type']       = "_type/"+type_id
@@ -343,10 +379,13 @@ def entitydata_form_data(
         form_data_dict['Entity_comment']    = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
         form_data_dict['orig_id']           = entity_id
         form_data_dict['orig_type']         = type_id
+        form_data_dict['orig_coll']         = coll_id
     if orig_id:
         form_data_dict['orig_id']           = orig_id
     if orig_type:
         form_data_dict['orig_type']         = orig_type
+    if orig_coll:
+        form_data_dict['orig_coll']         = orig_coll
     if action:
         form_data_dict['action']            = action
     if cancel:
@@ -402,75 +441,13 @@ def entitydata_delete_confirm_form_data(entity_id=None, search=None):
 #
 #   -----------------------------------------------------------------------------
 
-def entitydata_default_view_context_data(
-        entity_id=None, orig_id=None, type_id="testtype", type_ids=[],
-        action=None, update="Entity"
-    ):
-    context_dict = (
-        { 'title':              site_title()
-        , 'coll_id':            'testcoll'
-        , 'type_id':            'testtype'
-        , 'orig_id':            'orig_entity_id'
-        , 'fields':
-          [ { 'field_label':        'Id'
-            , 'field_name':         'entity_id'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            , 'field_id':           'Entity_id'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Slug'
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_label':        'Type'
-            , 'field_name':         'entity_type'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
-            , 'field_id':           'Entity_type'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Text'
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_label':        'Label'
-            , 'field_name':         'Entity_label'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_id':           'Entity_label'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Text'
-            , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
-          , { 'field_label':        'Comment'
-            , 'field_name':         'Entity_comment'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_id':           'Entity_comment'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Richtext'
-            , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
-          ]
-        , 'continuation_url':   entitydata_list_type_url("testcoll", type_id)
-        })
-    if entity_id:
-        context_dict['fields'][0]['field_value'] = entity_id
-        context_dict['fields'][2]['field_value'] = '%s testcoll/testtype/%s'%(update,entity_id)
-        context_dict['fields'][3]['field_value'] = '%s coll testcoll, type testtype, entity %s'%(update,entity_id)
-        context_dict['orig_id']     = entity_id
-    if type_id:
-        context_dict['fields'][1]['field_value'] = type_id       
-    if orig_id:
-        context_dict['orig_id']     = orig_id
-    if action:  
-        context_dict['action']      = action
-    return context_dict
-
 def entitydata_default_view_form_data(
-        coll_id="testcoll", 
+        coll_id="testcoll", orig_coll=None,
         type_id="testtype", orig_type=None,
         entity_id=None, orig_id=None, 
         action=None, cancel=None, close=None, view=None, edit=None, copy=None, 
         update="Entity",
-        add_view_field=None, use_view=None,
+        add_view_field=None, use_view=None, default_view=None,
         new_view=None, new_field=None, new_type=None, 
         new_enum=None, do_import=None
         ):
@@ -488,10 +465,13 @@ def entitydata_default_view_form_data(
         form_data_dict['Entity_comment']  = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
         form_data_dict['orig_id']         = entity_id
         form_data_dict['orig_type']       = type_id
+        form_data_dict['orig_coll']       = coll_id
     if orig_id:
         form_data_dict['orig_id']         = orig_id
     if orig_type:
         form_data_dict['orig_type']       = orig_type
+    if orig_coll:
+        form_data_dict['orig_coll']       = orig_coll
     if action:
         form_data_dict['action']          = action
     if cancel:
@@ -509,6 +489,8 @@ def entitydata_default_view_form_data(
     elif use_view:
         form_data_dict['use_view']        = "Show view"
         form_data_dict['view_choice']     = use_view
+    elif default_view:
+        form_data_dict['default_view']    = "default_view"
     elif new_view:
         form_data_dict['new_view']        = new_view
     elif new_field:
@@ -532,61 +514,70 @@ def entitydata_default_view_form_data(
 # Used in test_entitygenericedit - move?
 
 def entitydata_recordtype_view_context_data(
-        entity_id=None, orig_id=None, type_id="testtype", type_uri=None, type_ids=[],
-        action=None, update="Entity"
+        entity_id=None, orig_id=None, coll_id="testcoll", type_id="testtype", type_uri=None, type_ids=[],
+        action=None, update="Entity", view_label="Type definition"
     ):
+    if entity_id:
+        entity_label = '%s %s/%s/%s'%(update, coll_id, type_id, entity_id)
+        entity_descr = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
+    else:
+        entity_label = '%s data ... (%s/%s)'%(update, coll_id, type_id)
+        entity_descr = '%s description ... (%s/%s)'%(update, coll_id, type_id)
     context_dict = (
-        { 'title':              "Collection testcoll"
-        , 'coll_id':            'testcoll'
+        { 'title':              "%s - %s - Collection %s"%(entity_label, view_label, coll_id)
+        , 'heading':            view_label
+        , 'coll_id':            coll_id
         , 'type_id':            type_id
         , 'orig_id':            'orig_entity_id'
         , 'fields':
-          [ { 'field_label':        'Id'
-            , 'field_name':         'entity_id'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            , 'field_id':           'Type_id'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Slug'
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_label':        'Label'
-            , 'field_name':         'Type_label'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_id':           'Type_label'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Text'
-            , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
-          , { 'field_label':        'Comment'
-            , 'field_name':         'Type_comment'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_id':           'Type_comment'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Richtext'
-            , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
-            , 'options':            []
-            }
-          , { 'field_label':        'URI'
-            , 'field_name':         'Type_uri'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_id':           'Type_uri'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_target_type':  'annal:Identifier'
-            , 'field_value':        ""
-            , 'options':            []
-            }
+          [ context_field_row(
+              { 'field_label':        'Type Id'
+              , 'field_name':         'entity_id'
+              , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
+              , 'field_id':           'Type_id'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   'annal:EntityRef'
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_label':        'Label'
+              , 'field_name':         'Type_label'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_id':           'Type_label'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   'annal:Text'
+              , 'field_value':        entity_label
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_label':        'Comment'
+              , 'field_name':         'Type_comment'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_id':           'Type_comment'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   'annal:Richtext'
+              , 'field_value':        entity_descr
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_label':        'Type URI'
+              , 'field_name':         'Type_uri'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_id':           'Type_uri'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_value_type':   'annal:Identifier'
+              , 'field_value':        ""
+              , 'options':            []
+              })
           ]
-        , 'continuation_url':   entitydata_list_type_url("testcoll", type_id)
+        , 'continuation_url':   entitydata_list_type_url(coll_id, type_id)
         })
     if entity_id:
-        context_dict['fields'][0]['field_value'] = entity_id
-        context_dict['fields'][1]['field_value'] = '%s testcoll/testtype/%s'%(update,entity_id)
-        context_dict['fields'][2]['field_value'] = '%s coll testcoll, type testtype, entity %s'%(update,entity_id)
+        context_dict['fields'][0]['row_field_descs'][0]['field_value'] = entity_id
         context_dict['orig_id']                  = entity_id
     if type_uri:
-        context_dict['fields'][3]['field_value'] = type_uri # TestBasePath + "/c/%s/d/%s/%s/"%("testcoll", "testtype", entity_id)
+        context_dict['fields'][3]['row_field_descs'][0]['field_value'] = type_uri
     if orig_id:
         context_dict['orig_id']     = orig_id
     if action:  
@@ -596,9 +587,9 @@ def entitydata_recordtype_view_context_data(
 def entitydata_recordtype_view_form_data(
         coll_id="testcoll", 
         type_id="testtype", orig_type=None, type_uri=None,
-        entity_id=None, orig_id=None, 
+        entity_id="", orig_id=None, 
         action=None, cancel=None, update="Entity",
-        add_view_field=None, open_view=None):
+        add_view_field=None, open_view=None, customize=None):
     # log.info("entitydata_recordtype_view_form_data: entity_id %s"%(entity_id))
     form_data_dict = (
         { 'Type_label':         '%s data ... (%s/%s)'%(update, coll_id, type_id)
@@ -606,21 +597,25 @@ def entitydata_recordtype_view_form_data(
         , 'orig_id':            'orig_entity_id'
         , 'continuation_url':   entitydata_list_type_url(coll_id, orig_type or type_id)
         })
+    if entity_id is not None:
+        form_data_dict['entity_id']       = entity_id
     if entity_id and type_id:
         type_url = entity_url(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
         type_url = type_url.replace("___", entity_id)  # Preserve bad type in form data
         form_data_dict['entity_id']       = entity_id
-        form_data_dict['Type_label']      = '%s %s/%s/%s'%(update, coll_id, type_id, entity_id)
-        form_data_dict['Type_comment']    = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
         form_data_dict['Type_uri']        = "" # type_url
         form_data_dict['orig_id']         = entity_id
+    if orig_id:
+        form_data_dict['orig_id']         = orig_id
+    label_id = entity_id or orig_id
+    if label_id and type_id:
+        form_data_dict['Type_label']      = '%s %s/%s/%s'%(update, coll_id, type_id, label_id)
+        form_data_dict['Type_comment']    = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, label_id)
     if type_id:
         form_data_dict['entity_type']     = "_type/"+type_id
         form_data_dict['orig_type']       = extract_entity_id(type_id)
     if type_uri:
         form_data_dict['Type_uri']        = type_uri
-    if orig_id:
-        form_data_dict['orig_id']         = orig_id
     if orig_type:
         form_data_dict['orig_type']       = orig_type
     if action:
@@ -631,6 +626,8 @@ def entitydata_recordtype_view_form_data(
         form_data_dict['add_view_field']  = add_view_field
     elif open_view:
         form_data_dict['open_view']       = open_view
+    elif customize:
+        form_data_dict['customize']       = customize
     else:
         form_data_dict['save']            = 'Save'
     return form_data_dict
@@ -641,7 +638,10 @@ def entitydata_recordtype_view_form_data(
 #
 #   -----------------------------------------------------------------------------
 
-def entitylist_form_data(action, search="", list_id="Default_list", entities=None, continuation_url=None):
+def entitylist_form_data(
+        action, list_scope_all=None, search="", 
+        list_id="Default_list", entities=None, 
+        continuation_url=None):
     """
     Form data from entity list form submission
 
@@ -678,6 +678,8 @@ def entitylist_form_data(action, search="", list_id="Default_list", entities=Non
         , 'list_choice':        "_list/"+list_id
         , 'continuation_url':   continuation_url
         })
+    if list_scope_all is not None:
+        form_data['list_scope_all'] = list_scope_all
     if entities is not None:
         form_data['entity_select'] = entities
     if action in form_actions:
@@ -692,16 +694,23 @@ def entitylist_form_data(action, search="", list_id="Default_list", entities=Non
 #
 #   -----------------------------------------------------------------------------
 
-def default_fields(coll_id=None, type_id=None, entity_id=None, width=12, **kwargs):
+def default_fields(
+    coll_id=None, type_id=None, entity_id=None, 
+    width=12, **kwargs):
     """
     Returns a function that accepts a field width and returns a dictionary of entity values
     for testing.  The goal is to isolate default entity value settings from the test cases.
     """
-    def_label       = default_label(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
-    def_comment     = default_comment(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
+    def_label       = kwargs.get("default_label",
+        default_label(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
+        )
+    def_comment     = kwargs.get("default_comment",
+        default_comment(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
+        )
     def_label_esc   = def_label.replace("'", "&#39;")
     def_comment_esc = def_comment.replace("'", "&#39;")
     def_entity_url  = collection_entity_view_url(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
+    collection_url  = collection_view_url(coll_id).rstrip("/")
     def def_fields(width=12):
         fields = layout_classes(width=width)
         fields.update(
@@ -713,6 +722,7 @@ def default_fields(coll_id=None, type_id=None, entity_id=None, width=12, **kwarg
             , 'default_label_esc':   def_label_esc
             , 'default_comment_esc': def_comment_esc
             , 'default_entity_url':  def_entity_url
+            , 'collection_url':      collection_url
             })
         if kwargs:
             fields.update(kwargs)
@@ -730,6 +740,10 @@ def default_comment(coll_id=None, type_id=None, entity_id=None):
     # Note: for built-in types, default values matches corresponding sitedata _initial_values
     if type_id in ["_type", "_view", "_list", "_field"]:
         return ""
+    #@@
+    # if type_id == "_field":
+    #     return "(tooltip text here)"
+    #@@
     return message.ENTITY_DEFAULT_COMMENT%dict(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
 
 def error_label(coll_id=None, type_id=None, entity_id=None):
@@ -752,9 +766,7 @@ def layout_classes(width=12):
             , 'input_classes':          "view-value small-12 medium-6 columns"
             , 'col_head_classes':       "view-label col-head small-12 medium-4 columns"
             , 'col_item_classes':       "view-value col-???? small-12 medium-4 columns"
-            , 'button_wide_classes':    "small-4 columns"
-            , 'button_left_classes':    "form-buttons small-12 columns"
-            , 'button_right_classes':   "form-buttons small-12 columns text-right"
+            , 'button_wide_classes':    "small-12 medium-4 columns"
             })
     elif width == 6:
         class_dict = (
@@ -762,9 +774,7 @@ def layout_classes(width=12):
             , 'input_classes':          "view-value small-12 medium-8 columns"
             , 'col_head_classes':       "view-label col-head small-12 medium-6 columns"
             , 'col_item_classes':       "small-12 medium-6 columns"
-            , 'button_wide_classes':    "small-6 columns"
-            , 'button_left_classes':    "form-buttons small-12 columns"
-            , 'button_right_classes':   "form-buttons small-12 columns text-right"
+            , 'button_wide_classes':    "small-12 medium-6 columns"
             })
     elif width == 10:
         class_dict = (
@@ -773,6 +783,7 @@ def layout_classes(width=12):
     elif width == 12:
         class_dict = (
             { 'group_label_classes':    "group-label small-12 medium-2 columns"
+            , 'group_placeholder_classes':  "group-placeholder small-12 medium-10 columns"
             , 'group_space_classes':    "small-12 medium-2 columns hide-for-small-only"
             , 'group_row_head_classes': "small-12 medium-10 columns hide-for-small-only"
             , 'group_row_body_classes': "small-12 medium-10 columns"
@@ -784,11 +795,14 @@ def layout_classes(width=12):
             , 'space_classes':          "medium-2 columns show-for-medium-up"
             , 'button_wide_classes':    "small-12 medium-10 columns"
             , 'button_half_classes':    "form-buttons small-12 medium-5 columns"
-            , 'button_left_classes':    "form-buttons small-12 columns"
-            , 'button_right_classes':   "form-buttons small-12 columns text-right"
             })
     else:
         assert False, "Unexpected width %r"%width
+    class_dict.update(
+        { 'button_left_classes':        "form-buttons small-12 columns"
+        , 'button_right_classes':       "form-buttons small-12 columns text-right"
+        , 'button_r_med_up_classes':    "form-buttons small-12 columns medium-up-text-right"        
+        })
     return class_dict
 
 # End.
