@@ -33,7 +33,8 @@ from entity_testutils import (
     collection_dir, 
     collection_edit_url,
     collection_entity_view_url,
-    site_title
+    site_title,
+    context_field_row
     )
 
 #   -----------------------------------------------------------------------------
@@ -54,9 +55,6 @@ def recordtype_dir(coll_id="testcoll", type_id="testtype"):
 #   These use the Django `reverse` function so they correspond to
 #   the declared URI patterns.
 
-def recordtype_site_url(site, type_id="testtype"):
-    return site._entityurl + layout.SITE_TYPE_PATH%{'id': type_id} + "/"
-
 def recordtype_coll_url(site, coll_id="testcoll", type_id="testtype"):
     return urlparse.urljoin(
         site._entityurl,
@@ -111,7 +109,7 @@ def recordtype_value_keys(type_uri=False, supertype_uris=False):
     if type_uri:
         ks.add('annal:uri')
     if supertype_uris:
-        ks.add('annal:supertype_uris')
+        ks.add('annal:supertype_uri')
     return ks
 
 def recordtype_load_keys(type_uri=False, supertype_uris=False):
@@ -137,13 +135,13 @@ def recordtype_create_values(
     if type_uri:
         d['annal:uri'] = type_uri
         if supertype_uris is not None:
-            d['annal:supertype_uris'] = (
-                [ { 'annal:supertype_uri': st } for st in supertype_uris ]
+            d['annal:supertype_uri'] = (
+                [ { '@id': st } for st in supertype_uris ]
                 )
         else:
-            d['annal:supertype_uris'] = (
-                [ { 'annal:supertype_uri': type_uri+"/super1" }
-                , { 'annal:supertype_uri': type_uri+"/super2" }
+            d['annal:supertype_uri'] = (
+                [ { '@id': type_uri+"/super1" }
+                , { '@id': type_uri+"/super2" }
                 ])
     return d
 
@@ -169,10 +167,9 @@ def recordtype_read_values(
         hosturi=hosturi
         ).copy()
     d.update(
-        { '@id':            "./"
+        { '@id':            layout.COLL_BASE_TYPE_REF%{'id': type_id}
         , '@type':          ["annal:Type"]
-        # , '@base':          "../.."
-        , '@context':       ["../../coll_context.jsonld"]
+        , '@context':       [{"@base": "../../"}, "../../coll_context.jsonld"]
         })
     return d
 
@@ -183,95 +180,105 @@ def recordtype_read_values(
 #   -----------------------------------------------------------------------------
 
 def recordtype_entity_view_context_data(
-        coll_id="testcoll", type_id=None, orig_id=None, type_ids=[],
+        coll_id="testcoll", type_id="", orig_id=None, type_ids=[],
         action=None, update="RecordType",
         type_uri=None, supertype_uris=None #@@ need to deal with these.
     ):
+    if type_id:
+        type_label = "%s %s/%s"%(update, coll_id, type_id)
+        type_descr = "%s help for %s in collection %s"%(update, type_id, coll_id)
+    else:
+        type_label = "%s data ... (%s/%s)"%(update, coll_id, type_id)
+        type_descr = "%s description ... (%s/%s)"%(update, coll_id, type_id)
     context_dict = (
-        { 'title':              "Collection %s"%(coll_id)
+        { 'title':              "%s - Type definition - Collection %s"%(type_label, coll_id)
+        , 'heading':            "Type definition"
         , 'coll_id':            coll_id
         , 'type_id':            '_type'
         , 'orig_id':            'orig_type_id'
         , 'fields':
-          [ { 'field_id':           'Type_id'
-            , 'field_name':         'entity_id'
-            , 'field_target_type':  'annal:Slug'
-            , 'field_label':        'Id'
-            , 'field_render_type':  'EntityId'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_label'
-            , 'field_name':         'Type_label'
-            , 'field_target_type':  'annal:Text'
-            , 'field_label':        'Label'
-            , 'field_render_type':  'Text'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value':        '%s data ... (%s/%s)'%(update, coll_id, type_id)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_comment'
-            , 'field_name':         'Type_comment'
-            , 'field_label':        'Comment'
-            , 'field_target_type':  'annal:Richtext'
-            , 'field_render_type':  'Markdown'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value':        '%s description ... (%s/%s)'%(update, coll_id, type_id)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_uri'
-            , 'field_name':         'Type_uri'
-            , 'field_target_type':  'annal:Identifier'
-            , 'field_label':        'URI'
-            , 'field_render_type':  'Identifier'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_supertype_uris'
-            , 'field_name':         'Type_supertype_uris'
-            , 'field_target_type':  'annal:Type_supertype_uri'
-            , 'field_label':        'Supertype URIs'
-            , 'field_render_type':  'RepeatGroupRow'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12')
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_view'
-            , 'field_name':         'Type_view'
-            , 'field_target_type':  'annal:View'
-            , 'field_label':        'Default view'
-            , 'field_render_type':  'Enum_optional'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
-            }
-          , { 'field_id':           'Type_list'
-            , 'field_name':         'Type_list'
-            , 'field_target_type':  'annal:List'
-            , 'field_label':        'Default list'
-            , 'field_render_type':  'Enum_optional'
-            , 'field_value_mode':   'Value_direct'
-            , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
-            # , 'field_value':      (Supplied separately)
-            , 'options':            []
+          [ context_field_row(
+              { 'field_id':           'Type_id'
+              , 'field_name':         'entity_id'
+              , 'field_value_type':   'annal:EntityRef'
+              , 'field_label':        'Type Id'
+              , 'field_render_type':  'EntityId'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_id':           'Type_label'
+              , 'field_name':         'Type_label'
+              , 'field_value_type':   'annal:Text'
+              , 'field_label':        'Label'
+              , 'field_render_type':  'Text'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_value':        type_label
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_id':           'Type_comment'
+              , 'field_name':         'Type_comment'
+              , 'field_label':        'Comment'
+              , 'field_value_type':   'annal:Richtext'
+              , 'field_render_type':  'Markdown'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              , 'field_value':        type_descr
+              , 'options':            []
+              })
+          , context_field_row(
+              { 'field_id':           'Type_uri'
+              , 'field_name':         'Type_uri'
+              , 'field_value_type':   'annal:Identifier'
+              , 'field_label':        'Type URI'
+              , 'field_render_type':  'Identifier'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12')
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              })
+          , { 'field_id':             'Type_supertype_uris'
+            , 'field_name':           'Type_supertype_uris'
+            , 'field_value_type':     'annal:Type_supertype_uris'
+            , 'field_label':          'Supertype URIs'
+            , 'field_render_type':    'Group_Set_Row'
+            , 'field_value_mode':     'Value_direct'
+            , 'field_placement':      get_placement_classes('small:0,12')
+            # , 'field_value':        (Supplied separately)
+            , 'options':              []
             }
+          , context_field_row(
+              { 'field_id':           'Type_view'
+              , 'field_name':         'Type_view'
+              , 'field_value_type':   'annal:View'
+              , 'field_label':        'Default view'
+              , 'field_render_type':  'Enum_optional'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              },
+              { 'field_id':           'Type_list'
+              , 'field_name':         'Type_list'
+              , 'field_value_type':   'annal:List'
+              , 'field_label':        'Default list'
+              , 'field_render_type':  'Enum_optional'
+              , 'field_value_mode':   'Value_direct'
+              , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
+              # , 'field_value':      (Supplied separately)
+              , 'options':            []
+              })
           ]
         , 'continuation_url':   entitydata_list_type_url(coll_id, "_type")
         })
     if type_id:
         type_url = recordtype_url(coll_id=coll_id, type_id=type_id)
-        context_dict['fields'][0]['field_value'] = type_id
-        context_dict['fields'][1]['field_value'] = '%s %s/%s'%(update, coll_id, type_id)
-        context_dict['fields'][2]['field_value'] = '%s help for %s in collection %s'%(update, type_id, coll_id)
-        context_dict['fields'][3]['field_value'] = type_url or ""
+        context_dict['fields'][0]['row_field_descs'][0]['field_value'] = type_id
+        context_dict['fields'][3]['row_field_descs'][0]['field_value'] = type_url or ""
         context_dict['orig_id']     = type_id
     if orig_id:
         context_dict['orig_id']     = orig_id
@@ -280,8 +287,8 @@ def recordtype_entity_view_context_data(
     return context_dict
 
 def recordtype_entity_view_form_data(
-        coll_id="testcoll", 
-        type_id=None, orig_id=None, 
+        coll_id="testcoll", orig_coll=None,
+        type_id="", orig_id=None, 
         action=None, cancel=None, close=None, edit=None, copy=None, task=None,
         update="RecordType",
         type_uri=None
@@ -297,19 +304,24 @@ def recordtype_entity_view_form_data(
         , 'orig_id':            'orig_type_id'
         , 'continuation_url':   entitydata_list_type_url(coll_id, "_type")
         })
+    if type_id is not None:
+        form_data_dict['entity_id']           = type_id
     if type_id:
         type_url  = recordtype_url(coll_id=coll_id, type_id=type_id)
         type_help = '%s help for %s in collection %s'%(update, type_id, coll_id)
         form_data_dict['entity_id']           = type_id
-        form_data_dict['orig_id']             = type_id
         form_data_dict['Type_label']          = '%s %s/%s'%(update, coll_id, type_id)
         form_data_dict['Type_comment']        = type_help
         form_data_dict['Type_uri']            = type_url or ""
         form_data_dict['Type_view']           = "_view/Default_view"
         form_data_dict['Type_list']           = "_list/Default_list"
+        form_data_dict['orig_id']             = type_id
         form_data_dict['orig_type']           = "_type"
+        form_data_dict['orig_coll']           = coll_id
     if orig_id:
         form_data_dict['orig_id']   = orig_id
+    if orig_coll:
+        form_data_dict['orig_coll'] = orig_coll
     if action:
         form_data_dict['action']    = action
     if cancel:
@@ -326,8 +338,9 @@ def recordtype_entity_view_form_data(
         form_data_dict['save']      = "Save"
     if type_uri:
         form_data_dict['Type_uri']                                   = type_uri
-        form_data_dict['Type_supertype_uris__0__Type_supertype_uri'] = type_uri+"/super1"
-        form_data_dict['Type_supertype_uris__1__Type_supertype_uri'] = type_uri+"/super2"
+        type_uri_rstrip = type_uri.rstrip()
+        form_data_dict['Type_supertype_uris__0__Type_supertype_uri'] = type_uri_rstrip+"/super1"
+        form_data_dict['Type_supertype_uris__1__Type_supertype_uri'] = type_uri_rstrip+"/super2"
     return form_data_dict
 
 #   -----------------------------------------------------------------------------
