@@ -47,9 +47,6 @@ def annalistuser_dir(coll_id="testcoll", user_id="testuser"):
 #
 #   -----------------------------------------------------------------------------
 
-def annalistuser_site_url(site, user_id="testuser"):
-    return site._entityurl + layout.SITE_USER_PATH%{'id': user_id} + "/"
-
 def annalistuser_coll_url(site, coll_id="testcoll", user_id="testuser"):
     return urlparse.urljoin(
         site._entityurl,
@@ -100,7 +97,7 @@ def annalistuser_value_keys():
         , 'annal:url'
         , 'rdfs:label', 'rdfs:comment'
         , 'annal:user_uri'
-        , 'annal:user_permissions'
+        , 'annal:user_permission'
         ])
     return keys
 
@@ -121,7 +118,7 @@ def annalistuser_create_values(
         , 'rdfs:label':             user_name
         , 'rdfs:comment':           "User %s: permissions for %s in collection %s"%(user_id, user_name, coll_id)
         , 'annal:user_uri':         user_uri
-        , 'annal:user_permissions': user_permissions
+        , 'annal:user_permission': user_permissions
         })
     return d
 
@@ -154,9 +151,9 @@ def annalistuser_read_values(
             hosturi=hosturi
             )
     d.update(
-        { '@id':            layout.META_USER_REF%{'id': user_id}
+        { '@id':            layout.COLL_BASE_USER_REF%{'id': user_id}
         , '@type':          ["annal:User"]
-        , '@context':       [layout.USER_CONTEXT_FILE]
+        , '@context':       [ {'@base': '../../'}, layout.ENTITY_CONTEXT_FILE ]
         })
     return d
 
@@ -172,7 +169,7 @@ def annalistuser_view_form_data(
         user_uri="mailto:testuser@example.org", 
         user_permissions="VIEW CREATE UPDATE DELETE CONFIG ADMIN",
         action=None, cancel=None, close=None, edit=None, copy=None, 
-        orig_id=None
+        orig_coll=None, orig_id=None
         ):
     form_data_dict = (
         { 'entity_id':          user_id
@@ -182,10 +179,13 @@ def annalistuser_view_form_data(
         , 'User_uri':           user_uri
         , 'User_permissions':   user_permissions
         , 'orig_type':          "_user"
+        , "orig_coll":          coll_id
         , 'continuation_url':   entitydata_list_type_url(coll_id, "_user")
         })
     if orig_id:
         form_data_dict['orig_id']       = orig_id
+    if orig_coll:
+        form_data_dict['orig_coll']     = orig_coll
     if action:
         form_data_dict['action']        = action
     if cancel:
