@@ -19,6 +19,7 @@ from django.template                        import Context, Template, loader
 
 from annalist.views.fields.render_placement import Placement, get_placement_classes
 
+from annalist.tests.tests                   import TestHost, TestHostUri, TestBasePath, TestBaseUri
 from annalist.tests.AnnalistTestCase        import AnnalistTestCase
 
 #   -----------------------------------------------------------------------------
@@ -51,11 +52,12 @@ class FieldRendererTestSupport(AnnalistTestCase):
 
     def _make_test_context(self, val, 
             repeat_prefix="repeat_prefix_", 
-            target_value= None, 
+            target_value=None, 
             field_link=None, 
             target_link=None, 
             field_ref_type=None,
-            options=None
+            options=None,
+            coll_id="testcoll", coll=None
         ):
         cd = (
             { 'field':
@@ -69,7 +71,12 @@ class FieldRendererTestSupport(AnnalistTestCase):
               , 'field_view_value':     val
               , 'continuation_param':   "?continuation_url=test_cont"
               }
+            , 'collection':           coll
             , 'repeat_prefix':        repeat_prefix
+            , 'HOST':                 TestHostUri
+            , 'SITE':                 TestBaseUri
+            , 'COLL':                 TestBaseUri+"/c/"+coll_id+"/"
+            , 'BASE':                 TestBasePath+"/c/"+coll_id+"/d/"
             })
         if target_value is not None:
             cd['field']['target_value'] = target_value
