@@ -33,7 +33,6 @@ from annalist.models.sitedata               import SiteData
 from annalist.models.collection             import Collection
 from annalist.models.annalistuser           import AnnalistUser
 
-from annalist.views.annalistuserdelete      import AnnalistUserDeleteConfirmedView
 from annalist.views.fields.render_tokenset  import get_field_tokenset_renderer
 
 from AnnalistTestCase       import AnnalistTestCase
@@ -45,11 +44,12 @@ from entity_testutils       import (
     collection_entity_view_url,
     collection_create_values,
     create_user_permissions,
-    create_test_user
+    create_test_user,
+    context_view_field
     )
 from entity_testuserdata    import (
     annalistuser_dir,
-    annalistuser_site_url, annalistuser_coll_url, annalistuser_url, annalistuser_edit_url,
+    annalistuser_coll_url, annalistuser_url, annalistuser_edit_url,
     annalistuser_value_keys, annalistuser_load_keys,
     annalistuser_create_values, annalistuser_values, annalistuser_read_values,
     annalistuser_view_form_data,
@@ -77,10 +77,31 @@ class AnnalistUserTest(AnnalistTestCase):
         self.testsite = Site(TestBaseUri, TestBaseDir)
         self.sitedata = SiteData(self.testsite)
         self.testcoll = Collection(self.testsite, "testcoll")
+        self.layout = (
+            { 'enum_field_placement_dir':   layout.ENUM_FIELD_PLACEMENT_DIR
+            , 'enum_list_type_dir':         layout.ENUM_LIST_TYPE_DIR
+            , 'enum_render_type_dir':       layout.ENUM_RENDER_TYPE_DIR
+            , 'enum_value_type_dir':        layout.ENUM_VALUE_TYPE_DIR
+            , 'enum_value_mode_dir':        layout.ENUM_VALUE_MODE_DIR
+            , 'field_typeid':               layout.FIELD_TYPEID
+            , 'group_typeid':               layout.GROUP_TYPEID
+            , 'list_typeid':                layout.LIST_TYPEID
+            , 'type_typeid':                layout.TYPE_TYPEID
+            , 'user_typeid':                layout.USER_TYPEID
+            , 'view_typeid':                layout.VIEW_TYPEID
+            , 'vocab_typeid':               layout.VOCAB_TYPEID
+            , 'field_dir':                  layout.FIELD_DIR
+            , 'group_dir':                  layout.GROUP_DIR
+            , 'list_dir':                   layout.LIST_DIR
+            , 'type_dir':                   layout.TYPE_DIR
+            , 'user_dir':                   layout.USER_DIR
+            , 'view_dir':                   layout.VIEW_DIR
+            , 'vocab_dir':                  layout.VOCAB_DIR
+            })
         return
 
     def tearDown(self):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     def test_AnnalistUserTest(self):
@@ -92,7 +113,7 @@ class AnnalistUserTest(AnnalistTestCase):
         url = annalistuser_coll_url(self.testsite, coll_id="testcoll", user_id="testuser")
         self.assertEqual(usr._entitytype,   ANNAL.CURIE.User)
         self.assertEqual(usr._entityfile,   layout.USER_META_FILE)
-        self.assertEqual(usr._entityref,    layout.META_USER_REF%{'id': "testuser"})
+        self.assertEqual(usr._entityref,    layout.COLL_BASE_USER_REF%{'id': "testuser"})
         self.assertEqual(usr._entityid,     "testuser")
         self.assertEqual(usr._entityurl,    url)
         self.assertEqual(usr._entitydir,    annalistuser_dir(user_id="testuser"))
@@ -102,9 +123,15 @@ class AnnalistUserTest(AnnalistTestCase):
     def test_annalistuser1_data(self):
         usr = AnnalistUser(self.testcoll, "user1")
         self.assertEqual(usr.get_id(), "user1")
-        self.assertEqual(usr.get_type_id(), "_user")
-        self.assertIn("/c/testcoll/_annalist_collection/users/user1/", usr.get_url())
-        self.assertEqual(TestBaseUri + "/c/testcoll/d/_user/user1/", usr.get_view_url())
+        self.assertEqual(usr.get_type_id(), layout.USER_TYPEID)
+        self.assertIn(
+            "/c/testcoll/d/%(user_dir)s/user1/"%self.layout, 
+            usr.get_url()
+            )
+        self.assertEqual(
+            TestBaseUri + "/c/testcoll/d/%(user_typeid)s/user1/"%self.layout, 
+            usr.get_view_url()
+            )
         usr.set_values(annalistuser_create_values(user_id="user1"))
         td = usr.get_values()
         self.assertEqual(set(td.keys()), set(annalistuser_value_keys()))
@@ -115,9 +142,15 @@ class AnnalistUserTest(AnnalistTestCase):
     def test_annalistuser2_data(self):
         usr = AnnalistUser(self.testcoll, "user2")
         self.assertEqual(usr.get_id(), "user2")
-        self.assertEqual(usr.get_type_id(), "_user")
-        self.assertIn("/c/testcoll/_annalist_collection/users/user2/", usr.get_url())
-        self.assertEqual(TestBaseUri + "/c/testcoll/d/_user/user2/", usr.get_view_url())
+        self.assertEqual(usr.get_type_id(), layout.USER_TYPEID)
+        self.assertIn(
+            "/c/testcoll/d/%(user_dir)s/user2/"%self.layout, 
+            usr.get_url()
+            )
+        self.assertEqual(
+            TestBaseUri + "/c/testcoll/d/%(user_typeid)s/user2/"%self.layout, 
+            usr.get_view_url()
+            )
         usr.set_values(annalistuser_create_values(user_id="user2"))
         ugv = usr.get_values()
         self.assertEqual(set(ugv.keys()), set(annalistuser_value_keys()))
@@ -126,7 +159,9 @@ class AnnalistUserTest(AnnalistTestCase):
         return
 
     def test_annalistuser_create_load(self):
-        usr = AnnalistUser.create(self.testcoll, "user1", annalistuser_create_values(user_id="user1"))
+        usr = AnnalistUser.create(
+            self.testcoll, "user1", annalistuser_create_values(user_id="user1")
+            )
         uld = AnnalistUser.load(self.testcoll, "user1").get_values()
         ued = annalistuser_read_values(user_id="user1")
         self.assertKeysMatch(ued, uld)
@@ -137,23 +172,23 @@ class AnnalistUserTest(AnnalistTestCase):
         usr = AnnalistUser.load(self.testcoll, "_unknown_user_perms", altscope="all")
         self.assertEqual(usr.get_id(), "_unknown_user_perms")
         self.assertIn(
-            "/c/_annalist_site/_annalist_collection/users/_unknown_user_perms/", 
+            "/c/_annalist_site/d/%(user_dir)s/_unknown_user_perms/"%self.layout, 
             usr.get_url()
             )
         self.assertIn(
-            "/c/testcoll/d/_user/_unknown_user_perms", 
+            "/c/testcoll/d/%(user_typeid)s/_unknown_user_perms"%self.layout, 
             usr.get_view_url()
             )
-        self.assertEqual(usr.get_type_id(), "_user")
+        self.assertEqual(usr.get_type_id(), layout.USER_TYPEID)
         uld = usr.get_values()
         self.assertEqual(set(uld.keys()), set(annalistuser_load_keys()))
         uev = annalistuser_read_values(user_id="_unknown_user_perms")
         uev.update(
             { 'rdfs:label':             'Unknown user'
-            , 'rdfs:comment':           'Permissions for unauthenticated user.'
             , 'annal:user_uri':         'annal:User/_unknown_user_perms'
-            , 'annal:user_permissions': ['VIEW']
+            , 'annal:user_permission':  ['VIEW']
             })
+        uev.pop('rdfs:comment', None)
         self.assertDictionaryMatch(uld, uev)
         return
 
@@ -171,9 +206,13 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
     def setUp(self):
         init_annalist_test_site()
         self.testsite = Site(TestBaseUri, TestBaseDir)
-        self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
+        self.testcoll = Collection.create(
+            self.testsite, "testcoll", collection_create_values("testcoll")
+            )
         # For checking Location: header values...
-        self.continuation_url = TestHostUri + entitydata_list_type_url(coll_id="testcoll", type_id="_user")
+        self.continuation_url = TestHostUri + entitydata_list_type_url(
+            coll_id="testcoll", type_id=layout.USER_TYPEID
+            )
         # Login and permissions
         create_test_user(
             self.testcoll, "testuser", "testpassword",
@@ -185,7 +224,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     #   -----------------------------------------------------------------------------
@@ -291,15 +330,20 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
         return
 
     def test_get_new_user_form_rendering(self):
-        u = entitydata_edit_url("new", "testcoll", "_user", view_id="User_view")
+        u = entitydata_edit_url("new", "testcoll", layout.USER_TYPEID, view_id="User_view")
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<title>Collection testcoll</title>")
-        self.assertContains(r, "<h3>'_user' data in collection 'testcoll'</h3>")
-        field_vals = default_fields(coll_id="testcoll", type_id="_user", entity_id="00000001")
+        field_vals = default_fields(
+            coll_id="testcoll", type_id=layout.USER_TYPEID, entity_id="00000001",
+            tooltip1=context_view_field(r.context, 0, 0)['field_tooltip'],
+            tooltip2=context_view_field(r.context, 1, 0)['field_tooltip'],
+            tooltip3=context_view_field(r.context, 2, 0)['field_tooltip'],
+            tooltip4=context_view_field(r.context, 3, 0)['field_tooltip'],
+            tooltip5=context_view_field(r.context, 4, 0)['field_tooltip']
+            )
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" title="%(tooltip1)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>User Id</span>
@@ -313,7 +357,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=6)
         formrow2 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip2)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>User name</span>
@@ -327,7 +371,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow3 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip3)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>Description</span>
@@ -343,10 +387,10 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow4 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip4)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
-                        <span>URI</span>
+                        <span>User URI</span>
                     </div>
                     <div class="%(input_classes)s">
                         <input type="text" size="64" name="User_uri" 
@@ -357,71 +401,57 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow5 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip5)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>Permissions</span>
                     </div>
                     <div class="%(input_classes)s">
                         <input type="text" size="64" name="User_permissions" 
-                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE&#39;)"
+                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE CONFIG&#39;)"
                                value=""/>
                     </div>
                 </div>
             </div>
             """%field_vals(width=12)
-        formrow6a = """
-            <div class="%(space_classes)s">
-              <div class="row">
-                <div class="small-12 columns">
-                  &nbsp;
-                </div>
-              </div>
-            </div>
-            """%field_vals(width=2)
-        formrow6b = """
-            <div class="%(button_wide_classes)s">
-              <div class="row">
-                <div class="%(button_left_classes)s">
-                  <input type="submit" name="save"      value="Save" />
-                  <input type="submit" name="view"      value="View" />
-                  <input type="submit" name="cancel"    value="Cancel" />
-                </div>
-              </div>
-            </div>
-            """%field_vals(width=4)
         # log.info(r.content)
         self.assertContains(r, formrow1,  html=True)
         self.assertContains(r, formrow2,  html=True)
         self.assertContains(r, formrow3,  html=True)
         self.assertContains(r, formrow4,  html=True)
         self.assertContains(r, formrow5,  html=True)
-        self.assertContains(r, formrow6a, html=True)
-        self.assertContains(r, formrow6b, html=True)
         return
 
     def test_user_permissions_form_rendering(self):
         # Test rendering of permissions from list in entity
-        u = entitydata_edit_url("edit", "testcoll", "_user", "testuser", view_id="User_view")
+        u = entitydata_edit_url(
+            "edit", "testcoll", layout.USER_TYPEID, "testuser", view_id="User_view"
+            )
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        field_vals = default_fields(coll_id="testcoll", type_id="_user", entity_id="testuser")
+        field_vals = default_fields(
+            coll_id="testcoll", type_id=layout.USER_TYPEID, entity_id="testuser",
+            tooltip=context_view_field(r.context, 4, 0)['field_tooltip']
+            )
         formrow5 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>Permissions</span>
                     </div>
                     <div class="%(input_classes)s">
                         <input type="text" size="64" name="User_permissions" 
-                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE&#39;)"
+                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE CONFIG&#39;)"
                                value="VIEW CREATE UPDATE DELETE CONFIG ADMIN"/>
                     </div>
                 </div>
             </div>
             """%field_vals(width=12)
         # log.info(r.content)
+        # log.info
+        # for i in range(len(r.context['fields'])):
+        #     log.info("Field %d: %r"%(i, r.context['fields'][i]))
         self.assertContains(r, formrow5, html=True)
         return
 
@@ -431,21 +461,26 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             "baduserperms",
             user_permissions="VIEW CREATE UPDATE DELETE"
             )
-        u = entitydata_edit_url("edit", "testcoll", "_user", "baduserperms", view_id="User_view")
+        u = entitydata_edit_url(
+            "edit", "testcoll", layout.USER_TYPEID, "baduserperms", view_id="User_view"
+            )
         with SuppressLogging(logging.WARNING):
             r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        field_vals = default_fields(coll_id="testcoll", type_id="_user", entity_id="baduserperms")
+        field_vals = default_fields(
+            coll_id="testcoll", type_id=layout.USER_TYPEID, entity_id="baduserperms",
+            tooltip=context_view_field(r.context, 4, 0)['field_tooltip']
+            )
         formrow5 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" title="%(tooltip)s">
                 <div class="row view-value-row">
                     <div class="%(label_classes)s">
                         <span>Permissions</span>
                     </div>
                     <div class="%(input_classes)s">
                         <input type="text" size="64" name="User_permissions" 
-                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE&#39;)"
+                               placeholder="(user permissions; e.g. &#39;VIEW CREATE UPDATE DELETE CONFIG&#39;)"
                                value="VIEW CREATE UPDATE DELETE"/>
                     </div>
                 </div>
@@ -459,20 +494,49 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
     #   Form response tests
     #   -----------------------------------------------------------------------------
 
+    #   -------- edit user --------
+
+    def test_post_edit_user(self):
+        # The main purpose of this test is to check that user permissions are saved properly
+        self.assertFalse(AnnalistUser.exists(self.testcoll, "edituser"))
+        f = annalistuser_view_form_data(
+            action="edit", orig_id="_default_user_perms", orig_coll="_annalist_site",
+            user_id="edituser",
+            user_name="User edituser",
+            user_uri="mailto:edituser@example.org",
+            user_permissions="VIEW CREATE UPDATE DELETE"
+            )
+        u = entitydata_edit_url(
+            "edit", "testcoll", 
+            layout.USER_TYPEID, entity_id="_default_user_perms", 
+            view_id="User_view"
+            )
+        r = self.client.post(u, f)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertEqual(r.content,       "")
+        self.assertEqual(r['location'], self.continuation_url)
+        # Check that new user exists
+        self.assertTrue(AnnalistUser.exists(self.testcoll, "edituser"))
+        self._check_annalist_user_values("edituser", ["VIEW", "CREATE", "UPDATE", "DELETE"])
+        return
+
     #   -------- copy user --------
 
     def test_post_copy_user(self):
         # The main purpose of this test is to check that user permissions are saved properly
         self.assertFalse(AnnalistUser.exists(self.testcoll, "copyuser"))
         f = annalistuser_view_form_data(
-            action="copy", orig_id="_default_user_perms",
+            action="copy", orig_id="_default_user_perms", orig_coll="_annalist_site",
             user_id="copyuser",
             user_name="User copyuser",
             user_uri="mailto:copyuser@example.org",
             user_permissions="VIEW CREATE UPDATE DELETE"
             )
         u = entitydata_edit_url(
-            "copy", "testcoll", "_user", entity_id="_default_user_perms", view_id="User_view"
+            "copy", "testcoll", 
+            layout.USER_TYPEID, entity_id="_default_user_perms", 
+            view_id="User_view"
             )
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   302)
@@ -484,56 +548,4 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
         self._check_annalist_user_values("copyuser", ["VIEW", "CREATE", "UPDATE", "DELETE"])
         return
 
-#   -----------------------------------------------------------------------------
-#
-#   ConfirmAnnalistUserDeleteTests tests for completion of record deletion
-#
-#   -----------------------------------------------------------------------------
-
-class ConfirmAnnalistUserDeleteTests(AnnalistTestCase):
-    """
-    Tests for Annalist user deletion on response to confirmation form
-    """
-
-    def setUp(self):
-        init_annalist_test_site()
-        self.testsite = Site(TestBaseUri, TestBaseDir)
-        self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
-        self.user = User.objects.create_user('testuser', 'user@test.example.com', 'testpassword')
-        self.user.save()
-        self.client = Client(HTTP_HOST=TestHost)
-        loggedin = self.client.login(username="testuser", password="testpassword")
-        self.assertTrue(loggedin)
-        return
-
-    def tearDown(self):
-        resetSitedata()
-        return
-
-    def test_DeleteConfirmedViewTest(self):
-        self.assertEqual(AnnalistUserDeleteConfirmedView.__name__, "AnnalistUserDeleteConfirmedView", "Check AnnalistUserDeleteConfirmedView class name")
-        return
-
-    # test disabled - this functionality is handled by generic entity delete, and tested accordingly
-    #                 (there is no special-case logic used, e.g. for the configure page.)
-    # @unittest.skip("@@TODO: Delete user not yet implemented")
-    def no_test_post_confirmed_remove_user(self):
-        t = AnnalistUser.create(self.testcoll, "deleteuser", annalistuser_create_values("deleteuser"))
-        self.assertTrue(AnnalistUser.exists(self.testcoll, "deleteuser"))
-        # Submit positive confirmation
-        u = TestHostUri + annalistuser_edit_url("delete", "testcoll")
-        f = annalistuser_delete_confirm_form_data("deleteuser")
-        r = self.client.post(u, f)
-        self.assertEqual(r.status_code,     302)
-        self.assertEqual(r.reason_phrase,   "FOUND")
-        self.assertEqual(r.content,         "")
-        self.assertMatch(r['location'],    
-            "^"+TestHostUri+
-            collection_edit_url("testcoll")+
-            r"\?info_head=.*&info_message=.*deletetype.*testcoll.*$"
-            )
-        # Confirm deletion
-        self.assertFalse(AnnalistUser.exists(self.testcoll, "deletetype"))
-        return
-
 # End.
