@@ -24,6 +24,7 @@ from utils.SuppressLoggingContext   import SuppressLogging
 
 from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist                       import layout
+from annalist                       import message
 from annalist.models.site           import Site
 from annalist.models.collection     import Collection
 from annalist.models.annalistuser   import AnnalistUser
@@ -53,11 +54,17 @@ from entity_testtypedata    import (
     recordtype_create_values, recordtype_read_values
     )
 from entity_testviewdata    import (
+    recordview_edit_url,
     recordview_create_values, recordview_read_values,
     )
 from entity_testlistdata    import (
+    recordlist_edit_url,
     recordlist_create_values, recordlist_read_values,
     )
+from entity_testfielddata import (
+    recordfield_init_keys, recordfield_value_keys, recordfield_load_keys,
+    recordfield_create_values, recordfield_values, recordfield_read_values,
+    )
 from entity_testentitydata  import (
     entitydata_list_all_url
     )
@@ -70,6 +77,9 @@ from entity_testsitedata    import (
     get_site_fields, get_site_fields_sorted, 
     get_site_field_types, get_site_field_types_sorted, 
     )
+from entity_testcolldata    import (
+    collectiondata_view_url
+    )
 
 #   -----------------------------------------------------------------------------
 #
@@ -77,9 +87,10 @@ from entity_testsitedata    import (
 #
 #   -----------------------------------------------------------------------------
 
-site_types = get_site_types()
-site_views = get_site_views()
-site_lists = get_site_lists()
+site_types  = get_site_types()
+site_views  = get_site_views()
+site_lists  = get_site_lists()
+site_fields = get_site_fields()
 
 class CollectionTest(AnnalistTestCase):
     """
@@ -93,18 +104,22 @@ class CollectionTest(AnnalistTestCase):
         self.testcoll     = Collection(self.testsite, "testcoll")
         self.coll1        = collection_values("coll1")
         self.testcoll_add = collection_create_values("testcoll")
-        self.type1_add    = recordtype_create_values("testcoll", "type1")
-        self.type1        = recordtype_read_values("testcoll", "type1")
-        self.type2_add    = recordtype_create_values("testcoll", "type2")
-        self.type2        = recordtype_read_values("testcoll", "type2")
-        self.view1_add    = recordview_create_values("testcoll", "view1")
-        self.view1        = recordview_read_values("testcoll", "view1")
-        self.view2_add    = recordview_create_values("testcoll", "view2")
-        self.view2        = recordview_read_values("testcoll", "view2")
-        self.list1_add    = recordlist_create_values("testcoll", "list1")
-        self.list1        = recordlist_read_values("testcoll", "list1")
-        self.list2_add    = recordlist_create_values("testcoll", "list2")
-        self.list2        = recordlist_read_values("testcoll", "list2")
+        self.type1_add    = recordtype_create_values("testcoll",  "type1")
+        self.type1        = recordtype_read_values("testcoll",    "type1")
+        self.type2_add    = recordtype_create_values("testcoll",  "type2")
+        self.type2        = recordtype_read_values("testcoll",    "type2")
+        self.view1_add    = recordview_create_values("testcoll",  "view1")
+        self.view1        = recordview_read_values("testcoll",    "view1")
+        self.view2_add    = recordview_create_values("testcoll",  "view2")
+        self.view2        = recordview_read_values("testcoll",    "view2")
+        self.list1_add    = recordlist_create_values("testcoll",  "list1")
+        self.list1        = recordlist_read_values("testcoll",    "list1")
+        self.list2_add    = recordlist_create_values("testcoll",  "list2")
+        self.list2        = recordlist_read_values("testcoll",    "list2")
+        self.field1_add   = recordfield_create_values("testcoll", "field1")
+        self.field1       = recordfield_read_values("testcoll",   "field1")
+        self.field2_add   = recordfield_create_values("testcoll", "field2")
+        self.field2       = recordfield_read_values("testcoll",   "field2")
         return
 
     def tearDown(self):
@@ -113,7 +128,7 @@ class CollectionTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     def test_CollectionTest(self):
@@ -125,7 +140,8 @@ class CollectionTest(AnnalistTestCase):
         s = Site(TestBaseUri, TestBaseDir)
         c = Collection(s, "testcoll")
         self.assertEqual(c._entitytype,     ANNAL.CURIE.Collection)
-        self.assertEqual(c._entityfile,     layout.COLL_META_REF)
+        self.assertEqual(c._entitybase,     layout.COLL_BASE_REF)
+        self.assertEqual(c._entityfile,     layout.COLL_META_FILE)
         self.assertEqual(c._entityref,      layout.META_COLL_REF)
         self.assertEqual(c._entityid,       "testcoll")
         self.assertEqual(c._entityurl,      TestHostUri + collection_view_url(coll_id="testcoll"))
@@ -151,7 +167,7 @@ class CollectionTest(AnnalistTestCase):
         self.assertEqual(ugp[RDFS.CURIE.label],               "Test User")
         self.assertEqual(ugp[RDFS.CURIE.comment],             "User user1: permissions for Test User in collection testcoll")
         self.assertEqual(ugp[ANNAL.CURIE.user_uri],           "mailto:testuser@example.org")
-        self.assertEqual(ugp[ANNAL.CURIE.user_permissions],   ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"])
+        self.assertEqual(ugp[ANNAL.CURIE.user_permission],    ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"])
         return
 
     def test_get_local_user_not_defined(self):
@@ -168,7 +184,7 @@ class CollectionTest(AnnalistTestCase):
     def test_get_local_user_missing_fields(self):
         # E.g. what happens if user record is created through default view?  Don't return value.
         d = annalistuser_create_values(user_id="user1")
-        d.pop(ANNAL.CURIE.user_permissions)
+        d.pop(ANNAL.CURIE.user_permission)
         usr = AnnalistUser.create(self.testcoll, "user1", d)
         ugp = self.testcoll.get_user_permissions("user1", "mailto:testuser@example.org")
         self.assertIsNone(ugp)
@@ -179,9 +195,8 @@ class CollectionTest(AnnalistTestCase):
         self.assertEqual(ugp[ANNAL.CURIE.id],                 "_unknown_user_perms")
         self.assertEqual(ugp[ANNAL.CURIE.type_id],            "_user")
         self.assertEqual(ugp[RDFS.CURIE.label],               "Unknown user")
-        self.assertEqual(ugp[RDFS.CURIE.comment],             "Permissions for unauthenticated user.")
         self.assertEqual(ugp[ANNAL.CURIE.user_uri],           "annal:User/_unknown_user_perms")
-        self.assertEqual(ugp[ANNAL.CURIE.user_permissions],   ["VIEW"])
+        self.assertEqual(ugp[ANNAL.CURIE.user_permission],    ["VIEW"])
         return
 
     def test_get_site_user_uri_mismatch(self):
@@ -234,18 +249,18 @@ class CollectionTest(AnnalistTestCase):
 
     def test_add_view(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        viewnames = { t.get_id() for t in self.testcoll.views() }
+        viewnames = { v.get_id() for v in self.testcoll.views() }
         self.assertEqual(viewnames, site_views)
-        t1 = self.testcoll.add_view("view1", self.view1_add)
-        t2 = self.testcoll.add_view("view2", self.view2_add)
-        viewnames = { t.get_id() for t in self.testcoll.views() }
+        v1 = self.testcoll.add_view("view1", self.view1_add)
+        v2 = self.testcoll.add_view("view2", self.view2_add)
+        viewnames = { v.get_id() for v in self.testcoll.views() }
         self.assertEqual(viewnames, {"view1", "view2"}|site_views)
         return
 
     def test_get_view(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        t1 = self.testcoll.add_view("view1", self.view1_add)
-        t2 = self.testcoll.add_view("view2", self.view2_add)
+        v1 = self.testcoll.add_view("view1", self.view1_add)
+        v2 = self.testcoll.add_view("view2", self.view2_add)
         self.assertKeysMatch(self.testcoll.get_view("view1").get_values(), self.view1)
         self.assertKeysMatch(self.testcoll.get_view("view2").get_values(), self.view2)
         self.assertDictionaryMatch(self.testcoll.get_view("view1").get_values(), self.view1)
@@ -254,14 +269,14 @@ class CollectionTest(AnnalistTestCase):
 
     def test_remove_view(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        t1 = self.testcoll.add_view("view1", self.view1_add)
-        t2 = self.testcoll.add_view("view2", self.view2_add)
-        viewnames = { t.get_id() for t in self.testcoll.views() }
+        v1 = self.testcoll.add_view("view1", self.view1_add)
+        v2 = self.testcoll.add_view("view2", self.view2_add)
+        viewnames = { v.get_id() for v in self.testcoll.views() }
         self.assertEqual(viewnames, {"view1", "view2"}|site_views)
         self.testcoll.remove_view("view1")
-        viewnames = { t.get_id() for t in self.testcoll.views() }
+        viewnames = { v.get_id() for v in self.testcoll.views() }
         self.assertEqual(viewnames, {"view2"}|site_views)
-        viewnames = { t.get_id() for t in self.testcoll.views(altscope=None) }
+        viewnames = { v.get_id() for v in self.testcoll.views(altscope=None) }
         self.assertEqual(viewnames, {"view2"})
         return
 
@@ -269,35 +284,68 @@ class CollectionTest(AnnalistTestCase):
 
     def test_add_list(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        listnames = { t.get_id() for t in self.testcoll.lists() }
+        listnames = { l.get_id() for l in self.testcoll.lists() }
         self.assertEqual(listnames, site_lists)
-        t1 = self.testcoll.add_list("list1", self.list1_add)
-        t2 = self.testcoll.add_list("list2", self.list2_add)
-        listnames = { t.get_id() for t in self.testcoll.lists() }
+        l1 = self.testcoll.add_list("list1", self.list1_add)
+        l2 = self.testcoll.add_list("list2", self.list2_add)
+        listnames = { l.get_id() for l in self.testcoll.lists() }
         self.assertEqual(listnames, {"list1", "list2"}|site_lists)
-        listnames = { t.get_id() for t in self.testcoll.lists(altscope=None) }
+        listnames = { l.get_id() for l in self.testcoll.lists(altscope=None) }
         self.assertEqual(listnames, {"list1", "list2"})
         return
 
     def test_get_list(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        t1 = self.testcoll.add_list("list1", self.list1_add)
-        t2 = self.testcoll.add_list("list2", self.list2_add)
+        l1 = self.testcoll.add_list("list1", self.list1_add)
+        l2 = self.testcoll.add_list("list2", self.list2_add)
         self.assertDictionaryMatch(self.testcoll.get_list("list1").get_values(), self.list1)
         self.assertDictionaryMatch(self.testcoll.get_list("list2").get_values(), self.list2)
         return
 
     def test_remove_list(self):
         self.testsite.add_collection("testcoll", self.testcoll_add)
-        t1 = self.testcoll.add_list("list1", self.list1_add)
-        t2 = self.testcoll.add_list("list2", self.list2_add)
-        listnames = { t.get_id() for t in self.testcoll.lists() }
+        l1 = self.testcoll.add_list("list1", self.list1_add)
+        l2 = self.testcoll.add_list("list2", self.list2_add)
+        listnames = { l.get_id() for l in self.testcoll.lists() }
         self.assertEqual(listnames, {"list1", "list2"}|site_lists)
         self.testcoll.remove_list("list1")
-        listnames = { t.get_id() for t in self.testcoll.lists() }
+        listnames = { l.get_id() for l in self.testcoll.lists() }
         self.assertEqual(listnames, {"list2"}|site_lists)
         return
 
+    # View/list fields
+
+    def test_add_field(self):
+        self.testsite.add_collection("testcoll", self.testcoll_add)
+        fieldnames = { f.get_id() for f in self.testcoll.fields() }
+        self.assertEqual(fieldnames, site_fields)
+        f1 = self.testcoll.add_field("field1", self.field1_add)
+        f2 = self.testcoll.add_field("field2", self.field2_add)
+        fieldnames = { f.get_id() for f in self.testcoll.fields() }
+        self.assertEqual(fieldnames, {"field1", "field2"}|site_fields)
+        return
+
+    def test_get_field(self):
+        self.testsite.add_collection("testcoll", self.testcoll_add)
+        f1 = self.testcoll.add_field("field1", self.field1_add)
+        f2 = self.testcoll.add_field("field2", self.field2_add)
+        self.assertDictionaryMatch(self.testcoll.get_field("field1").get_values(), self.field1)
+        self.assertDictionaryMatch(self.testcoll.get_field("field2").get_values(), self.field2)
+        return
+
+    def test_remove_field(self):
+        self.testsite.add_collection("testcoll", self.testcoll_add)
+        f1 = self.testcoll.add_field("field1", self.field1_add)
+        f2 = self.testcoll.add_field("field2", self.field2_add)
+        fieldnames = { f.get_id() for f in self.testcoll.fields() }
+        self.assertEqual(fieldnames, {"field1", "field2"}|site_fields)
+        self.testcoll.remove_field("field1")
+        fieldnames =  { f.get_id() for f in self.testcoll.fields() }
+        self.assertEqual(fieldnames, {"field2"}|site_fields)
+        fieldnames =  { f.get_id() for f in self.testcoll.fields(altscope=None) }
+        self.assertEqual(fieldnames, {"field2"})
+        return
+
     # Alternative parent setting
 
     def test_set_alt_entities_1(self):
@@ -407,7 +455,7 @@ class CollectionEditViewTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     def test_CollectionEditViewTest(self):
@@ -425,13 +473,9 @@ class CollectionEditViewTest(AnnalistTestCase):
     def test_get_view_no_collection(self):
         u = collection_view_url(coll_id="no_collection")
         r = self.client.get(u)
-        self.assertEqual(r.reason_phrase, "FOUND")
-        self.assertEqual(r.content,       "")
-        u1 = TestHostUri + entitydata_list_all_url(coll_id="no_collection")
-        self.assertEqual(r['location'], u1)
-        r1 = self.client.get(u1)
-        self.assertEqual(r1.status_code,   404)
-        self.assertEqual(r1.reason_phrase, "Not found")
+        self.assertEqual(r.status_code,   404)
+        self.assertEqual(r.reason_phrase, "Not found")
+        self.assertContains(r, "<p>Collection no_collection does not exist</p>", status_code=404)
         return
 
     def test_get_view_newer_version(self):
@@ -439,14 +483,9 @@ class CollectionEditViewTest(AnnalistTestCase):
         self.testsite.add_collection("newer_version", collmeta, annal_ver="99.99.99")
         u = collection_view_url(coll_id="newer_version")
         r = self.client.get(u)
-        self.assertEqual(r.reason_phrase, "FOUND")
-        self.assertEqual(r.content,       "")
-        u1 = TestHostUri + entitydata_list_all_url(coll_id="newer_version")
-        self.assertEqual(r['location'], u1)
-        r1 = self.client.get(u1)
-        self.assertEqual(r1.status_code,   500)
-        self.assertEqual(r1.reason_phrase, "Server error")
-        self.assertContains(r1, "created by software version 99.99.99", status_code=500)
+        self.assertEqual(r.status_code,   500)
+        self.assertEqual(r.reason_phrase, "Server error")
+        self.assertContains(r, "created by software version 99.99.99", status_code=500)
         return
 
     def test_get_edit(self):
@@ -454,7 +493,10 @@ class CollectionEditViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
         self.assertContains(r, "<title>Collection coll1</title>")
-        self.assertContains(r, "<h3>Customize collection coll1</h3>")
+        self.assertContains(r, 
+            '<h2 class="page-heading">Customize collection: Collection coll1</h2>', 
+            html=True
+            )
         return
 
     def test_get_edit_no_collection(self):
@@ -539,8 +581,12 @@ class CollectionEditViewTest(AnnalistTestCase):
         return
 
     def test_post_delete_type(self):
+        # Create empty type
+        empty_type_values = recordtype_create_values("coll1", "type_empty")
+        empty_type        = self.coll1.add_type("type_empty", empty_type_values)
+        # Now try to delete it
         form_data = (
-            { "typelist":   "type1"
+            { "typelist":   "type_empty"
             , "type_delete":  "Delete"
             })
         r = self.client.post(self.edit_url, form_data)
@@ -554,7 +600,20 @@ class CollectionEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['cancel_action'], self.edit_url)
         action_params = json.loads(r.context['action_params'])
         self.assertEqual(action_params['type_delete'], ["Delete"])
-        self.assertEqual(action_params['typelist'],    ["type1"])
+        self.assertEqual(action_params['typelist'],    ["type_empty"])
+        return
+
+    def test_post_delete_type_with_values(self):
+        form_data = (
+            { "typelist":   "type1"
+            , "type_delete":  "Delete"
+            })
+        r = self.client.post(self.edit_url, form_data)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertIn(self.edit_url+"?",                   r['location'])
+        self.assertIn("error_head=Problem%20with%20input", r['location'])
+        self.assertIn("error_message=",                    r['location'])
         return
 
     def test_post_delete_type_no_selection(self):
@@ -565,8 +624,77 @@ class CollectionEditViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        erroruri = self.edit_url+r"\?error_head=.*\&error_message=.*"
-        self.assertMatch(r['location'], TestHostUri+erroruri)
+        self.assertIn(self.edit_url+"?",                   r['location'])
+        self.assertIn("error_head=Problem%20with%20input", r['location'])
+        self.assertIn("error_message=",                    r['location'])
+        return
+
+    def test_post_delete_list(self):
+        # Try to delete list
+        form_data = (
+            { "listlist":   "list1"
+            , "list_delete":  "Delete"
+            })
+        r = self.client.post(self.edit_url, form_data)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
+        self.assertTemplateUsed(r, "annalist_confirm.html")
+        # Check confirmation form content
+        confirmed_action_uri = recordlist_edit_url("delete", "coll1")
+        self.assertContains(r, '''<form method="POST" action="'''+TestBasePath+'''/confirm/">''', status_code=200)
+        self.assertEqual(r.context['confirmed_action'], confirmed_action_uri)
+        self.assertEqual(r.context['cancel_action'], self.edit_url)
+        action_params = json.loads(r.context['action_params'])
+        self.assertEqual(action_params['list_delete'], ["Delete"])
+        self.assertEqual(action_params['listlist'],    ["list1"])
+        return
+
+    def test_post_delete_view(self):
+        # Try to delete view
+        form_data = (
+            { "viewlist":   "view1"
+            , "view_delete":  "Delete"
+            })
+        r = self.client.post(self.edit_url, form_data)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
+        self.assertTemplateUsed(r, "annalist_confirm.html")
+        # Check confirmation form content
+        confirmed_action_uri = recordview_edit_url("delete", "coll1")
+        self.assertContains(r, '''<form method="POST" action="'''+TestBasePath+'''/confirm/">''', status_code=200)
+        self.assertEqual(r.context['confirmed_action'], confirmed_action_uri)
+        self.assertEqual(r.context['cancel_action'], self.edit_url)
+        action_params = json.loads(r.context['action_params'])
+        self.assertEqual(action_params['view_delete'], ["Delete"])
+        self.assertEqual(action_params['viewlist'],    ["view1"])
+        return
+
+    def test_post_migrate(self):
+        form_data = (
+            { "migrate":  "Migrate data"
+            })
+        r = self.client.post(self.edit_url, form_data)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertEqual(r.content,       "")
+        l1 = TestHostUri+self.edit_url
+        l2 = "info_head=Action%20completed"
+        l3 = "info_message=Migrated%20data%20for%20collection%20coll1"
+        self.assertIn(l1, r['location'])
+        self.assertIn(l2, r['location'])
+        self.assertIn(l3, r['location'])
+        return
+
+    def test_post_metadata(self):
+        form_data = (
+            { "metadata":  "Collection metadata"
+            })
+        r = self.client.post(self.edit_url, form_data)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertEqual(r.content,       "")
+        metadataurl = collectiondata_view_url(coll_id="coll1", action="edit")
+        self.assertEqual(r['location'], TestHostUri+metadataurl+self.continuation)
         return
 
     def test_post_close(self):
