@@ -26,8 +26,6 @@ from annalist.models.recordlist     import RecordList
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.defaultedit     import EntityDefaultEditView
-
 from AnnalistTestCase       import AnnalistTestCase
 from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
 from tests                  import test_layout
@@ -93,8 +91,8 @@ def coll123_create_data(site):
 def collbib_create_data(site):
     bibcoll = Site.initialize_bib_data(site, 
         settings.SITE_SRC_ROOT+"/annalist/data/bibdata",
-        label="Bibliographic definitions", 
-        description="Bibliographic definitions for testing"
+        # label="Bibliographic definitions", 
+        # description="Bibliographic definitions for testing"
         )
     return
 
@@ -136,11 +134,13 @@ class CreateSiteData(AnnalistTestCase):
         coll123_create_data(testsite)
         return
 
-    def test_CreateBibTestSiteData(self):
-        testsite = site_create_data(TestBaseUri, "bibtestinit")
-        collbib_create_data(testsite)
-        coll123_create_data(testsite)
-        return
+    #@@
+    # def test_CreateBibTestSiteData(self):
+    #     testsite = site_create_data(TestBaseUri, "bibtestinit")
+    #     collbib_create_data(testsite)
+    #     coll123_create_data(testsite)
+    #     return
+    #@@
 
     def test_CreateEmptySiteData(self):
         emptysite = site_create_data(TestBaseUri, "empty")
