@@ -32,11 +32,13 @@ from init_tests             import init_annalist_test_site, init_annalist_test_c
 
 class TestEntityRootType(EntityRoot):
 
-    _entitytype = "test:EntityRootType"
-    _entitypath = None
-    _entityfile = ".sub/manifest.jsonld"
-    _entityref  = "../"
-    _contextref = "../../coll_context.jsonld"
+    _entitytype   = "test:EntityRootType"
+    _entitypath   = None
+    _entitybase   = ".sub/"
+    _entityfile   = "manifest.jsonld"
+    _entityref    = "../"
+    _contextbase  = "testrootbase/"
+    _contextref   = "../../coll_context.jsonld"
 
 
 class EntityRootTest(TestCase):
@@ -53,7 +55,7 @@ class EntityRootTest(TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     def test_EntityRootTest(self):
@@ -162,7 +164,8 @@ class EntityRootTest(TestCase):
     def test_entityroot_subclass(self):
         e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e._entitytype,     "test:EntityRootType")
-        self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
+        self.assertEqual(e._entitybase,     ".sub/")
+        self.assertEqual(e._entityfile,     "manifest.jsonld")
         self.assertEqual(e._entityref,      "../")
         self.assertEqual(e._entityid,       None)
         self.assertEqual(e._entityurl,      TestBaseUri+"/")
@@ -175,13 +178,11 @@ class EntityRootTest(TestCase):
         test_values = (
             { 'type':   'annal:EntityRoot'
             , 'title':  'Name collection coll1'
-            # , 'uri':    '/annalist/coll1'
             })
         test_values_returned = (
             { '@id':            '../'
             , '@type':          ['test:EntityRootType']
-            # , '@base':          "../.."
-            , '@context':       ["../../coll_context.jsonld"]
+            , '@context':       [{"@base": "testrootbase/"}, "../../coll_context.jsonld"]
             , 'annal:id':       'testId'
             , 'annal:type_id':  None
             , 'annal:type':     'test:EntityRootType'
@@ -192,7 +193,8 @@ class EntityRootTest(TestCase):
         e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e.set_id("testId")
         self.assertEqual(e._entitytype,     "test:EntityRootType")
-        self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
+        self.assertEqual(e._entitybase,     ".sub/")
+        self.assertEqual(e._entityfile,     "manifest.jsonld")
         self.assertEqual(e._entityref,      "../")
         self.assertEqual(e._entityid,       "testId")
         self.assertEqual(e._entityurl,      TestBaseUri+"/")
@@ -273,21 +275,25 @@ class EntityRootTest(TestCase):
 
 class TestEntityType(Entity):
 
-    _entitytype = "test:EntityType"
-    _entitypath = None
-    _entityview = "%(id)s/"
-    _entityfile = ".sub/manifest.jsonld"
-    _entityref  = "../"
-    _contextref = "../../coll_context.jsonld"
+    _entitytype   = "test:EntityType"
+    _entityroot   = None
+    _entityview   = "%(id)s/"
+    _entitybase   = ".sub/"
+    _entityfile   = "manifest.jsonld"
+    _entityref    = "../"
+    _contextbase  = "testtypebase/"
+    _contextref   = "../../coll_context.jsonld"
 
 class TestEntityTypeSub(Entity):
 
-    _entitytype = "test:EntityTypeSub"
-    _entitypath = "sub/%(id)s"
-    _entityview = "sub/%(id)s/"
-    _entityfile = ".sub/manifest.jsonld"
-    _entityref  = "../"
-    _contextref = "../../coll_context.jsonld"
+    _entitytype   = "test:EntityTypeSub"
+    _entityroot   = "sub/%(id)s"
+    _entityview   = "sub/%(id)s/"
+    _entitybase   = ".sub/"
+    _entityfile   = "manifest.jsonld"
+    _entityref    = "../"
+    _contextbase  = "testtypesubbase/"
+    _contextref   = "../../coll_context.jsonld"
 
 class EntityTest(AnnalistTestCase):
     """
@@ -303,7 +309,7 @@ class EntityTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     #   ----------------------------------------
@@ -340,7 +346,8 @@ class EntityTest(AnnalistTestCase):
     def values_reloaded(self, 
             entity_id='testid', entity_type='test:EntityType', 
             entity_title=None, 
-            entity_parent_path=""
+            entity_parent_path="",
+            entity_base_url="testtypebase/"
             ):
         vals = self.values_returned(
             entity_id=entity_id, entity_type=entity_type, 
@@ -350,8 +357,7 @@ class EntityTest(AnnalistTestCase):
         vals.update(
             { '@id':        '../'   # Local testing only: see `TestEntityType`
             , '@type':      [entity_type]
-            # , '@base':      "../.."
-            , '@context':   ["../../coll_context.jsonld"]
+            , '@context':   [{"@base": entity_base_url}, "../../coll_context.jsonld"]
             })
         return vals
 
@@ -376,6 +382,11 @@ class EntityTest(AnnalistTestCase):
         self.assertEqual(e._entityviewurl,  TestBaseUri+"/testid/")
         self.assertEqual(e._entitydir,      TestBaseDir+"/testid/")
         self.assertEqual(e._values,         None)
+        self.assertEqual(e.get_root_url(),  TestBaseUri+"/testid")
+        self.assertEqual(e.get_view_url(),  TestBaseUri+"/testid/")
+        self.assertEqual(e.get_base_url(),  TestBaseUri+"/testid/")
+        self.assertEqual(e.get_data_url(),  None)
+        self.assertEqual(e.get_data_url("testfile.jsonld"),  TestBaseUri+"/testid/testfile.jsonld")
         return
 
     def test_entity_type_id(self):
@@ -391,13 +402,18 @@ class EntityTest(AnnalistTestCase):
         r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityType(r, "testid")
         self.assertEqual(e._entitytype,     "test:EntityType")
-        self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
+        self.assertEqual(e._entityfile,     "manifest.jsonld")
         self.assertEqual(e._entityref,      "../")
         self.assertEqual(e._entityid,       "testid")
         self.assertEqual(e._entityurl,      TestBaseUri+"/testid/")
         self.assertEqual(e._entitydir,      TestBaseDir+"/testid/")
         self.assertEqual(e._values,         None)
         self.assertEqual(e._dir_path(),     (TestBaseDir+"/testid/.sub", TestBaseDir+"/testid/.sub/manifest.jsonld"))
+        self.assertEqual(e.get_root_url(),  None)
+        self.assertEqual(e.get_view_url(),  TestBaseUri+"/testid/")
+        self.assertEqual(e.get_base_url(),  TestBaseUri+"/testid/.sub/")
+        self.assertEqual(e.get_data_url(),  "http://test.example.com/testsite/testid/.sub/manifest.jsonld")
+        self.assertEqual(e.get_data_url("testfile.jsonld"),  TestBaseUri+"/testid/.sub/testfile.jsonld")
         return
 
     def test_entity_dict(self):
@@ -537,7 +553,8 @@ class EntityTest(AnnalistTestCase):
         test_values = self.values_created(entity_type='test:EntityTypeSub', entity_title='Name entity test2')
         test_values_returned = self.values_reloaded(
             entity_id='testid2', entity_type='test:EntityTypeSub', entity_title='Name entity test2',
-            entity_parent_path="/sub"
+            entity_parent_path="/sub",
+            entity_base_url="testtypesubbase/"
             )
 
         r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
@@ -565,11 +582,11 @@ class EntityTest(AnnalistTestCase):
         self.assertEqual(eid, "00000001")
         self.assertFalse(TestEntityType.exists(r, eid))
         eid = TestEntityType.allocate_new_id(r)
-        self.assertEqual(eid, "00000001")
+        self.assertEqual(eid, "00000002")
         e = TestEntityType.create(r, eid, test_values)
         self.assertTrue(TestEntityType.exists(r, eid))
         eid = TestEntityType.allocate_new_id(r)
-        self.assertEqual(eid, "00000002")
+        self.assertEqual(eid, "00000003")
         return
 
 # End.
