@@ -21,25 +21,30 @@ from django.test.client             import Client
 
 from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist                       import layout
+from annalist                       import message
+
 from annalist.models.site           import Site
 from annalist.models.collection     import Collection
 from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.defaultedit             import EntityDefaultEditView
+#@@ from annalist.views.defaultedit             import EntityDefaultEditView
 from annalist.views.form_utils.fieldchoice  import FieldChoice
 
 from AnnalistTestCase       import AnnalistTestCase
 from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
 from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 from entity_testutils       import (
+    make_message, make_quoted_message,
     site_dir, collection_dir, 
     collection_edit_url,
     collection_create_values,
     site_title,
     render_select_options, render_choice_options,
-    create_test_user
+    create_test_user,
+    context_view_field,
+    context_bind_fields
     )
 from entity_testtypedata    import (
     recordtype_url,
@@ -103,7 +108,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        #@@checkme@@ resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -140,9 +145,11 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
     #   Form rendering tests
     #   -----------------------------------------------------------------------------
 
-    def test_EntityDefaultEditView(self):
-        self.assertEqual(EntityDefaultEditView.__name__, "EntityDefaultEditView", "Check EntityDefaultEditView class name")
-        return
+    #@@
+    # def test_EntityDefaultEditView(self):
+    #     self.assertEqual(EntityDefaultEditView.__name__, "EntityDefaultEditView", "Check EntityDefaultEditView class name")
+    #     return
+    #@@
 
     def test_get_form_rendering(self):
         u = entitydata_edit_url("new", "testcoll", "testtype")
@@ -150,9 +157,15 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
         self.assertContains(r, "Collection testcoll")
-        field_vals = default_fields(coll_id="testcoll", type_id="testtype", entity_id="00000001")
+        field_vals = default_fields(
+            coll_id="testcoll", type_id="testtype", entity_id="00000001",
+            tooltip1=context_view_field(r.context, 0, 0)['field_tooltip'],
+            tooltip2=context_view_field(r.context, 0, 1)['field_tooltip'],
+            tooltip3=context_view_field(r.context, 1, 0)['field_tooltip'],
+            tooltip4=context_view_field(r.context, 2, 0)['field_tooltip'],
+            )
         formrow1 = """
-              <div class="small-12 medium-6 columns">
+              <div class="small-12 medium-6 columns" title="%(tooltip1)s">
                 <div class="row view-value-row">
                   <div class="%(label_classes)s">
                       <span>Id</span>
@@ -166,7 +179,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
               </div>
             """%field_vals(width=6)
         formrow2 = ("""
-              <div class="small-12 medium-6 columns">
+              <div class="small-12 medium-6 columns" title="%(tooltip2)s">
                 <div class="row view-value-row">
                   <div class="%(label_classes)s">
                       <span>Type</span>
@@ -184,7 +197,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
             """)%field_vals(width=6)
         formrow3 = """
               <!-- editable text field -->
-              <div class="small-12 columns">
+              <div class="small-12 columns" title="%(tooltip3)s">
                 <div class="row view-value-row">
                   <div class="%(label_classes)s">
                       <span>Label</span>
@@ -198,7 +211,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
               </div>
             """%field_vals(width=12)
         formrow4 = """
-              <div class="small-12 columns">
+              <div class="small-12 columns" title="%(tooltip4)s">
                 <div class="row view-value-row">
                   <div class="%(label_classes)s">
                       <span>Comment</span>
@@ -230,80 +243,67 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "testtype")
         self.assertEqual(r.context['entity_id'],        "00000001")
-        self.assertEqual(r.context['orig_id'],          "00000001")
+        self.assertEqual(r.context['orig_id'],          None)
         self.assertEqual(r.context['action'],           "new")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
-        self.assertEqual(len(r.context['fields']), 4)
+        self.assertEqual(len(r.context['fields']), 3)
+        f0  = context_view_field(r.context, 0, 0)
+        f1  = context_view_field(r.context, 0, 1)
+        f2  = context_view_field(r.context, 1, 0)
+        f3  = context_view_field(r.context, 2, 0)
         # 1st field
-        field_id_help = (
-            "A short identifier that distinguishes this record from "+
-            "all other records of the same type in the same collection."
-            )
-        self.assertEqual(r.context['fields'][0]['field_id'],            'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'],          'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'],         'Id')
-        self.assertEqual(r.context['fields'][0]['field_help'],          field_id_help)
-        self.assertEqual(r.context['fields'][0]['field_placeholder'],   "(entity id)")
-        self.assertEqual(r.context['fields'][0]['field_property_uri'],  "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'],   "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][0]['field_target_type'],   "annal:Slug")
-        self.assertEqual(r.context['fields'][0].field_value,            "00000001")
-        self.assertEqual(r.context['fields'][0]['field_value'],         "00000001")
-        self.assertEqual(r.context['fields'][0]['entity_type_id'],      "testtype")
-        self.assertEqual(r.context['fields'][0]['options'],             self.no_options)
+        self.assertEqual(f0['field_id'],            'Entity_id')
+        self.assertEqual(f0['field_name'],          'entity_id')
+        self.assertEqual(f0['field_label'],         'Id')
+        self.assertEqual(f0['field_placeholder'],   "(entity id)")
+        self.assertEqual(f0['field_property_uri'],  "annal:id")
+        self.assertEqual(f0['field_render_type'],   "EntityId")
+        self.assertEqual(f0['field_value_mode'],    "Value_direct")
+        self.assertEqual(f0['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f0.field_value,            "00000001")
+        self.assertEqual(f0['field_value'],         "00000001")
+        self.assertEqual(f0['entity_type_id'],      "testtype")
+        self.assertEqual(f0['options'],             self.no_options)
         # 2nd field
-        field_type_help = (
-            "A short identifier that identifies the type of the corresponding entity."
-            )
-        self.assertEqual(r.context['fields'][1]['field_id'],            'Entity_type')
-        self.assertEqual(r.context['fields'][1]['field_name'],          'entity_type')
-        self.assertEqual(r.context['fields'][1]['field_label'],         'Type')
-        self.assertEqual(r.context['fields'][1]['field_help'],          field_type_help)
-        self.assertEqual(r.context['fields'][1]['field_placeholder'],   "(type id)")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'],  "annal:type_id")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],   "EntityTypeId")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Slug")
-        self.assertEqual(r.context['fields'][1].field_value,            "testtype")
-        self.assertEqual(r.context['fields'][1]['field_value'],         "testtype")
-        self.assertEqual(r.context['fields'][1]['entity_type_id'],      "testtype")
-        self.assertEqual(set(r.context['fields'][1]['options']),        set(self.type_ids))
+        self.assertEqual(f1['field_id'],            'Entity_type')
+        self.assertEqual(f1['field_name'],          'entity_type')
+        self.assertEqual(f1['field_label'],         'Type')
+        self.assertEqual(f1['field_placeholder'],   "(type id)")
+        self.assertEqual(f1['field_property_uri'],  "annal:type_id")
+        self.assertEqual(f1['field_render_type'],   "EntityTypeId")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f1.field_value,            "testtype")
+        self.assertEqual(f1['field_value'],         "testtype")
+        self.assertEqual(f1['entity_type_id'],      "testtype")
+        self.assertEqual(set(f1['options']),        set(self.type_ids))
         # 3rd field
-        field_label_help = (
-            "Short string used to describe entity when displayed"
-            )
         field_label_value = default_label("testcoll", "testtype", "00000001")
-        self.assertEqual(r.context['fields'][2]['field_id'],            'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_name'],          'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_label'],         'Label')
-        self.assertEqual(r.context['fields'][2]['field_help'],          field_label_help)
-        self.assertEqual(r.context['fields'][2]['field_placeholder'],   "(label)")
-        self.assertEqual(r.context['fields'][2]['field_property_uri'],  "rdfs:label")
-        self.assertEqual(r.context['fields'][2]['field_render_type'],   "Text")
-        self.assertEqual(r.context['fields'][2]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][2]['field_target_type'],   "annal:Text")
-        self.assertEqual(r.context['fields'][2]['field_value'],         field_label_value)
-        self.assertEqual(r.context['fields'][2]['entity_type_id'],      "testtype")
-        self.assertEqual(r.context['fields'][2]['options'],             self.no_options)
+        self.assertEqual(f2['field_id'],            'Entity_label')
+        self.assertEqual(f2['field_name'],          'Entity_label')
+        self.assertEqual(f2['field_label'],         'Label')
+        self.assertEqual(f2['field_placeholder'],   "(label)")
+        self.assertEqual(f2['field_property_uri'],  "rdfs:label")
+        self.assertEqual(f2['field_render_type'],   "Text")
+        self.assertEqual(f2['field_value_mode'],    "Value_direct")
+        self.assertEqual(f2['field_value_type'],    "annal:Text")
+        self.assertEqual(f2['field_value'],         field_label_value)
+        self.assertEqual(f2['entity_type_id'],      "testtype")
+        self.assertEqual(f2['options'],             self.no_options)
         # 4th field
-        field_comment_help = (
-            "Descriptive text about an entity."
-            )
         field_comment_value = default_comment("testcoll", "testtype", "00000001")
-        self.assertEqual(r.context['fields'][3]['field_id'],            'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_name'],          'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_label'],         'Comment')
-        self.assertEqual(r.context['fields'][3]['field_help'],          field_comment_help)
-        self.assertEqual(r.context['fields'][3]['field_placeholder'],   "(description)")
-        self.assertEqual(r.context['fields'][3]['field_property_uri'],  "rdfs:comment")
-        self.assertEqual(r.context['fields'][3]['field_render_type'],   "Markdown")
-        self.assertEqual(r.context['fields'][3]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][3]['field_target_type'],   "annal:Richtext")
-        self.assertEqual(r.context['fields'][3]['field_value'],         field_comment_value)
-        self.assertEqual(r.context['fields'][3]['entity_type_id'],      "testtype")
-        self.assertEqual(r.context['fields'][3]['options'],             self.no_options)
+        self.assertEqual(f3['field_id'],            'Entity_comment')
+        self.assertEqual(f3['field_name'],          'Entity_comment')
+        self.assertEqual(f3['field_label'],         'Comment')
+        self.assertEqual(f3['field_placeholder'],   "(description)")
+        self.assertEqual(f3['field_property_uri'],  "rdfs:comment")
+        self.assertEqual(f3['field_render_type'],   "Markdown")
+        self.assertEqual(f3['field_value_mode'],    "Value_direct")
+        self.assertEqual(f3['field_value_type'],    "annal:Richtext")
+        self.assertEqual(f3['field_value'],         field_comment_value)
+        self.assertEqual(f3['entity_type_id'],      "testtype")
+        self.assertEqual(f3['options'],             self.no_options)
         return
 
     def test_get_edit(self):
@@ -311,7 +311,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         r = self.client.get(u+"?continuation_url=/xyzzy/")
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<title>Collection testcoll</title>")
+        # log.info(r.content)
+        self.assertContains(r, "<title>Entity testcoll/testtype/entity1 - Default record view - Collection testcoll</title>")
         # Test context
         view_url = entity_url(coll_id="testcoll", type_id="testtype", entity_id="entity1")
         self.assertEqual(r.context['coll_id'],          "testcoll")
@@ -321,74 +322,61 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['action'],           "edit")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
-        self.assertEqual(len(r.context['fields']), 4)        
+        self.assertEqual(len(r.context['fields']), 3)
+        f0  = context_view_field(r.context, 0, 0)
+        f1  = context_view_field(r.context, 0, 1)
+        f2  = context_view_field(r.context, 1, 0)
+        f3  = context_view_field(r.context, 2, 0)
         # 1st field
-        field_id_help = (
-            "A short identifier that distinguishes this record from "+
-            "all other records of the same type in the same collection."
-            )
-        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
-        self.assertEqual(r.context['fields'][0]['field_help'],         field_id_help)
-        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
-        self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_value'],        "entity1")
-        self.assertEqual(r.context['fields'][0]['options'],            self.no_options)
+        self.assertEqual(f0['field_id'],            'Entity_id')
+        self.assertEqual(f0['field_name'],          'entity_id')
+        self.assertEqual(f0['field_label'],         'Id')
+        self.assertEqual(f0['field_placeholder'],   "(entity id)")
+        self.assertEqual(f0['field_property_uri'],  "annal:id")
+        self.assertEqual(f0['field_render_type'],   "EntityId")
+        self.assertEqual(f0['field_value_mode'],    "Value_direct")
+        self.assertEqual(f0['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f0['field_value'],         "entity1")
+        self.assertEqual(f0['options'],             self.no_options)
         # 2nd field
-        field_type_help = (
-            "A short identifier that identifies the type of the corresponding entity."
-            )
-        self.assertEqual(r.context['fields'][1]['field_id'],           'Entity_type')
-        self.assertEqual(r.context['fields'][1]['field_name'],         'entity_type')
-        self.assertEqual(r.context['fields'][1]['field_label'],        'Type')
-        self.assertEqual(r.context['fields'][1]['field_help'],         field_type_help)
-        self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(type id)")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "annal:type_id")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],  "EntityTypeId")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][1]['field_value'],        "testtype")
-        self.assertEqual(set(r.context['fields'][1]['options']),       set(self.type_ids))
+        self.assertEqual(f1['field_id'],            'Entity_type')
+        self.assertEqual(f1['field_name'],          'entity_type')
+        self.assertEqual(f1['field_label'],         'Type')
+        self.assertEqual(f1['field_placeholder'],   "(type id)")
+        self.assertEqual(f1['field_property_uri'],  "annal:type_id")
+        self.assertEqual(f1['field_render_type'],   "EntityTypeId")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f1['field_value'],         "testtype")
+        self.assertEqual(set(f1['options']),        set(self.type_ids))
         # 3rd field
-        field_label_help = (
-            "Short string used to describe entity when displayed"
-            )
         field_label_value = (
             "Entity testcoll/testtype/entity1"
             )
-        self.assertEqual(r.context['fields'][2]['field_id'],           'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_name'],         'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_label'],        'Label')
-        self.assertEqual(r.context['fields'][2]['field_help'],         field_label_help)
-        self.assertEqual(r.context['fields'][2]['field_placeholder'],  "(label)")
-        self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][2]['field_render_type'],  "Text")
-        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Text")
-        self.assertEqual(r.context['fields'][2]['field_value'],        field_label_value)
-        self.assertEqual(r.context['fields'][2]['options'],            self.no_options)
+        self.assertEqual(f2['field_id'],            'Entity_label')
+        self.assertEqual(f2['field_name'],          'Entity_label')
+        self.assertEqual(f2['field_label'],         'Label')
+        self.assertEqual(f2['field_placeholder'],   "(label)")
+        self.assertEqual(f2['field_property_uri'],  "rdfs:label")
+        self.assertEqual(f2['field_render_type'],   "Text")
+        self.assertEqual(f2['field_value_mode'],    "Value_direct")
+        self.assertEqual(f2['field_value_type'],    "annal:Text")
+        self.assertEqual(f2['field_value'],         field_label_value)
+        self.assertEqual(f2['options'],             self.no_options)
         # 4th field
-        field_comment_help = (
-            "Descriptive text about an entity."
-            )
         field_comment_value = (
             "Entity coll testcoll, type testtype, entity entity1"
             )
-        self.assertEqual(r.context['fields'][3]['field_id'],           'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_name'],         'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_label'],        'Comment')
-        self.assertEqual(r.context['fields'][3]['field_help'],         field_comment_help)
-        self.assertEqual(r.context['fields'][3]['field_placeholder'],  "(description)")
-        self.assertEqual(r.context['fields'][3]['field_property_uri'], "rdfs:comment")
-        self.assertEqual(r.context['fields'][3]['field_render_type'],  "Markdown")
-        self.assertEqual(r.context['fields'][3]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][3]['field_target_type'],  "annal:Richtext")
-        self.assertEqual(r.context['fields'][3]['field_value'],        field_comment_value)
-        self.assertEqual(r.context['fields'][3]['options'],            self.no_options)
+        self.assertEqual(f3['field_id'],            'Entity_comment')
+        self.assertEqual(f3['field_name'],          'Entity_comment')
+        self.assertEqual(f3['field_label'],         'Comment')
+        self.assertEqual(f3['field_placeholder'],   "(description)")
+        self.assertEqual(f3['field_property_uri'],  "rdfs:comment")
+        self.assertEqual(f3['field_render_type'],   "Markdown")
+        self.assertEqual(f3['field_value_mode'],    "Value_direct")
+        self.assertEqual(f3['field_value_type'],    "annal:Richtext")
+        self.assertEqual(f3['field_value'],         field_comment_value)
+        self.assertEqual(f3['options'],             self.no_options)
         return
 
     def test_get_edit_not_exists(self):
@@ -399,8 +387,9 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<title>Annalist error</title>", status_code=404)
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         # log.debug(r.content)
-        def_label = error_label("testcoll", "testtype", "entitynone")
-        self.assertContains(r, "<p>Entity %s does not exist</p>"%(def_label), status_code=404)
+        err_label = error_label("testcoll", "testtype", "entitynone")
+        msg_text  = make_message(message.ENTITY_DOES_NOT_EXIST, id="entitynone", label=err_label)
+        self.assertContains(r, "<p>%s</p>"%msg_text, status_code=404)
         return
 
     #   -----------------------------------------------------------------------------
@@ -444,7 +433,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>Problem with entity identifier</h3>")
         # Test context
         expect_context = entitydata_context_data(action="new")
-        self.assertDictionaryMatch(r.context, expect_context)
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         return
 
     def test_post_new_entity_invalid_id(self):
@@ -459,8 +448,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
             entity_id="!badentity", orig_id="orig_entity_id", action="new"
             )
         # log.info(repr(f))
-        # log.info(repr(r.context['fields'][1]))
-        self.assertDictionaryMatch(r.context, expect_context)
+        # log.info(repr(context_bind_fields(r.context)['fields'][1]))
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         return
 
     def test_new_entity_default_type(self):
@@ -546,7 +535,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         self.assertContains(r, "<h3>Problem with entity identifier</h3>")
         expect_context = entitydata_context_data(action="copy")
-        self.assertDictionaryMatch(r.context, expect_context)
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         return
 
     def test_post_copy_entity_invalid_id(self):
@@ -559,7 +548,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         expect_context = entitydata_context_data(
             entity_id="!badentity", orig_id="orig_entity_id", action="copy"
             )
-        self.assertDictionaryMatch(r.context, expect_context)
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         return
 
     #   -------- edit type --------
@@ -597,7 +586,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self._create_entity_data("entityedittype")
         e1 = self._check_entity_data_values("entityedittype")
         self.assertFalse(RecordType.exists(self.testcoll, "newtype"))
-        newtype = RecordType.create(self.testcoll, "newtype", recordtype_create_values("newtype"))
+        newtype     = RecordType.create(self.testcoll, "newtype", recordtype_create_values("newtype"))
         newtypedata = RecordTypeData(self.testcoll, "newtype")
         self.assertTrue(RecordType.exists(self.testcoll, "newtype"))
         self.assertFalse(RecordTypeData.exists(self.testcoll, "newtype"))
@@ -647,7 +636,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>Problem with entity identifier</h3>")
         # Test context for re-rendered form
         expect_context = entitydata_context_data(action="edit", update="Updated entity")
-        self.assertDictionaryMatch(r.context, expect_context)
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         # Check stored entity is unchanged
         self._check_entity_data_values("edittype")
         return
@@ -668,7 +657,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         expect_context = entitydata_context_data(
             entity_id="!badentity", orig_id="orig_entity_id", action="edit"
             )
-        self.assertDictionaryMatch(r.context, expect_context)
+        self.assertDictionaryMatch(context_bind_fields(r.context), expect_context)
         # Check stored entity is unchanged
         self._check_entity_data_values("edittype")
         return
