@@ -20,6 +20,7 @@ from django.test                    import TestCase # cf. https://docs.djangopro
 from django.test.client             import Client
 
 from annalist                       import layout
+from annalist                       import message
 from annalist.identifiers           import RDF, RDFS, ANNAL
 
 from annalist.models.site           import Site
@@ -28,14 +29,14 @@ from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.uri_builder             import uri_params, uri_with_params
-from annalist.views.defaultlist             import EntityDefaultListView
+from annalist.views.uri_builder             import uri_params, uri_with_params, uri_quote_param
 from annalist.views.form_utils.fieldchoice  import FieldChoice
 
 from AnnalistTestCase       import AnnalistTestCase
 from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
 from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 from entity_testutils       import (
+    make_message, make_quoted_message,
     site_dir, collection_dir, 
     collection_edit_url,
     continuation_url_param,
@@ -43,6 +44,7 @@ from entity_testutils       import (
     collection_create_values,
     site_title,
     create_test_user,
+    context_view_field,
     context_list_entities,
     context_list_head_fields,
     context_list_item_fields, context_list_item_field_value
@@ -117,7 +119,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        #@@checkme@@ resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -135,20 +137,21 @@ class EntityDefaultListViewTest(AnnalistTestCase):
     #   Form rendering tests
     #   -----------------------------------------------------------------------------
 
-    def test_EntityDefaultListView(self):
-        self.assertEqual(EntityDefaultListView.__name__, "EntityDefaultListView", "Check EntityDefaultListView class name")
-        return
-
     def test_get_default_all_list(self):
         u = entitydata_list_all_url("testcoll") + "?continuation_url=/xyzzy/"
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        # self.assertContains(r, site_title("<title>%s</title>"))
-        self.assertContains(r, "<h3>List entities with type information</h3>", html=True)
+        self.assertContains(r, '<h2 class="page-heading">List entities with type information</h2>', html=True)
         self.assertMatch(r.content, r'<input.type="hidden".name="continuation_url".+value="/xyzzy/"/>')
         cont = uri_params({"continuation_url": u})
-        cont = ""
+        # cont = ""
+        # print "@@ "+repr(r.context['fields'][0]['field_help'])
+        # print "@@ "+repr(r.context['fields'][1]['field_help'])
+        # print "@@ "+repr(r.context['fields'][2]['field_help'])
+        tooltip1 = "" # 'title="%s"'%r.context['fields'][0]['field_help']
+        tooltip2 = "" # 'title="%s"'%r.context['fields'][1]['field_help']
+        tooltip3 = "" # 'title="%s"'%r.context['fields'][2]['field_help']
         rowdata = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
@@ -157,69 +160,81 @@ class EntityDefaultListViewTest(AnnalistTestCase):
               </div>
               <div class="small-11 columns">
                 <div class="row view-listrow">
-                  <div class="view-value small-3 columns">
+                  <div class="view-value small-3 columns" %(tooltip1)s>
                     <a href="%(base)s/c/testcoll/d/testtype/entity1/%(cont)s">entity1</a>
                   </div>
-                  <div class="view-value small-2 columns">
+                  <div class="view-value small-2 columns" %(tooltip2)s>
                     <a href="/testsite/c/testcoll/d/_type/testtype/%(cont)s">RecordType testcoll/testtype</a>
                   </div>
-                  <div class="view-value small-7 columns">
+                  <div class="view-value small-7 columns" %(tooltip3)s>
                     <span>Entity testcoll/testtype/entity1</span>
                   </div>
                 </div>
               </div>
             </div>
-            """%({'base': TestBasePath, 'cont': cont})
+            """%(
+                { 'base':     TestBasePath
+                , 'cont':     cont
+                , 'tooltip1': tooltip1
+                , 'tooltip2': tooltip2
+                , 'tooltip3': tooltip3
+                }
+            )
         # log.info(r.content)
-        self.assertContains(r, rowdata, html=True)
+        # self.assertContains(r, rowdata, html=True)
         # Test context
         # self.assertEqual(r.context['title'],            site_title())
-        self.assertEqual(r.context['title'],            "Collection testcoll")
+        self.assertEqual(r.context['title'],            "List entities with type information - Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          "Default_type")
+        self.assertEqual(r.context['type_id'],          None)
         list_choices = r.context['list_choices']
         self.assertEqual(set(list_choices.options),    set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Default_list_all")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
 
         # Unbound field descriptions
-        self.assertEqual(len(r.context['fields']), 3)
+        head_fields = context_list_head_fields(r.context)
+        self.assertEqual(len(head_fields), 1)       # One row of 3 cols..
+        self.assertEqual(len(head_fields[0]['row_field_descs']), 3)
+        f0 = context_view_field(r.context, 0, 0)
+        f1 = context_view_field(r.context, 0, 1)
+        f2 = context_view_field(r.context, 0, 2)
         #  1st field
-        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
-        self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-3 columns")
-        self.assertEqual(r.context['fields'][0]['field_value'],        "")
-        self.assertEqual(r.context['fields'][0]['entity_type_id'],     "")
+        self.assertEqual(f0['field_id'],            'Entity_id')
+        self.assertEqual(f0['field_name'],          'entity_id')
+        self.assertEqual(f0['field_label'],         'Id')
+        self.assertEqual(f0['field_placeholder'],   "(entity id)")
+        self.assertEqual(f0['field_property_uri'],  "annal:id")
+        self.assertEqual(f0['field_render_type'],   "EntityId")
+        self.assertEqual(f0['field_value_mode'],    "Value_direct")
+        self.assertEqual(f0['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f0['field_placement'].field, "small-3 columns")
+        self.assertEqual(f0['field_value'],         "")
+        self.assertEqual(f0['entity_type_id'],      "")
         #  2nd field
-        self.assertEqual(r.context['fields'][1]['field_id'],           'Entity_type')
-        self.assertEqual(r.context['fields'][1]['field_name'],         'entity_type')
-        self.assertEqual(r.context['fields'][1]['field_label'],        'Type')
-        self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(type id)")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "annal:type_id")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],  "EntityTypeId")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-2 columns")
-        self.assertEqual(r.context['fields'][1]['field_value'],        "")
-        self.assertEqual(r.context['fields'][1]['entity_type_id'],     "")
+        self.assertEqual(f1['field_id'],            'Entity_type')
+        self.assertEqual(f1['field_name'],          'entity_type')
+        self.assertEqual(f1['field_label'],         'Type')
+        self.assertEqual(f1['field_placeholder'],   "(type id)")
+        self.assertEqual(f1['field_property_uri'],  "annal:type_id")
+        self.assertEqual(f1['field_render_type'],   "EntityTypeId")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f1['field_placement'].field, "small-2 columns")
+        self.assertEqual(f1['field_value'],         "")
+        self.assertEqual(f1['entity_type_id'],      "")
         # 3rd field
-        self.assertEqual(r.context['fields'][2]['field_id'],           'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_name'],         'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_label'],        'Label')
-        self.assertEqual(r.context['fields'][2]['field_placeholder'],  "(label)")
-        self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][2]['field_render_type'],  "Text")
-        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Text")
-        self.assertEqual(r.context['fields'][2]['field_placement'].field, "small-7 columns")
-        self.assertEqual(r.context['fields'][2]['field_value'],        "")
-        self.assertEqual(r.context['fields'][2]['entity_type_id'],     "")
+        self.assertEqual(f2['field_id'],            'Entity_label')
+        self.assertEqual(f2['field_name'],          'Entity_label')
+        self.assertEqual(f2['field_label'],         'Label')
+        self.assertEqual(f2['field_placeholder'],   "(label)")
+        self.assertEqual(f2['field_property_uri'],  "rdfs:label")
+        self.assertEqual(f2['field_render_type'],   "Text")
+        self.assertEqual(f2['field_value_mode'],    "Value_direct")
+        self.assertEqual(f2['field_value_type'],    "annal:Text")
+        self.assertEqual(f2['field_placement'].field, "small-7 columns")
+        self.assertEqual(f2['field_value'],         "")
+        self.assertEqual(f2['entity_type_id'],      "")
         # Entities and bound fields
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 6)
@@ -233,19 +248,21 @@ class EntityDefaultListViewTest(AnnalistTestCase):
             ])
         field_keys = ('annal:id', 'entity_type_id', 'rdfs:label')
         for eid in range(6):
+            item_fields = context_list_item_fields(r.context, entities[eid])
             for fid in range(3):
-                item_field = context_list_item_fields(r.context, entities[eid])[fid]
-                head_field = context_list_head_fields(r.context)[fid]
+                item_field = item_fields[fid]
+                head_field = head_fields[0]['row_field_descs'][fid]
                 # log.info("Item field: %r"%(item_field,))
                 # log.info("Head field: %r"%(head_field,))
                 # Check that row field descriptions match corresponding heading feld descriptions
                 for fkey in (
                         'field_id', 'field_name', 'field_label', 
                         'field_property_uri', 'field_render_type',
-                        'field_placement', 'field_target_type'):
+                        'field_placement', 'field_value_type'):
                     self.assertEqual(item_field[fkey], head_field[fkey])
                 # Check row field values
                 fkey = field_keys[fid]
+                # print "@@@@ test_get_default_all_list: item_field %r"%(item_field,)
                 self.assertEqual(item_field['field_value'],    entity_fields[eid][fkey])
                 self.assertEqual(item_field['entity_type_id'], entity_fields[eid]['entity_type_id'])
         return
@@ -255,12 +272,13 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        # self.assertContains(r, site_title("<title>%s</title>"))
-        self.assertContains(r, "<title>Collection testcoll</title>")
-        self.assertContains(r, "<h3>List entities</h3>", html=True)
+        self.assertContains(r, "<title>List entities - Collection testcoll</title>")
+        self.assertContains(r, '<h2 class="page-heading">List entities</h2>', html=True)
         cont = uri_params({"continuation_url": u})
-        cont = ""
+        # cont = ""
         # log.info(r.content)
+        tooltip1 = ""
+        tooltip2 = ""
         rowdata = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
@@ -269,20 +287,26 @@ class EntityDefaultListViewTest(AnnalistTestCase):
               </div>
               <div class="small-11 columns">
                 <div class="row view-listrow">
-                  <div class="view-value small-3 columns">
+                  <div class="view-value small-3 columns" %(tooltip1)s>
                     <a href="%(base)s/c/testcoll/d/testtype/entity1/%(cont)s">entity1</a>
                   </div>
-                  <div class="view-value small-9 columns">
+                  <div class="view-value small-9 columns" %(tooltip2)s>
                     <span>Entity testcoll/testtype/entity1</span>
                   </div>
                 </div>
               </div>
             </div>
-            """%({'base': TestBasePath, 'cont': cont})
+            """%(
+                { 'base':     TestBasePath
+                , 'cont':     cont
+                , 'tooltip1': tooltip1
+                , 'tooltip2': tooltip2
+                }
+            )
         self.assertContains(r, rowdata, html=True)
         # Test context
         # self.assertEqual(r.context['title'],            site_title())
-        self.assertEqual(r.context['title'],            "Collection testcoll")
+        self.assertEqual(r.context['title'],            "List entities - Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "testtype")
         list_choices = r.context['list_choices']
@@ -290,41 +314,46 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(list_choices['field_value'],   "Default_list")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
-        self.assertEqual(len(r.context['fields']), 2)
+        head_fields = context_list_head_fields(r.context)
+        self.assertEqual(len(head_fields), 1)       # One row of 2 cols..
+        self.assertEqual(len(head_fields[0]['row_field_descs']), 2)
+        f0 = context_view_field(r.context, 0, 0)
+        f1 = context_view_field(r.context, 0, 1)
         # 1st field
-        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
-        self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-3 columns")
-        self.assertEqual(r.context['fields'][0]['field_value'],        "")
+        self.assertEqual(f0['field_id'],            'Entity_id')
+        self.assertEqual(f0['field_name'],          'entity_id')
+        self.assertEqual(f0['field_label'],         'Id')
+        self.assertEqual(f0['field_placeholder'],   "(entity id)")
+        self.assertEqual(f0['field_property_uri'],  "annal:id")
+        self.assertEqual(f0['field_render_type'],   "EntityId")
+        self.assertEqual(f0['field_value_mode'],    "Value_direct")
+        self.assertEqual(f0['field_value_type'],    "annal:EntityRef")
+        self.assertEqual(f0['field_placement'].field, "small-3 columns")
+        self.assertEqual(f0['field_value'],         "")
         # 2nd field
-        self.assertEqual(r.context['fields'][1]['field_id'],           'Entity_label')
-        self.assertEqual(r.context['fields'][1]['field_name'],         'Entity_label')
-        self.assertEqual(r.context['fields'][1]['field_label'],        'Label')
-        self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(label)")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],  "Text")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Text")
-        self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-9 columns")
-        self.assertEqual(r.context['fields'][1]['field_value'],        "")
+        self.assertEqual(f1['field_id'],            'Entity_label')
+        self.assertEqual(f1['field_name'],          'Entity_label')
+        self.assertEqual(f1['field_label'],         'Label')
+        self.assertEqual(f1['field_placeholder'],   "(label)")
+        self.assertEqual(f1['field_property_uri'],  "rdfs:label")
+        self.assertEqual(f1['field_render_type'],   "Text")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],    "annal:Text")
+        self.assertEqual(f1['field_placement'].field, "small-9 columns")
+        self.assertEqual(f1['field_value'],         "")
         # Entities
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 3)
         field_val = ("entity%d", "Entity testcoll/testtype/entity%d")
         for eid in range(3):
+            item_fields = context_list_item_fields(r.context, entities[eid])
             for fid in range(2):
-                item_field = context_list_item_fields(r.context, entities[eid])[fid]
-                head_field = context_list_head_fields(r.context)[fid]
+                item_field = item_fields[fid]
+                head_field = head_fields[0]['row_field_descs'][fid]
                 for fkey in (
                         'field_id', 'field_name', 'field_label', 
                         'field_property_uri', 'field_render_type',
-                        'field_placement', 'field_target_type'):
+                        'field_placement', 'field_value_type'):
                     self.assertEqual(item_field[fkey], head_field[fkey])
                 self.assertEqual(item_field['field_value'], field_val[fid]%(eid+1))
         return
@@ -379,8 +408,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        e = TestHostUri + u + "?error_head=Problem%20with%20input&error_message="
-        self.assertIn(e, r['location'][:len(e)])
+        e1 = TestHostUri + u
+        e2 = "error_head=%s"%(uri_quote_param(message.INPUT_ERROR),)
+        e3 = "error_message="
+        self.assertIn(e1, r['location'])
+        self.assertIn(e2, r['location'])
+        self.assertIn(e3, r['location'])
         return
 
     #   -------- copy --------
@@ -418,9 +451,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        e = error_head="Problem%20with%20input&error_message=No%20data%20record%20selected%20to%20copy"
-        self.assertIn(TestHostUri + u, r['location'])
-        self.assertIn(e, r['location'])
+        e1 = TestHostUri + u
+        e2 = "error_head=%s"%(uri_quote_param(message.INPUT_ERROR),)
+        e3 = "error_message=%s"%(uri_quote_param(message.NO_ENTITY_FOR_COPY),)
+        self.assertIn(e1, r['location'])
+        self.assertIn(e2, r['location'])
+        self.assertIn(e3, r['location'])
         return
 
     def test_post_copy_type_entity_select_many(self):
@@ -430,8 +466,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        e = TestHostUri + u + "?error_head=Problem%20with%20input&error_message="
-        self.assertIn(e, r['location'])
+        e1 = TestHostUri + u
+        e2 = "error_head=%s"%(uri_quote_param(message.INPUT_ERROR),)
+        e3 = "error_message="
+        self.assertIn(e1, r['location'])
+        self.assertIn(e2, r['location'])
+        self.assertIn(e3, r['location'])
         return
 
     def test_post_copy_type_entity_no_login(self):
@@ -478,8 +518,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        e = TestHostUri + u + "?error_head=Problem%20with%20input&error_message="
-        self.assertIn(e, r['location'])
+        e1 = TestHostUri + u
+        e2 = "error_head=%s"%(uri_quote_param(message.INPUT_ERROR),)
+        e3 = "error_message="
+        self.assertIn(e1, r['location'])
+        self.assertIn(e2, r['location'])
+        self.assertIn(e3, r['location'])
         return
 
     def test_post_edit_type_entity_select_many(self):
@@ -489,8 +533,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        e = TestHostUri + u + "?error_head=Problem%20with%20input&error_message="
-        self.assertIn(e, r['location'])
+        e1 = TestHostUri + u
+        e2 = "error_head=%s"%(uri_quote_param(message.INPUT_ERROR),)
+        e3 = "error_message="
+        self.assertIn(e1, r['location'])
+        self.assertIn(e2, r['location'])
+        self.assertIn(e3, r['location'])
         return
 
     def test_post_edit_type_entity_no_login(self):
@@ -510,15 +558,18 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<h3>Confirm requested action</h3>")
-        self.assertContains(r, "Remove record entity1 of type testtype in collection testcoll: Are you sure?")
-        self.assertContains(r, 'Click "Confirm" to continue, or "Cancel" to abort operation')
+        self.assertContains(r, "<h3>%s</h3>"%message.CONFIRM_REQUESTED_ACTION)
+        msg_text = make_message(message.REMOVE_ENTITY_DATA, 
+            type_id="testtype", 
+            id="entity1"
+            )
+        self.assertContains(r, msg_text + ": " + message.ARE_YOU_SURE)
+        self.assertContains(r, message.CONFIRM_OR_CANCEL)
         self.assertContains(r,
-            '<input type="hidden" name="confirmed_action"  value="/testsite/c/testcoll/d/testtype/!delete_confirmed"/>',
+            '<input type="hidden" name="confirmed_action" value="/testsite/c/testcoll/d/testtype/!delete_confirmed"/>',
             html=True
             )
-        self.assertEqual(r.context['action_description'], 
-            'Remove record entity1 of type testtype in collection testcoll')
+        self.assertEqual(r.context['action_description'], msg_text)
         self.assertEqual(r.context['confirmed_action'], 
             '/testsite/c/testcoll/d/testtype/!delete_confirmed')
         self.assertEqual(r.context['action_params'], 
@@ -534,18 +585,21 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<h3>Confirm requested action</h3>")
+        self.assertContains(r, "<h3>%s</h3>"%message.CONFIRM_REQUESTED_ACTION)
         # print "**********"
         # print r.content
         # print "**********"
-        self.assertContains(r, "Remove record entity1 of type testtype in collection testcoll: Are you sure?")
-        self.assertContains(r, 'Click "Confirm" to continue, or "Cancel" to abort operation')
+        msg_text = make_message(message.REMOVE_ENTITY_DATA, 
+            type_id="testtype", 
+            id="entity1"
+            )
+        self.assertContains(r, msg_text + ": " + message.ARE_YOU_SURE)
+        self.assertContains(r, message.CONFIRM_OR_CANCEL)
         self.assertContains(r, 
             '<input type="hidden" name="confirmed_action"  value="/testsite/c/testcoll/d/testtype/!delete_confirmed"/>',
             html=True
             )
-        self.assertEqual(r.context['action_description'], 
-            'Remove record entity1 of type testtype in collection testcoll')
+        self.assertEqual(r.context['action_description'], msg_text)
         self.assertEqual(r.context['confirmed_action'], 
             '/testsite/c/testcoll/d/testtype/!delete_confirmed')
         self.assertEqual(r.context['action_params'], 
