@@ -58,7 +58,7 @@ class ConfirmEntityDataDeleteTests(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections") #@@checkme@@
         return
 
     def test_CollectionActionViewTest(self):
@@ -82,7 +82,10 @@ class ConfirmEntityDataDeleteTests(AnnalistTestCase):
             entitydata_list_all_url("testcoll")
             )
         self.assertMatch(r['location'],    
-            r"info_head=.*&info_message=.*deleteentity.*testcoll.*$"
+            r"info_head=.*$"
+            )
+        self.assertMatch(r['location'],    
+            r"info_message=.*deleteentity.*testcoll.*$"
             )
         self.assertNotIn("search=testcoll", r['location'])
         # Confirm deletion
@@ -104,7 +107,10 @@ class ConfirmEntityDataDeleteTests(AnnalistTestCase):
             entitydata_list_all_url("testcoll")
             )
         self.assertMatch(r['location'],    
-            r"info_head=.*&info_message=.*deleteentity.*testcoll.*$"
+            r"info_head=.*$"
+            )
+        self.assertMatch(r['location'],    
+            r"info_message=.*deleteentity.*testcoll.*$"
             )
         self.assertIn("search=testcoll", r['location'])
         # Confirm deletion
