@@ -23,6 +23,7 @@ from utils.SuppressLoggingContext   import SuppressLogging
 
 from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist                       import layout
+from annalist                       import message
 
 from annalist.models.entitytypeinfo import EntityTypeInfo
 from annalist.models.site           import Site
@@ -41,15 +42,20 @@ from AnnalistTestCase       import AnnalistTestCase
 from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
 from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 from entity_testutils       import (
+    make_message, make_quoted_message,
     collection_create_values,
     site_dir, collection_dir, 
     continuation_url_param,
+    collection_view_url,
     collection_edit_url,
     collection_entity_view_url,
     site_title,
     render_select_options,
     render_choice_options,
-    create_test_user
+    create_test_user,
+    context_view_field,
+    check_default_view_context_fields,
+    check_type_view_context_fields
     )
 from entity_testtypedata    import (
     recordtype_dir, 
@@ -62,7 +68,7 @@ from entity_testentitydata  import (
     entitydata_list_type_url,
     entitydata_value_keys, entitydata_create_values, entitydata_values,
     entitydata_delete_confirm_form_data,
-    entitydata_default_view_context_data, entitydata_default_view_form_data,
+    entitydata_default_view_form_data,
     entitydata_recordtype_view_context_data, entitydata_recordtype_view_form_data,
     default_fields, default_label, default_comment, error_label,
     layout_classes
@@ -112,7 +118,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     #   -----------------------------------------------------------------------------
@@ -129,7 +135,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
     def _check_entity_data_values(self, entity_id, type_id="testtype", update="Entity", update_dict=None):
         "Helper function checks content of form-updated record type entry with supplied entity_id"
         # log.info("_check_entity_data_values: type_id %s, entity_id %s"%(type_id, entity_id))
-        typeinfo = EntityTypeInfo(self.testsite, self.testcoll, type_id)
+        typeinfo = EntityTypeInfo(self.testcoll, type_id)
         self.assertTrue(typeinfo.entityclass.exists(typeinfo.entityparent, entity_id))
         e = typeinfo.entityclass.load(typeinfo.entityparent, entity_id)
         self.assertEqual(e.get_id(), entity_id)
@@ -145,7 +151,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         return e
 
     #   -----------------------------------------------------------------------------
-    #   Form rendering tests
+    #   Form rendering and response tests
     #   -----------------------------------------------------------------------------
 
     def test_get_default_form_no_login(self):
@@ -156,6 +162,66 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         return
 
+    def test_post_default_form_use_view(self):
+        self._create_entity_data("entityuseview")
+        self.assertTrue(EntityData.exists(self.testdata, "entityuseview"))
+        f = entitydata_default_view_form_data(
+                entity_id="entityuseview", action="view",
+                use_view="_view/Type_view", 
+                )
+        f.pop('entity_id', None)
+        u = entitydata_edit_url(
+            "view", "testcoll", "testtype", "entityuseview", view_id="Default_view"
+            )
+        r = self.client.post(u, f)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertEqual(r.content,       "")
+        v = TestHostUri + entitydata_edit_url(
+            "view", "testcoll", "testtype", entity_id="entityuseview", view_id="Type_view"
+            )
+        c = continuation_url_param("/testsite/c/testcoll/d/testtype/")
+        self.assertIn(v, r['location'])
+        self.assertIn(c, r['location'])
+        self._check_entity_data_values("entityuseview")
+        return
+
+    def test_post_default_form_default_view(self):
+        # Set default entity view, then ensure collection view redirects to it
+        self._create_entity_data("entitydefaultview")
+        f = entitydata_default_view_form_data(
+                entity_id="entitydefaultview", action="view",
+                default_view="default_view", 
+                )
+        f.pop('entity_id', None)
+        u = entitydata_edit_url(
+            action="view", view_id="Default_view",
+            coll_id="testcoll", type_id="testtype", entity_id="entitydefaultview"
+            )
+        r = self.client.post(u, f)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        self.assertEqual(r.content,       "")
+        v = TestHostUri + u
+        self.assertIn(v, r['location'])
+        ih = "info_head=" + make_quoted_message(message.ACTION_COMPLETED)
+        im = "info_message=" + make_quoted_message(message.DEFAULT_VIEW_UPDATED,
+            view_id="Default_view",
+            entity_id="entitydefaultview")
+        self.assertIn(ih, r['location'])
+        self.assertIn(im, r['location'])
+        # Get collection root and check redirect to entity view
+        u2 = collection_view_url(coll_id="testcoll")
+        r2 = self.client.get(u2)
+        self.assertEqual(r2.status_code,   302)
+        self.assertEqual(r2.reason_phrase, "FOUND")
+        self.assertEqual(r2.content,       "")
+        v2 = TestHostUri + entitydata_edit_url(
+            coll_id="testcoll", view_id="Default_view", type_id="testtype", entity_id="entitydefaultview"
+            )
+        self.assertEqual(v2, r2['location'])
+        return
+
     def test_post_default_form_use_view_no_login(self):
         self.client.logout()
         f = entitydata_default_view_form_data(
@@ -181,20 +247,40 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         r = self.client.get(u+"?continuation_url=/xyzzy/")
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<title>Collection testcoll</title>")
-        cont_uri = "?continuation_url=%s"%u + "%%3Fcontinuation_url=/xyzzy/"
-        cont_uri = ""
+        # log.info(r.content)     #@@
+        entity_label = """Entity testcoll/testtype/entity1"""
+        view_label   = """Type definition"""
+        permalink    = """<a href="./" class="permalink">&#x1F517;</a>"""
+        entity_title = "%s - %s - Collection testcoll"%(entity_label, view_label)
+        self.assertContains(r, "<title>%s</title>"%(entity_title,))
+        self.assertContains(r, '<h2 class="page-heading">%s %s</h2>'%(view_label, permalink), html=True)
+        cont_uri = "?continuation_url=%s"%u + "%3Fcontinuation_url=/xyzzy/"
         field_vals = default_fields(
             coll_id="testcoll", type_id="testtype", entity_id="00000001",
+            entity_base      = "/testsite/c/testcoll/d/testtype/entity1",
             entity_url       = "/testsite/c/testcoll/d/testtype/entity1/" + cont_uri,
             default_view_url = "/testsite/c/testcoll/d/_view/Default_view/" + cont_uri,
-            default_list_url = "/testsite/c/testcoll/d/_list/Default_list/" + cont_uri
+            default_list_url = "/testsite/c/testcoll/d/_list/Default_list/" + cont_uri,
+            tooltip1="", # 'title="%s"'%r.context['fields'][0]['field_help'],
+            tooltip2="", # 'title="%s"'%r.context['fields'][1]['field_help'],
+            tooltip3="", # 'title="%s"'%r.context['fields'][2]['field_help'],
+            tooltip4="", # 'title="%s"'%r.context['fields'][3]['field_help'],
+            tooltip5="", # 'title="%s"'%r.context['fields'][4]['field_help'],
+            tooltip6="", # 'title="%s"'%r.context['fields'][5]['field_help'],
+            tooltip7="", # 'title="%s"'%r.context['fields'][6]['field_help'],
+            button_subtype_tip=
+                "Create a subtype of the current type.  "+
+                "(View and list type identifiers are copied from the current type; "+
+                "the URI of the current type is inserted as a supertype URI of the new type; "+
+                "other fields are taken from the corresponding '_initial_values' record, "+
+                "and may be extended or modified later.)"+
+                "",
             )
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" %(tooltip1)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
-                  <span>Id</span>
+                  <span>Type Id</span>
                 </div>
                 <div class="%(input_classes)s">
                   <a href="%(entity_url)s">entity1</a>
@@ -203,7 +289,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=6)
         formrow2 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" %(tooltip2)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Label</span>
@@ -215,7 +301,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow3 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" %(tooltip3)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Comment</span>
@@ -229,10 +315,10 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow4 = """
-            <div class="small-12 columns">
+            <div class="small-12 columns" %(tooltip4)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
-                  <span>URI</span>
+                  <span>Type URI</span>
                 </div>
                 <div class="%(input_classes)s">
                   <span>&nbsp;</span>
@@ -241,7 +327,19 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow5 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 columns" %(tooltip5)s>
+              <div class="grouprow row">
+                <div class="%(group_label_classes)s">
+                  <span>Supertype URIs</span>
+                </div>
+                <div class="%(group_placeholder_classes)s">
+                  <span>(None specified)</span>
+                </div>
+              </div>
+            </div>
+            """%field_vals(width=12)
+        formrow6 = """
+            <div class="small-12 medium-6 columns" %(tooltip6)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Default view</span>
@@ -252,8 +350,8 @@ class GenericEntityViewViewTest(AnnalistTestCase):
               </div>
             </div>
             """%field_vals(width=6)
-        formrow6 = """
-            <div class="small-12 medium-6 columns">
+        formrow7 = """
+            <div class="small-12 medium-6 columns" %(tooltip7)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Default list</span>
@@ -264,7 +362,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
               </div>
             </div>
             """%field_vals(width=6)
-        formrow7a = """
+        formrow8a = """
             <div class="%(space_classes)s">
               <div class="row">
                 <div class="small-12 columns">
@@ -273,27 +371,36 @@ class GenericEntityViewViewTest(AnnalistTestCase):
               </div>
             </div>
             """%field_vals(width=2)
-        formrow7b = """
+        formrow8b = """
             <div class="%(button_wide_classes)s">
               <div class="row">
                 <div class="%(button_left_classes)s">
-                  <input type="submit" name="edit"  value="Edit" />
-                  <input type="submit" name="copy"  value="Copy" />
-                  <input type="submit" name="close" value="Close" />
+                  <input type="submit" name="edit"  value="Edit"
+                         title="Edit entity data." />
+                  <input type="submit" name="copy"  value="Copy"
+                         title="Copy, then edit entity data as new entity." />
+                  <input type="submit" name="close" value="Close"
+                         title="Return to previous page." />
                 </div>
               </div>
             </div>
             """%field_vals(width=4)
-        formrow7c = """
+        formrow8c = """
             <div class="%(button_wide_classes)s">
               <div class="row">
-                <div class="%(button_right_classes)s">
-                  <input type="submit" name="open_view"     value="View description" />
+                <div class="%(button_r_med_up_classes)s">
+                  <!-- <input type="submit" name="open_view"    value="View description" /> -->
+                  <input type="submit" name="Define_subtype" value="Define subtype"
+                         title="%(button_subtype_tip)s" />
+                  <input type="submit" name="default_view" value="Set default view"
+                         title="Select this display as the default view for collection 'testcoll'." />
+                  <input type="submit" name="customize"    value="Customize"
+                         title="Open 'Customize' view for collection 'testcoll'." />
                 </div>
               </div>
             </div>
-            """%field_vals(width=4)
-        formrow8 = ("""
+            """%field_vals(width=6)
+        formrow9a = ("""
             <div class="row view-value-row">
               <div class="%(label_classes)s">
                 <span>Choose view</span>
@@ -315,7 +422,22 @@ class GenericEntityViewViewTest(AnnalistTestCase):
               </div>
             </div>
             """)%field_vals(width=6)
-        formrow9 = """
+        formrow9b = ("""
+            <div class="row view-value-row">
+              <div class="link-bar small-12 columns">
+                <a href="%(entity_base)s/entity_data.ttl" title="Retrieve underlying data as Turtle">
+                  Turtle
+                </a>
+                <a href="%(entity_base)s/entity_data.jsonld" title="Retrieve underlying data as JSON-LD">
+                  JSON-LD
+                </a>
+                <a href="%(entity_base)s/entity_data.jsonld?type=application/json" title="Display underlying JSON data">
+                  <img src="/static/images/get_the_data_88x31.png" alt="get_the_data">
+                </a>
+              </div>
+            </div>
+            """)%field_vals(width=6)
+        formrow10 = """
             <div class="%(button_right_classes)s">
               <div class="row">
                 <div class="small-12 columns">
@@ -327,19 +449,21 @@ class GenericEntityViewViewTest(AnnalistTestCase):
               </div>
             </div>
             """%field_vals(width=6)
-        # log.info(r.content)
+        # log.info(r.content)     #@@
         self.assertContains(r, formrow1,  html=True)
         self.assertContains(r, formrow2,  html=True)
         self.assertContains(r, formrow3,  html=True)
         self.assertContains(r, formrow4,  html=True)
         self.assertContains(r, formrow5,  html=True)
         self.assertContains(r, formrow6,  html=True)
-        self.assertContains(r, formrow7a, html=True)
-        self.assertContains(r, formrow7b, html=True)
-        # self.assertContains(r, formrow7c, html=True)
-        self.assertContains(r, formrow8,  html=True)
+        self.assertContains(r, formrow7,  html=True)
+        self.assertContains(r, formrow8a, html=True)
+        self.assertContains(r, formrow8b, html=True)
+        self.assertContains(r, formrow8c, html=True)
+        self.assertContains(r, formrow9a, html=True)
+        self.assertContains(r, formrow9b, html=True)
         # New buttons hidden (for now)
-        # self.assertContains(r, formrow9, html=True)
+        # self.assertContains(r, formrow10, html=True)
         return
 
     def test_get_view(self):
@@ -348,107 +472,75 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<h3>'testtype' data in collection 'testcoll'</h3>")
-        # Test context
-        view_url = collection_entity_view_url(coll_id="testcoll", type_id="testtype", entity_id="entity1")
+        check_type_view_context_fields(self, r, 
+            action="edit",
+            entity_id="entity1", orig_entity_id="entity1",
+            type_id="testtype",
+            type_label="Entity testcoll/testtype/entity1",
+            type_comment="Entity coll testcoll, type testtype, entity entity1",
+            type_uri="",
+            type_supertype_uris=[],
+            type_view="Default_view", type_view_options=self.no_view_id + self.view_options,
+            type_list="Default_list", type_list_options=self.no_list_id + self.list_options,
+            type_aliases=[],
+            )
+        return
+
+    # Entity data view
+    def test_get_data_view(self):
+        u = entitydata_edit_url("edit", "testcoll", "testtype", entity_id="entity1")
+        r = self.client.get(u)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
+        check_default_view_context_fields(self, r, 
+            action="edit",
+            type_id="testtype",
+            entity_id="entity1", orig_entity_id="entity1",
+            entity_label="Entity testcoll/testtype/entity1",
+            entity_comment="Entity coll testcoll, type testtype, entity entity1",
+            entity_data_ref=entity_url("testcoll", "testtype", entity_id="entity1")+layout.ENTITY_DATA_FILE,
+            view_id="Default_view"
+            )
+        u = collection_entity_view_url(coll_id="testcoll", type_id="testtype", entity_id="entity1")
+        r = self.client.get(u)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "testtype")
         self.assertEqual(r.context['entity_id'],        "entity1")
         self.assertEqual(r.context['orig_id'],          "entity1")
-        self.assertEqual(r.context['action'],           "edit")
-        # Fields
-        self.assertEqual(len(r.context['fields']), 8)
-        # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'],           'Type_id')
-        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(type id)")
-        self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][0]['field_value'],        "entity1")
-        self.assertEqual(r.context['fields'][0]['options'],            self.no_options)
-        # 2nd field - Label
-        type_label_value = (
-            "Entity testcoll/testtype/entity1"
+        self.assertEqual(r.context['action'],           "view")
+        self.assertEqual(r.context['continuation_url'], "")
+        self.assertEqual(
+            r.context['entity_data_ref'],      
+            u+layout.ENTITY_DATA_FILE
             )
-        self.assertEqual(r.context['fields'][1]['field_id'],           'Type_label')
-        self.assertEqual(r.context['fields'][1]['field_name'],         'Type_label')
-        self.assertEqual(r.context['fields'][1]['field_label'],        'Label')
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Text")
-        self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][1]['field_value'],        type_label_value)
-        self.assertEqual(r.context['fields'][1]['options'],            self.no_options)
-        # 3rd field - comment
-        type_comment_value = (
-            "Entity coll testcoll, type testtype, entity entity1"
+        self.assertEqual(
+            r.context['entity_data_ref_json'], 
+            u+layout.ENTITY_DATA_FILE+"?type=application/json"
             )
-        self.assertEqual(r.context['fields'][2]['field_id'],           'Type_comment')
-        self.assertEqual(r.context['fields'][2]['field_name'],         'Type_comment')
-        self.assertEqual(r.context['fields'][2]['field_label'],        'Comment')
-        self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:comment")
-        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Richtext")
-        self.assertEqual(r.context['fields'][2]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][2]['field_value'],        type_comment_value)
-        self.assertEqual(r.context['fields'][2]['options'],            self.no_options)
-        # 4th field - URI
-        # (NOTE: blank unless explcicit value specified)
-        self.assertEqual(r.context['fields'][3]['field_id'],           'Type_uri')
-        self.assertEqual(r.context['fields'][3]['field_name'],         'Type_uri')
-        self.assertEqual(r.context['fields'][3]['field_label'],        'URI')
-        self.assertEqual(r.context['fields'][3]['field_property_uri'], "annal:uri")
-        self.assertEqual(r.context['fields'][3]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][3]['field_target_type'],  "annal:Identifier")
-        self.assertEqual(r.context['fields'][3]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][3]['field_value'],        "")
-        self.assertEqual(r.context['fields'][3]['options'],            self.no_options)
-        # 5th field - Supertype URIs
-        type_supertype_uris_help = (
-            "References to URIs/CURIEs of supertypes."
-            )
-        self.assertEqual(r.context['fields'][4]['field_id'],          'Type_supertype_uris')
-        self.assertEqual(r.context['fields'][4]['field_name'],        'Type_supertype_uris')
-        self.assertEqual(r.context['fields'][4]['field_label'],       'Supertype URIs')
-        self.assertEqual(r.context['fields'][4]['field_help'],        type_supertype_uris_help)
-        self.assertEqual(r.context['fields'][4]['field_placeholder'], "(Supertype URIs or CURIEs)")
-        self.assertEqual(r.context['fields'][4]['field_property_uri'], "annal:supertype_uris")
-        self.assertEqual(r.context['fields'][4]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][4]['field_target_type'],  "annal:Type_supertype_uri")
-        self.assertEqual(r.context['fields'][4]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][4]['field_value'],        "") #@@
-        self.assertEqual(r.context['fields'][4]['options'],            self.no_options)
-        # 6th field - view id
-        self.assertEqual(r.context['fields'][5]['field_id'],           'Type_view')
-        self.assertEqual(r.context['fields'][5]['field_name'],         'Type_view')
-        self.assertEqual(r.context['fields'][5]['field_label'],        'Default view')
-        self.assertEqual(r.context['fields'][5]['field_property_uri'], "annal:type_view")
-        self.assertEqual(r.context['fields'][5]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][5]['field_target_type'],  "annal:View")
-        self.assertEqual(r.context['fields'][5]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][5]['field_value'],        "Default_view")
-        self.assertEqual(r.context['fields'][5]['options'],            self.no_view_id + self.view_options)
-        # 7th field - list id
-        self.assertEqual(r.context['fields'][6]['field_id'],           'Type_list')
-        self.assertEqual(r.context['fields'][6]['field_name'],         'Type_list')
-        self.assertEqual(r.context['fields'][6]['field_label'],        'Default list')
-        self.assertEqual(r.context['fields'][6]['field_property_uri'], "annal:type_list")
-        self.assertEqual(r.context['fields'][6]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][6]['field_target_type'],  "annal:List")
-        self.assertEqual(r.context['fields'][6]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][6]['field_value'],        "Default_list")
-        self.assertEqual(r.context['fields'][6]['options'],            self.no_list_id + self.list_options)
+        return
+
+    # Entity data content negotiation
+    def test_get_data_json(self):
+        """
+        Request collection data as JSON-LD
+        """
+        u = collection_entity_view_url(coll_id="testcoll", type_id="testtype", entity_id="entity1")
+        r = self.client.get(u, HTTP_ACCEPT="application/ld+json")
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+        v = r['Location']
+        self.assertEqual(v, TestHostUri+u+layout.ENTITY_DATA_FILE)
         return
 
     def test_get_view_no_collection(self):
-        u = entitydata_edit_url("view", "no_collection", "_field", entity_id="entity1", view_id="Type_view")
+        u = entitydata_edit_url("view", "no_collection", "testtype", entity_id="entity1", view_id="Type_view")
         r = self.client.get(u)
         self.assertEqual(r.status_code,   404)
         self.assertEqual(r.reason_phrase, "Not found")
-        self.assertContains(r, "Collection no_collection does not exist", status_code=404)
+        msg_text = make_message(message.COLLECTION_NOT_EXISTS, id="no_collection")
+        self.assertContains(r, msg_text, status_code=404)
         return
 
     def test_get_view_no_type(self):
@@ -457,16 +549,18 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             r = self.client.get(u)
         self.assertEqual(r.status_code,   404)
         self.assertEqual(r.reason_phrase, "Not found")
-        self.assertContains(r, "Record type no_type in collection testcoll does not exist", status_code=404)
+        msg_text  = make_message(message.RECORD_TYPE_NOT_EXISTS, id="no_type")
+        self.assertContains(r, msg_text, status_code=404)
         return
 
     def test_get_view_no_view(self):
-        u = entitydata_edit_url("edit", "testcoll", "_field", entity_id="entity1", view_id="no_view")
+        u = entitydata_edit_url("edit", "testcoll", "testtype", entity_id="entity1", view_id="no_view")
         with SuppressLogging(logging.WARNING):
             r = self.client.get(u)
-        self.assertEqual(r.status_code,   404)
-        self.assertEqual(r.reason_phrase, "Not found")
-        self.assertContains(r, "Record view no_view in collection testcoll does not exist", status_code=404)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
+        msg_text = make_message(message.RECORD_VIEW_NOT_EXISTS, id="no_view")
+        self.assertContains(r, msg_text)
         return
 
     def test_get_view_no_entity(self):
@@ -478,7 +572,8 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         # log.debug(r.content)
         err_label = error_label("testcoll", "testtype", "entitynone")
-        self.assertContains(r, "<p>Entity %s does not exist</p>"%err_label, status_code=404)
+        msg_text  = make_message(message.ENTITY_DOES_NOT_EXIST, id="entitynone", label=err_label)
+        self.assertContains(r, "<p>%s</p>"%msg_text, status_code=404)
         return
 
     #   -----------------------------------------------------------------------------
@@ -512,9 +607,6 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         c = continuation_url_param(u, prev_cont=l)
         self.assertIn(e, r['location'])
         self.assertIn(c, r['location'])
-        # 'http://test.example.com/testsite/c/testcoll/v/Default_view/testtype/entityview/!edit
-        #   ?continuation_url=/testsite/c/testcoll/v/Default_view/testtype/entityview/!view
-        #   %3Fcontinuation_url=/testsite/c/testcoll/d/testtype/'
         return
 
     def test_post_view_entity_copy(self):
