@@ -17,6 +17,7 @@ from django.contrib.auth.models     import User
 from django.test                    import TestCase
 from django.test.client             import Client
 
+from annalist                       import layout
 from annalist.identifiers           import ANNAL, RDFS
 
 from annalist.models.entitytypeinfo import EntityTypeInfo
@@ -29,13 +30,15 @@ from annalist.models.entitydata     import EntityData
 from AnnalistTestCase       import AnnalistTestCase
 from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
 from init_tests             import (
-    init_annalist_test_site, init_annalist_bib_site, 
-    init_annalist_test_coll, init_annalist_bib_coll, 
+    init_annalist_test_site,
+    init_annalist_test_coll,
+    init_annalist_named_test_coll,
     resetSitedata
     )
 from entity_testutils       import (
     collection_create_values,
     create_test_user, create_user_permissions,
+    context_view_field,
     context_list_entities,
     context_list_head_fields, context_list_item_fields,
     context_list_item_field, context_list_item_field_value
@@ -57,12 +60,8 @@ class FieldAliasTest(AnnalistTestCase):
     """
 
     def setUp(self):
-        self.testsite  = init_annalist_bib_site()
-        self.testcoll  = init_annalist_bib_coll()
-        # init_annalist_test_site()
-        # self.testsite = Site(TestBaseUri, TestBaseDir)
-        # self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
-        # self.testtype = RecordType.create(self.testcoll, "testtype", recordtype_create_values("testtype"))
+        self.testsite  = init_annalist_test_site()
+        self.testcoll  = init_annalist_named_test_coll(layout.BIBDATA_ID)
         # Create BibEntry record (BibEntry_type defines field alias)
         self.testdata   = RecordTypeData.create(self.testcoll, "BibEntry_type", {})
         self.bibentity1_data = (
@@ -100,12 +99,12 @@ class FieldAliasTest(AnnalistTestCase):
         return
 
     def tearDown(self):
-        # resetSitedata(scope="collections")
+        resetSitedata(scope="collections")
         return
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        # resetSitedata()
         return
 
     def test_view_field_alias(self):
@@ -118,7 +117,6 @@ class FieldAliasTest(AnnalistTestCase):
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<title>Collection testcoll</title>")
         # Test context
         url = entity_url(coll_id="testcoll", type_id="BibEntry_type", entity_id="bibentity1")
         self.assertEqual(r.context['coll_id'],          "testcoll")
@@ -127,11 +125,12 @@ class FieldAliasTest(AnnalistTestCase):
         self.assertEqual(r.context['orig_id'],          "bibentity1")
         self.assertEqual(r.context['action'],           "edit")
         # Fields
-        self.assertEqual(len(r.context['fields']), 4)        
+        self.assertEqual(len(r.context['fields']), 3)
         # Check aliased label field
-        self.assertEqual(r.context['fields'][2]['field_id'], 'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_property_uri'], RDFS.CURIE.label)
-        self.assertEqual(r.context['fields'][2]['field_value'], self.bibentity1_data['bib:title'])
+        f2 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f2['field_id'], 'Entity_label')
+        self.assertEqual(f2['field_property_uri'], RDFS.CURIE.label)
+        self.assertEqual(f2['field_value'], self.bibentity1_data['bib:title'])
         return
 
     def test_list_field_alias(self):
@@ -147,15 +146,18 @@ class FieldAliasTest(AnnalistTestCase):
         self.assertEqual(r.context['list_choices']['field_value'], "Default_list")
         # Fields
         head_fields = context_list_head_fields(r.context)
-        self.assertEqual(len(head_fields), 2)
+        self.assertEqual(len(head_fields), 1)       # One row of 2 cols..
+        self.assertEqual(len(head_fields[0]['row_field_descs']), 2)
+        f0 = context_view_field(r.context, 0, 0)
+        f1 = context_view_field(r.context, 0, 1)
         # 1st field
-        self.assertEqual(head_fields[0]['field_id'], 'Entity_id')
-        self.assertEqual(head_fields[0]['field_property_uri'], "annal:id")
-        self.assertEqual(head_fields[0]['field_value'], "")
+        self.assertEqual(f0['field_id'], 'Entity_id')
+        self.assertEqual(f0['field_property_uri'], "annal:id")
+        self.assertEqual(f0['field_value'], "")
         # 2nd field
-        self.assertEqual(head_fields[1]['field_id'], 'Entity_label')
-        self.assertEqual(head_fields[1]['field_property_uri'], "rdfs:label")
-        self.assertEqual(head_fields[1]['field_value'], "")
+        self.assertEqual(f1['field_id'], 'Entity_label')
+        self.assertEqual(f1['field_property_uri'], "rdfs:label")
+        self.assertEqual(f1['field_value'], "")
         # List entities (actually, just the one)
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 1)
@@ -193,7 +195,7 @@ class FieldAliasTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         # Check entity exists,and compare data with expected
-        typeinfo = EntityTypeInfo(self.testsite, self.testcoll, "BibEntry_type")
+        typeinfo = EntityTypeInfo(self.testcoll, "BibEntry_type")
         self.assertTrue(typeinfo.entityclass.exists(typeinfo.entityparent, "bibentity1"))
         e = typeinfo.entityclass.load(typeinfo.entityparent, "bibentity1")
         self.assertEqual(e.get_id(), "bibentity1")
@@ -203,7 +205,8 @@ class FieldAliasTest(AnnalistTestCase):
         del v['bib:author']
         self.assertDictionaryMatch(e.get_values(), v)
         self.assertEqual(e.get_values().get(RDFS.CURIE.label, None),   None)
-        self.assertEqual(e.get_values().get(RDFS.CURIE.comment, None), None)
+        # Comment defaults from aliased label value
+        self.assertEqual(e.get_values().get(RDFS.CURIE.comment, None), f['Bib_title'])
         return
 
     def test_save_field_alias_target(self):
@@ -226,8 +229,8 @@ class FieldAliasTest(AnnalistTestCase):
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
-        # Check entity exists,and compare data with expected
-        typeinfo = EntityTypeInfo(self.testsite, self.testcoll, "BibEntry_type")
+        # Check entity exists, and compare data with expected
+        typeinfo = EntityTypeInfo(self.testcoll, "BibEntry_type")
         self.assertTrue(typeinfo.entityclass.exists(typeinfo.entityparent, "bibentity1"))
         e = typeinfo.entityclass.load(typeinfo.entityparent, "bibentity1")
         self.assertEqual(e.get_id(), "bibentity1")
