@@ -24,12 +24,10 @@ from annalist.identifiers               import RDF, RDFS, ANNAL
 from annalist.models.site               import Site
 from annalist.models.collection         import Collection
 
-from annalist.views.fielddescription    import FieldDescription, field_description_from_view_field
-
+from annalist.views.fields.field_description    import FieldDescription, field_description_from_view_field
 from annalist.views.fields.render_placement     import get_placement_classes
 from annalist.views.fields                      import render_repeatgroup
 from annalist.views.fields.render_repeatgroup   import RenderRepeatGroup
-from annalist.views.fields.render_text          import RenderText
 from annalist.views.fields.render_fieldvalue    import RenderFieldValue, get_template
 from annalist.views.form_utils.fieldchoice      import FieldChoice
 
@@ -67,7 +65,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     def _get_repeatgroup_context(self):
@@ -95,7 +93,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
                 # ----- Field description -----
                 , 'field_id':                   'View_fields'
                 , 'field_name':                 'View_fields'
-                , 'field_target_type':          'annal:Field_group'
+                , 'field_value_type':          'annal:Field_group'
                 , 'field_label':                'Fields'
                 , 'field_help':                 'This resource descibes the repeated field description used when displaying and/or editing a record view description'
                 , 'field_render_type':          'RepeatGroup'
@@ -103,7 +101,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
                 , 'field_property_uri':         'annal:view_fields'
                 , 'field_placement':            get_placement_classes("small:0,12")
                 , 'field_placeholder':          '(repeat field description)'
-                , 'field_group_ref':            'View_field_view'
+                #@@ , 'field_group_ref':            'View_field_view'
                 , 'group_id':                   'View_fields'
                 , 'group_label':                'Fields'
                 , 'group_add_label':            'Add field'
@@ -147,17 +145,18 @@ class FieldRenderingTest(FieldRendererTestSupport):
         rendered_text = re.sub(r'\s+', " ", rendered_text)
         field_choices = (
                   [ FieldChoice(id="")
-                  , FieldChoice(id="_field/Entity_comment",         label="Comment")
-                  , FieldChoice(id="_field/Entity_id",              label="Id")
-                  , FieldChoice(id="_field/Entity_label",           label="Label")
-                  , FieldChoice(id="_field/Entity_see_also_repeat", label="See also")
-                  , FieldChoice(id="_field/Entity_type",            label="Type")
+                  , FieldChoice(id="_field/Entity_comment",     label="Comment")
+                  , FieldChoice(id="_field/Entity_id",          label="Id")
+                  , FieldChoice(id="_field/Entity_label",       label="Label")
+                  , FieldChoice(id="_field/Entity_see_also_r",  label="See also")
+                  , FieldChoice(id="_field/Entity_type",        label="Type")
+                  , FieldChoice(id="_field/Entity_uri",         label="Entity URI")
                   ])
         expect_elements = (
             [ '''<div class="group-label small-2 columns"> <span>Fields</span> </div>'''
             , '''<div class="row selectable">'''
             , '''<div class="view-label small-12 medium-4 columns"> <span>Field id</span> </div>'''
-            , '''<div class="view-label small-12 medium-4 columns"> <span>Property</span> </div>'''
+            , '''<div class="view-label small-12 medium-4 columns"> <span>Property URI</span> </div>'''
             , '''<div class="view-label small-12 medium-4 columns"> <span>Position/size</span> </div>'''
             # 1st field
             , '''<input type="checkbox" name="View_fields__select_fields"'''+
@@ -226,7 +225,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
                     , 'repeat_prefix':      context['repeat_prefix']
                     })
 
-        fieldrender = RenderFieldValue(
+        fieldrender = RenderFieldValue("render_int",
             view_renderer=render_int_view(), 
             edit_renderer=render_int_edit()
             )
@@ -250,7 +249,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
             "field/annalist_edit_text.html",
             "Can't load edit template"
             )
-        fieldrender = RenderFieldValue(
+        fieldrender = RenderFieldValue("render_compiled_template",
             view_template=view_template, 
             edit_template=edit_template
             )
@@ -268,7 +267,7 @@ class FieldRenderingTest(FieldRendererTestSupport):
 
     # Rendering using template file name supplied
     def test_RenderFieldValue_files(self):
-        fieldrender = RenderFieldValue(
+        fieldrender = RenderFieldValue("render_template",
             view_file="field/annalist_view_text.html", 
             edit_file="field/annalist_edit_text.html"
             )
