@@ -23,7 +23,6 @@ from annalist.models.recordtype             import RecordType
 from annalist.models.recordtypedata         import RecordTypeData
 from annalist.models.recordview             import RecordView
 from annalist.models.recordfield            import RecordField
-from annalist.models.recordgroup            import RecordGroup
 from annalist.models.entitydata             import EntityData
 
 from annalist.views.form_utils.fieldchoice  import FieldChoice
@@ -34,7 +33,9 @@ from init_tests             import init_annalist_test_site, init_annalist_test_c
 from entity_testutils       import (
     collection_create_values,
     render_select_options, render_choice_options,
-    create_test_user
+    create_test_user,
+    context_field_map,
+    context_view_field
     )
 from entity_testtypedata    import (
     recordtype_create_values, 
@@ -70,7 +71,7 @@ class RefMultifieldTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     # Support methods
@@ -128,13 +129,13 @@ class RefMultifieldTest(AnnalistTestCase):
             , 'rdfs:label':         "Test view label"
             , 'rdfs:comment':       "Test view comment"
             , 'annal:record_type':  "img_type"
-            , 'annal:add_field':    True
+            # , 'annal:add_field':    True
             , 'annal:view_fields':
               [ { 'annal:field_id':         "Entity_id"
                 , 'annal:field_placement':  "small:0,12;medium:0,6"
                 }
               , { 'annal:field_id':         "Test_refimg_field"
-                , 'annal:field_placement':  "small:0,12;medium:0,6"
+                , 'annal:field_placement':  "small:0,12;medium:6,6"
                 }
               ]
             })
@@ -148,13 +149,13 @@ class RefMultifieldTest(AnnalistTestCase):
             , 'rdfs:label':         "Test rptimg view label"
             , 'rdfs:comment':       "Test rptimg view comment"
             , 'annal:record_type':  "img_type"
-            , 'annal:add_field':    True
+            # , 'annal:add_field':    True
             , 'annal:view_fields':
               [ { 'annal:field_id':         "Entity_id"
                 , 'annal:field_placement':  "small:0,12;medium:0,6"
                 }
               , { 'annal:field_id':         "Test_rptref_field"
-                , 'annal:field_placement':  "small:0,12;medium:0,6"
+                , 'annal:field_placement':  "small:0,12;medium:6,6"
                 }
               ]
             })
@@ -168,61 +169,37 @@ class RefMultifieldTest(AnnalistTestCase):
             , "rdfs:comment":               "Image reference field comment"
             , "annal:field_render_type":    "RefMultifield"
             , "annal:field_value_mode":     "Value_entity"
-            , "annal:field_target_type":    "annal:Field_group"
+            , "annal:field_value_type":     "annal:Field_group"
             , "annal:field_entity_type":    "test:img_type"
             , "annal:placeholder":          "(ref image field)"
             , "annal:property_uri":         "test:ref_image"
             , "annal:field_placement":      "small:0,12;medium:0,6"
-            , "annal:group_ref":            "Test_refimg_group"
+            , "annal:field_fields": 
+                [ { "annal:field_id":   "Test_comment" }
+                , { "annal:field_id":   "Test_image"}
+                ]
             , "annal:field_ref_type":       "img_type"
             })
         self.assertTrue(refimg_field is not None)
         return refimg_field
 
-    def _create_refimg_group(self):
-        test_refimg_group = RecordGroup.create(self.testcoll, "Test_refimg_group",
-            { "annal:type":         "annal:Field_group"
-            , "annal:uri":          "test:test_refimg_group"
-            , "rdfs:label":         "Ref image group label"
-            , "rdfs:comment":       "Ref image group comment"
-            , "annal:record_type":  "test:img_type"
-            , "annal:group_fields": 
-                [ { "annal:field_id":   "Test_comment" }
-                , { "annal:field_id":   "Test_image"}
-                ]
-            })
-        self.assertTrue(test_refimg_group is not None)
-        return test_refimg_group
-
     def _create_rptref_field(self, label_add=None, label_delete=None):
         rptref_field = RecordField.create(self.testcoll, "Test_rptref_field",
             { "annal:type":                 "annal:Field"
             , "rdfs:label":                 "Repeat image reference"
             , "rdfs:comment":               "Repeat image reference field comment"
-            , "annal:field_render_type":    "RepeatGroupRow"
+            , "annal:field_render_type":    "Group_Seq_Row"
             , "annal:field_value_mode":     "Value_direct"
-            , "annal:field_target_type":    "annal:Field_group"
+            , "annal:field_value_type":     "annal:Field_group"
             , "annal:placeholder":          "(repeat image field)"
             , "annal:property_uri":         "test:rpt_image"
             , "annal:field_placement":      "small:0,12"
-            , "annal:group_ref":            "Test_rptref_group"
-            })
-        self.assertTrue(rptref_field is not None)
-        return rptref_field
-
-    def _create_rptref_group(self):
-        rptref_group = RecordGroup.create(self.testcoll, "Test_rptref_group",
-            { "annal:type":         "annal:Field_group"
-            , "annal:uri":          "test:test_rptref_group"
-            , "rdfs:label":         "Repeat image group label"
-            , "rdfs:comment":       "Repeat image group comment"
-            , "annal:record_type":  "test:img_type"
-            , "annal:group_fields": 
+            , "annal:field_fields": 
                 [ { "annal:field_id":   "Test_refimg_field" }
                 ]
             })
-        self.assertTrue(rptref_group is not None)
-        return rptref_group
+        self.assertTrue(rptref_field is not None)
+        return rptref_field
 
     def _create_refimg_image_field(self):
         refimg_image_field = RecordField.create(self.testcoll, "Test_image",
@@ -231,7 +208,7 @@ class RefMultifieldTest(AnnalistTestCase):
             , "rdfs:comment":               "In view mode, displays an image."
             , "annal:field_render_type":    "RefImage"
             , "annal:field_value_mode":     "Value_direct"
-            , "annal:field_target_type":    "annal:Image"
+            , "annal:field_value_type":    "annal:Image"
             , "annal:field_entity_type":    "test:img_type"
             , "annal:placeholder":          "(view image field)"
             , "annal:property_uri":         "test:image"
@@ -246,7 +223,7 @@ class RefMultifieldTest(AnnalistTestCase):
             , "rdfs:comment":               "In view mode, displays a comment."
             , "annal:field_render_type":    "Markdown"
             , "annal:field_value_mode":     "Value_direct"
-            , "annal:field_target_type":    "annal:Richtext"
+            , "annal:field_value_type":    "annal:Richtext"
             , "annal:field_entity_type":    "test:img_type"
             , "annal:placeholder":          "(view comment field)"
             , "annal:field_placement":      "small:0,12;medium:0,6"
@@ -263,8 +240,6 @@ class RefMultifieldTest(AnnalistTestCase):
         refimg_view = self._create_refimg_view()
         # Create multifield reference field
         refimg_field = self._create_refimg_field()
-        # Create multifield reference group
-        refimg_group = self._create_refimg_group()
         # Create multifield reference fields (where needed)
         refimg_image_field = self._create_refimg_image_field()
         refimg_image_field = self._create_refimg_comment_field()
@@ -279,12 +254,8 @@ class RefMultifieldTest(AnnalistTestCase):
         rptimg_view = self._create_rptimg_view()
         # Create repeat field referencing multifield reference
         rptref_field = self._create_rptref_field()
-        # Create multifield reference group
-        rptref_group = self._create_rptref_group()
         # Create multifield reference field
         refimg_field = self._create_refimg_field()
-        # Create multifield reference group
-        refimg_group = self._create_refimg_group()
         # Create multifield reference fields (where needed)
         refimg_image_field = self._create_refimg_image_field()
         refimg_image_field = self._create_refimg_comment_field()
@@ -302,40 +273,54 @@ class RefMultifieldTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
 
         # Check render context
-        self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          "ref_type")
-        self.assertEqual(r.context['entity_id'],        "Test_ref_entity")
-        self.assertEqual(r.context['action'],           "view")
+        self.assertEqual(r.context['coll_id'],      "testcoll")
+        self.assertEqual(r.context['type_id'],      "ref_type")
+        self.assertEqual(r.context['entity_id'],    "Test_ref_entity")
+        self.assertEqual(r.context['action'],       "view")
         # Fields
-        self.assertEqual(len(r.context['fields']), 2)
+        # print "@@ field map:\bn"+context_field_map(r.context)
+        self.assertEqual(len(r.context['fields']), 1)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'],            "Entity_id")
-        self.assertEqual(r.context['fields'][0]['field_name'],          "entity_id")
-        self.assertEqual(r.context['fields'][0]['field_label'],         "Id")
-        self.assertEqual(r.context['fields'][0]['field_value'],         "Test_ref_entity")
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0['field_id'],            "Entity_id")
+        self.assertEqual(f0['field_name'],          "entity_id")
+        self.assertEqual(f0['field_label'],         "Id")
+        self.assertEqual(f0['field_value'],         "Test_ref_entity")
         # 2nd field - multifield group
-        self.assertEqual(r.context['fields'][1]['field_id'],            "Test_refimg_field")
-        self.assertEqual(r.context['fields'][1]['field_name'],          "Test_refimg_field")
-        self.assertEqual(r.context['fields'][1]['field_label'],         "Image reference")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],   "RefMultifield")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_entity")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],     "Test_refimg_group")
-        self.assertEqual(r.context['fields'][1]['group_label'],         "Image reference")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'],  "test:ref_image")
-        self.assertEqual(r.context['fields'][1]['field_value'],         "Test_img_entity")
+        f1 = context_view_field(r.context, 0, 1)
+        self.assertEqual(f1['field_id'],            "Test_refimg_field")
+        self.assertEqual(f1['field_name'],          "Test_refimg_field")
+        self.assertEqual(f1['field_label'],         "Image reference")
+        self.assertEqual(f1['field_render_type'],   "RefMultifield")
+        self.assertEqual(f1['field_value_mode'],    "Value_entity")
+        self.assertEqual(f1['field_value_type'],    "annal:Field_group")
+        field_list = (
+            [ { "annal:field_id":   "Test_comment" }
+            , { "annal:field_id":   "Test_image"}
+            ])
+        self.assertEqual(f1['group_field_list'],    field_list)
+        self.assertEqual(f1['group_label'],         "Image reference")
+        self.assertEqual(f1['field_property_uri'],  "test:ref_image")
+        self.assertEqual(f1['field_value'],         "Test_img_entity")
 
         # Test rendered result
         field_vals = default_fields(
             coll_id="testcoll", type_id="ref_type", entity_id="Test_ref_entity", 
             view_id="Test_refimg_view",
             basepath=TestBasePath,
-            cont_uri_param=""
+            cont_uri_param="",
             )
         cont_uri_param = (
             "?continuation_url=%(basepath)s/c/%(coll_id)s/v/%(view_id)s/%(type_id)s/%(entity_id)s/!view"%
             field_vals()
             )
+
+        field_vals = default_fields(
+            coll_id="testcoll", type_id="ref_type", entity_id="Test_ref_entity", 
+            view_id="Test_refimg_view",
+            basepath=TestBasePath,
+            cont_uri_param=cont_uri_param,
+            )
         formrow1 = """
             <div class="small-12 medium-6 columns">
               <div class="row view-value-row">
@@ -354,10 +339,13 @@ class RefMultifieldTest(AnnalistTestCase):
             view_id="Test_refimg_view",
             field_id="image_field",
             basepath=TestBasePath,
-            ref_image="%s/c/testcoll/d/img_type/Test_img_entity/image_field.jpeg"%(TestBasePath,)
+            ref_image="%s/c/testcoll/d/img_type/Test_img_entity/image_field.jpeg"%(TestBasePath,),
+            cont_uri_param=cont_uri_param,
+            tooltip2a="",
+            tooltip2b="",
             )
         formrow2a = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" %(tooltip2a)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>View comment field</span>
@@ -368,21 +356,21 @@ class RefMultifieldTest(AnnalistTestCase):
               </div>
             </div>
             """%tgt_field_vals(width=6)
-        formrow2b = (
-            """<div class="small-12 medium-6 columns"> """+
-              """<div class="row view-value-row"> """+
-                """<div class="%(label_classes)s"> """+
-                  """<span>View image field</span> """+
-                """</div> """+
-                """<div class="%(input_classes)s"> """+
-                  """<a href="%(ref_image)s" target="_blank"> """+
-                    """<img src="%(ref_image)s" """+
-                    """     alt="Image at '%(ref_image)s'" /> """+
-                  """</a> """+
-                """</div> """+
-              """</div> """+
-            """</div> """
-            )%tgt_field_vals(width=6)
+        formrow2b = """
+            <div class="small-12 medium-6 columns" %(tooltip2b)s>
+              <div class="row view-value-row">
+                <div class="%(label_classes)s">
+                  <span>View image field</span>
+                </div>
+                <div class="%(input_classes)s">
+                  <a href="%(ref_image)s" target="_blank">
+                    <img src="%(ref_image)s"
+                         alt="Image at '%(ref_image)s'" />
+                  </a>
+                </div>
+              </div>
+            </div>
+            """%tgt_field_vals(width=6)
         # log.info(r.content)
         self.assertContains(r, formrow1,  html=True)
         self.assertContains(r, formrow2a, html=True)
@@ -399,37 +387,44 @@ class RefMultifieldTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
 
         # Check render context
-        self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          "ref_type")
-        self.assertEqual(r.context['entity_id'],        "Test_ref_entity")
-        self.assertEqual(r.context['action'],           "edit")
+        self.assertEqual(r.context['coll_id'],      "testcoll")
+        self.assertEqual(r.context['type_id'],      "ref_type")
+        self.assertEqual(r.context['entity_id'],    "Test_ref_entity")
+        self.assertEqual(r.context['action'],       "edit")
         # Fields
-        self.assertEqual(len(r.context['fields']), 2)
+        self.assertEqual(len(r.context['fields']), 1)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'],            "Entity_id")
-        self.assertEqual(r.context['fields'][0]['field_name'],          "entity_id")
-        self.assertEqual(r.context['fields'][0]['field_label'],         "Id")
-        self.assertEqual(r.context['fields'][0]['field_value'],         "Test_ref_entity")
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0['field_id'],            "Entity_id")
+        self.assertEqual(f0['field_name'],          "entity_id")
+        self.assertEqual(f0['field_label'],         "Id")
+        self.assertEqual(f0['field_value'],         "Test_ref_entity")
         # 2nd field - multifield group
-        self.assertEqual(r.context['fields'][1]['field_id'],            "Test_refimg_field")
-        self.assertEqual(r.context['fields'][1]['field_name'],          "Test_refimg_field")
-        self.assertEqual(r.context['fields'][1]['field_label'],         "Image reference")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],   "RefMultifield")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_entity")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],     "Test_refimg_group")
-        self.assertEqual(r.context['fields'][1]['group_label'],         "Image reference")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'],  "test:ref_image")
-        self.assertEqual(r.context['fields'][1]['field_value'],         "Test_img_entity")
+        f1 = context_view_field(r.context, 0, 1)
+        self.assertEqual(f1['field_id'],            "Test_refimg_field")
+        self.assertEqual(f1['field_name'],          "Test_refimg_field")
+        self.assertEqual(f1['field_label'],         "Image reference")
+        self.assertEqual(f1['field_render_type'],   "RefMultifield")
+        self.assertEqual(f1['field_value_mode'],    "Value_entity")
+        self.assertEqual(f1['field_value_type'],   "annal:Field_group")
+        field_list = (
+            [ { "annal:field_id":   "Test_comment" }
+            , { "annal:field_id":   "Test_image"}
+            ])
+        self.assertEqual(f1['group_field_list'],    field_list)
+        self.assertEqual(f1['group_label'],         "Image reference")
+        self.assertEqual(f1['field_property_uri'],  "test:ref_image")
+        self.assertEqual(f1['field_value'],         "Test_img_entity")
 
         # Test rendered result
         field_vals    = default_fields(
             coll_id="testcoll", type_id="ref_type", entity_id="Test_ref_entity", 
             view_id="Test_refimg_view",
-            basepath=TestBasePath
+            basepath=TestBasePath,
+            tooltip1=f0['field_tooltip'],
             )
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" title="%(tooltip1)s">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Id</span>
@@ -444,13 +439,14 @@ class RefMultifieldTest(AnnalistTestCase):
             """%field_vals(width=6)
 
         tgt_field_vals = default_fields(
-            coll_id="testcoll", type_id="img_type", entity_id="Test_img_entity", 
+            coll_id="testcoll", type_id="img_type", entity_id="Test_img_entity",
             view_id="Test_refimg_view",
             field_id="Test_refimg_field",
-            basepath=TestBasePath
+            basepath=TestBasePath,
+            tooltip2=f1['field_tooltip'],
             )
         formrow2 = ("""
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" title="%(tooltip2)s">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Image reference</span>
@@ -483,28 +479,33 @@ class RefMultifieldTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
 
         # Check render context
-        self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          "ref_type")
-        self.assertEqual(r.context['entity_id'],        "Test_rpt_entity")
-        self.assertEqual(r.context['action'],           "view")
+        self.assertEqual(r.context['coll_id'],      "testcoll")
+        self.assertEqual(r.context['type_id'],      "ref_type")
+        self.assertEqual(r.context['entity_id'],    "Test_rpt_entity")
+        self.assertEqual(r.context['action'],       "view")
         # Fields
         self.assertEqual(len(r.context['fields']), 2)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'],            "Entity_id")
-        self.assertEqual(r.context['fields'][0]['field_name'],          "entity_id")
-        self.assertEqual(r.context['fields'][0]['field_label'],         "Id")
-        self.assertEqual(r.context['fields'][0]['field_value'],         "Test_rpt_entity")
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0['field_id'],            "Entity_id")
+        self.assertEqual(f0['field_name'],          "entity_id")
+        self.assertEqual(f0['field_label'],         "Id")
+        self.assertEqual(f0['field_value'],         "Test_rpt_entity")
         # 2nd field - multifield group
-        self.assertEqual(r.context['fields'][1]['field_id'],            "Test_rptref_field")
-        self.assertEqual(r.context['fields'][1]['field_name'],          "Test_rptref_field")
-        self.assertEqual(r.context['fields'][1]['field_label'],         "Repeat image reference")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],   "RepeatGroupRow")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],     "Test_rptref_group")
-        self.assertEqual(r.context['fields'][1]['group_label'],         "Repeat image reference")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'],  "test:rpt_image")
-        self.assertEqual(r.context['fields'][1]['field_value'][0],      {'test:ref_image': 'Test_img_entity'})
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1['field_id'],            "Test_rptref_field")
+        self.assertEqual(f1['field_name'],          "Test_rptref_field")
+        self.assertEqual(f1['field_label'],         "Repeat image reference")
+        self.assertEqual(f1['field_render_type'],   "Group_Seq_Row")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],   "annal:Field_group")
+        field_list = (
+            [ { "annal:field_id":   "Test_refimg_field" }
+            ])
+        self.assertEqual(f1['group_field_list'],    field_list)
+        self.assertEqual(f1['group_label'],         "Repeat image reference")
+        self.assertEqual(f1['field_property_uri'],  "test:rpt_image")
+        self.assertEqual(f1['field_value'][0],      {'test:ref_image': 'Test_img_entity'})
 
         # Test rendered result
         field_vals    = default_fields(
@@ -517,6 +518,13 @@ class RefMultifieldTest(AnnalistTestCase):
             "?continuation_url=%(basepath)s/c/%(coll_id)s/v/%(view_id)s/%(type_id)s/%(entity_id)s/!view"%
             field_vals()
             )
+
+        field_vals    = default_fields(
+            coll_id="testcoll", type_id="ref_type", entity_id="Test_rpt_entity", 
+            view_id="Test_rptimg_view",
+            basepath=TestBasePath,
+            cont_uri_param=cont_uri_param,
+            )
         formrow1 = """
             <div class="small-12 medium-6 columns">
               <div class="row view-value-row">
@@ -534,7 +542,7 @@ class RefMultifieldTest(AnnalistTestCase):
               <span>Repeat image reference</span>
             </div>
             """
-        # Note two grou wrappers here: one for repeat and one for multifield ref...
+        # Note two group wrappers here: one for repeat and one for multifield ref...
         formrow2b = """
             <div class="small-12 medium-10 columns hide-for-small-only">
               <div class="row">
@@ -561,7 +569,8 @@ class RefMultifieldTest(AnnalistTestCase):
             view_id="Test_rptimg_view",
             field_id="image_field",
             basepath=TestBasePath,
-            ref_image="%s/c/testcoll/d/img_type/Test_img_entity/image_field.jpeg"%(TestBasePath,)
+            ref_image="%s/c/testcoll/d/img_type/Test_img_entity/image_field.jpeg"%(TestBasePath,),
+            cont_uri_param=cont_uri_param,
             )
         formrow3a = """
             <div class="small-12 medium-2 columns">
@@ -635,37 +644,43 @@ class RefMultifieldTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
 
         # Check render context
-        self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          "ref_type")
-        self.assertEqual(r.context['entity_id'],        "Test_rpt_entity")
-        self.assertEqual(r.context['action'],           "edit")
+        self.assertEqual(r.context['coll_id'],      "testcoll")
+        self.assertEqual(r.context['type_id'],      "ref_type")
+        self.assertEqual(r.context['entity_id'],    "Test_rpt_entity")
+        self.assertEqual(r.context['action'],       "edit")
         # Fields
         self.assertEqual(len(r.context['fields']), 2)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'],            "Entity_id")
-        self.assertEqual(r.context['fields'][0]['field_name'],          "entity_id")
-        self.assertEqual(r.context['fields'][0]['field_label'],         "Id")
-        self.assertEqual(r.context['fields'][0]['field_value'],         "Test_rpt_entity")
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0['field_id'],            "Entity_id")
+        self.assertEqual(f0['field_name'],          "entity_id")
+        self.assertEqual(f0['field_label'],         "Id")
+        self.assertEqual(f0['field_value'],         "Test_rpt_entity")
         # 2nd field - multifield group
-        self.assertEqual(r.context['fields'][1]['field_id'],            "Test_rptref_field")
-        self.assertEqual(r.context['fields'][1]['field_name'],          "Test_rptref_field")
-        self.assertEqual(r.context['fields'][1]['field_label'],         "Repeat image reference")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],   "RepeatGroupRow")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],     "Test_rptref_group")
-        self.assertEqual(r.context['fields'][1]['group_label'],         "Repeat image reference")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'],  "test:rpt_image")
-        self.assertEqual(r.context['fields'][1]['field_value'][0],      {'test:ref_image': 'Test_img_entity'})
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1['field_id'],            "Test_rptref_field")
+        self.assertEqual(f1['field_name'],          "Test_rptref_field")
+        self.assertEqual(f1['field_label'],         "Repeat image reference")
+        self.assertEqual(f1['field_render_type'],   "Group_Seq_Row")
+        self.assertEqual(f1['field_value_mode'],    "Value_direct")
+        self.assertEqual(f1['field_value_type'],   "annal:Field_group")
+        field_list = (
+            [ { "annal:field_id":   "Test_refimg_field" }
+            ])
+        self.assertEqual(f1['group_field_list'],    field_list)
+        self.assertEqual(f1['group_label'],         "Repeat image reference")
+        self.assertEqual(f1['field_property_uri'],  "test:rpt_image")
+        self.assertEqual(f1['field_value'][0],      {'test:ref_image': 'Test_img_entity'})
 
         # Test rendered result
         field_vals    = default_fields(
             coll_id="testcoll", type_id="ref_type", entity_id="Test_rpt_entity", 
             view_id="Test_rptimg_view",
-            basepath=TestBasePath
+            basepath=TestBasePath,
+            tooltip1=f0['field_tooltip'],
             )
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" title="%(tooltip1)s">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Id</span>
@@ -704,7 +719,8 @@ class RefMultifieldTest(AnnalistTestCase):
             coll_id="testcoll", type_id="img_type", entity_id="Test_img_entity", 
             view_id="Test_rptimg_view",
             repeat_id="Test_rptref_field",
-            field_id="Test_refimg_field"
+            field_id="Test_refimg_field",
+            tooltip3b=f1._field_description['group_field_descs'][0]['field_tooltip_test']
             )
         formrow3a = """
             <div class="small-12 medium-2 columns hide-for-small-only">
@@ -721,7 +737,7 @@ class RefMultifieldTest(AnnalistTestCase):
                 </div>
                 <div class="small-11 columns">
                   <div class="edit-grouprow row">
-                    <div class="%(col_item_classes)s">
+                    <div class="%(col_item_classes)s" title="%(tooltip3b)s">
                       <div class="row show-for-small-only">
                         <div class="view-label small-12 columns">
                           <span>Image reference</span>
