@@ -23,7 +23,6 @@ from annalist.models.recordtype         import RecordType
 from annalist.models.recordtypedata     import RecordTypeData
 from annalist.models.recordview         import RecordView
 from annalist.models.recordfield        import RecordField
-from annalist.models.recordgroup        import RecordGroup
 from annalist.models.entitydata         import EntityData
 
 from AnnalistTestCase       import AnnalistTestCase
@@ -31,7 +30,8 @@ from tests                  import TestHost, TestHostUri, TestBasePath, TestBase
 from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 from entity_testutils       import (
     collection_create_values,
-    create_test_user
+    create_test_user,
+    context_view_field
     )
 from entity_testtypedata    import (
     recordtype_create_values, 
@@ -63,7 +63,7 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections")
         return
 
     # Support methods
@@ -75,7 +75,7 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
             , 'rdfs:label':         "Test view label"
             , 'rdfs:comment':       "Test view comment"
             , 'annal:record_type':  "testtype"
-            , 'annal:add_field':    True
+            # , 'annal:add_field':    True
             , 'annal:view_fields':
               [ { 'annal:field_id':         "Entity_id"
                 , 'annal:field_placement':  "small:0,12;medium:0,6"
@@ -93,33 +93,22 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
             { "annal:type":                 "annal:Field"
             , "rdfs:label":                 "Test repeat field label"
             , "rdfs:comment":               "Test repeat field comment"
-            , "annal:field_render_type":    "RepeatGroupRow"
+            , "annal:field_render_type":    "Group_Seq_Row"
+            , "annal:field_value_mode":     "Value_direct"
             , "annal:field_value_type":     "annal:Field_group"
             , "annal:field_entity_type":    "test:testtype"
             , "annal:placeholder":          "(test repeat field)"
             , "annal:property_uri":         "test:repeat_fields"
             , "annal:field_placement":      "small:0,12"
-            , "annal:group_ref":            "testrepeatgroup"
+            , "annal:field_fields": 
+                [ { "annal:field_id":   "Entity_comment" }
+                ]
             , "annal:repeat_label_add":     label_add
             , "annal:repeat_label_delete":  label_delete
             })
         self.assertTrue(testrepeatfield is not None)
         return testrepeatfield
 
-    def _create_testrepeatgroup(self):
-        testrepeatgroup = RecordGroup.create(self.testcoll, "testrepeatgroup",
-            { "annal:type":         "annal:Field_group"
-            , "annal:uri":          "test:testrecordgroup"
-            , "rdfs:label":         "Test record group label"
-            , "rdfs:comment":       "Test record group comment"
-            , "annal:record_type":  "test:testtype"
-            , "annal:group_fields": 
-                [ { "annal:field_id":   "Entity_comment" }
-                ]
-            })
-        self.assertTrue(testrepeatgroup is not None)
-        return testrepeatgroup
-
     def _create_testentity(self):
         testentity = EntityData.create(self.testdata, "testentity",
             { "annal:type":         "test:testtype"
@@ -136,7 +125,6 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         testrepeatfield = self._create_testrepeatfield(
             label_add="Add group",
             label_delete="Remove group")
-        testrepeatgroup = self._create_testrepeatgroup()
         # Render view
         u = entitydata_edit_url("new", "testcoll", "testtype", view_id="testview")
         r = self.client.get(u)
@@ -150,28 +138,36 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         # Fields
         self.assertEqual(len(r.context['fields']), 2)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_value'], "00000001")
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0['field_id'], 'Entity_id')
+        self.assertEqual(f0['field_name'], 'entity_id')
+        self.assertEqual(f0['field_label'], 'Id')
+        self.assertEqual(f0['field_value'], "00000001")
         # 2nd field - repeat group
-        self.assertEqual(r.context['fields'][1]['field_id'],           "testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['field_name'],         "testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['field_label'],        "Test repeat field label")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],  "RepeatGroupRow")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],    "testrepeatgroup")
-        self.assertEqual(r.context['fields'][1]['group_label'],        "Test repeat field label")
-        self.assertEqual(r.context['fields'][1]['group_add_label'],    "Add group")
-        self.assertEqual(r.context['fields'][1]['group_delete_label'], "Remove group")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "test:repeat_fields")
-        self.assertEqual(len(r.context['fields'][1]['field_value']), 0)
-        self.assertEqual(r.context['fields'][1]['field_value'], "") #@@ Really?
+        f1 = context_view_field(r.context, 1, 0)
+        repeat_fields = (
+            [ { "annal:field_id":   "Entity_comment" }
+            ])
+        self.assertEqual(f1['field_id'],           "testrepeatfield")
+        self.assertEqual(f1['field_name'],         "testrepeatfield")
+        self.assertEqual(f1['field_label'],        "Test repeat field label")
+        self.assertEqual(f1['field_render_type'],  "Group_Seq_Row")
+        self.assertEqual(f1['field_value_mode'],   "Value_direct")
+        self.assertEqual(f1['field_value_type'],   "annal:Field_group")
+        self.assertEqual(f1['group_field_list'],   repeat_fields)
+        self.assertEqual(f1['group_label'],        "Test repeat field label")
+        self.assertEqual(f1['group_add_label'],    "Add group")
+        self.assertEqual(f1['group_delete_label'], "Remove group")
+        self.assertEqual(f1['field_property_uri'], "test:repeat_fields")
+        self.assertEqual(len(f1['field_value']),   0)
+        self.assertEqual(f1['field_value'],        "") #@@ Really?
         # Test rendered result
-        field_vals = default_fields(coll_id="testcoll", type_id="testtype", entity_id="00000001")
+        field_vals = default_fields(
+            coll_id="testcoll", type_id="testtype", entity_id="00000001",
+            tooltip1=f0['field_tooltip'],
+            )
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" title="%(tooltip1)s">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Id</span>
@@ -232,7 +228,6 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         testrepeatfield = self._create_testrepeatfield(
             label_add=None,
             label_delete="")
-        testrepeatgroup = self._create_testrepeatgroup()
         # Render view
         u = entitydata_edit_url("new", "testcoll", "testtype", view_id="testview")
         r = self.client.get(u)
@@ -246,24 +241,29 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         # Fields
         self.assertEqual(len(r.context['fields']), 2)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_value'], "00000001")
+        i = 0
+        self.assertEqual(context_view_field(r.context, i, 0)['field_id'], 'Entity_id')
+        self.assertEqual(context_view_field(r.context, i, 0)['field_name'], 'entity_id')
+        self.assertEqual(context_view_field(r.context, i, 0)['field_label'], 'Id')
+        self.assertEqual(context_view_field(r.context, i, 0)['field_value'], "00000001")
         # 2nd field - repeat group
-        self.assertEqual(r.context['fields'][1]['field_id'],           "testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['field_name'],         "testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['field_label'],        "Test repeat field label")
-        self.assertEqual(r.context['fields'][1]['field_render_type'],  "RepeatGroupRow")
-        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
-        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Field_group")
-        self.assertEqual(r.context['fields'][1]['field_group_ref'],    "testrepeatgroup")
-        self.assertEqual(r.context['fields'][1]['group_label'],        "Test repeat field label")
-        self.assertEqual(r.context['fields'][1]['group_add_label'],    "Add testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['group_delete_label'], "Remove testrepeatfield")
-        self.assertEqual(r.context['fields'][1]['field_property_uri'], "test:repeat_fields")
-        self.assertEqual(len(r.context['fields'][1]['field_value']), 0)
-        self.assertEqual(r.context['fields'][1]['field_value'], "") #@@ Really?
+        i = 1
+        repeat_fields = (
+            [ { "annal:field_id":   "Entity_comment" }
+            ])
+        self.assertEqual(context_view_field(r.context, i, 0)['field_id'],           "testrepeatfield")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_name'],         "testrepeatfield")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_label'],        "Test repeat field label")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_render_type'],  "Group_Seq_Row")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_value_mode'],   "Value_direct")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_value_type'],   "annal:Field_group")
+        self.assertEqual(context_view_field(r.context, i, 0)['group_field_list'],   repeat_fields)
+        self.assertEqual(context_view_field(r.context, i, 0)['group_label'],        "Test repeat field label")
+        self.assertEqual(context_view_field(r.context, i, 0)['group_add_label'],    "Add testrepeatfield")
+        self.assertEqual(context_view_field(r.context, i, 0)['group_delete_label'], "Remove testrepeatfield")
+        self.assertEqual(context_view_field(r.context, i, 0)['field_property_uri'], "test:repeat_fields")
+        self.assertEqual(len(context_view_field(r.context, i, 0)['field_value']), 0)
+        self.assertEqual(context_view_field(r.context, i, 0)['field_value'], "") #@@ Really?
         return
 
     def test_RenderRepeatGroup_view_no_values(self):
@@ -272,7 +272,6 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         testrepeatfield = self._create_testrepeatfield(
             label_add="Add group",
             label_delete="Remove group")
-        testrepeatgroup = self._create_testrepeatgroup()
         testentity      = self._create_testentity()
         # Render view
         u = entitydata_edit_url(
@@ -282,15 +281,16 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
-        self.assertContains(r, "<title>Collection testcoll</title>")
         # Test rendered result
+        cont_uri_param = "?continuation_url="+u
         field_vals = default_fields(
             coll_id="testcoll", type_id="testtype", entity_id="00000001",
-            view_url=v, cont_uri_param="" 
+            view_url=v, cont_uri_param=cont_uri_param,
+            tooltip1="",
+            tooltip2="",
             )
-        cont_uri_param = "?continuation_url="+u
         formrow1 = """
-            <div class="small-12 medium-6 columns">
+            <div class="small-12 medium-6 columns" %(tooltip1)s>
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Id</span>
@@ -302,16 +302,16 @@ class RepeatGroupRenderingTest(AnnalistTestCase):
             </div>
             """%field_vals(width=6)
         formrow2 = """
-            <div class="row">
+            <div class="grouprow row">
               <div class="%(group_label_classes)s">
                 <span>Test repeat field label</span>
               </div>
               <div class="group-placeholder %(group_row_body_classes)s">
-                <span>(None)</span>
+                <span>(None specified)</span>
               </div>
             </div>
             """%field_vals(width=12)
-        # log.info(r.content)
+        # log.info(r.content) #@@
         self.assertContains(r, formrow1,  html=True)
         self.assertContains(r, formrow2, html=True)
         return
