@@ -38,8 +38,8 @@ def expect_render_option(choice_val, select_val, placeholder):
         option_val  = "opt_type/"+choice_val
         choice_text = "label "+choice_val
         value_text  = ''' value="%s"'''%option_val
-    fc = FieldChoice(id=option_val, label=choice_text)
-    return '''<option%s%s>%s</option>'''%(value_text, selected, fc.option_label_html())
+    fc = FieldChoice(id=option_val, label=choice_text, choice_value=(choice_val=="dup"))
+    return '''<option%s%s>%s</option>'''%(value_text, selected, fc.choice_html())
 
 def expect_render_select(select_name, choice_list, select_val, placeholder):
     sel  = """<select name="%s">"""%select_name
@@ -47,7 +47,7 @@ def expect_render_select(select_name, choice_list, select_val, placeholder):
         val = "opt_type/"+select_val if select_val else ""
         lab = "opt_type/"+select_val if select_val else placeholder
         fc = FieldChoice(id=val, label=lab)
-        nopt = ['''<option value="%s" selected="selected">%s</option>'''%(val, fc.option_label_html())]
+        nopt = ['''<option value="%s" selected="selected">%s</option>'''%(val, fc.choice_html())]
     else:
         nopt = []
     opts = [ expect_render_option(choice_val, select_val, placeholder) for choice_val in choice_list ]
@@ -91,7 +91,6 @@ class SelectRenderingTest(FieldRendererTestSupport):
     def test_RenderChoiceValue(self):
         def expect_render(valkey, vallabel, vallink, valchoices):
             valcont = "?continuation_url=test_cont"
-            valcont = ""
             if vallink and valkey in valchoices:
                 render_view = """<a href="%s">%s</a> """%(vallink+valcont, vallabel)
             elif valkey == "":
@@ -107,10 +106,11 @@ class SelectRenderingTest(FieldRendererTestSupport):
             return {'view': render_view, 'edit': render_edit}
         noval = "(No 'test label' selected)"
         test_values = (
-            [ ( "aa", "label aa",    "http://example.org/aa", ["aa", "bb", "cc"])
-            , ( "",   noval,         None,                    ["", "aa", "bb", "cc"])
-            , ( "dd", "opt_type/dd", "http://example.org/dd", ["aa", "bb", "cc"])
-            , ( "",   noval,         None,                    ["aa", "bb", "cc"])
+            [ ( "aa",  "label aa",    "http://example.org/aa",  ["aa", "bb", "cc"])
+            , ( "",    noval,         None,                     ["", "aa", "bb", "cc"])
+            , ( "dd",  "opt_type/dd", "http://example.org/dd",  ["aa", "bb", "cc"])
+            , ( "",    noval,         None,                     ["aa", "bb", "cc"])
+            , ( "dup", "label dup",   "http://example.org/dup", ["aa", "bb", "cc", "dup", "dup"])
             ])
         test_value_context_renders = (
             [ ( self._make_select_test_context(valkey, vallink, valchoices, noval), 
@@ -133,7 +133,6 @@ class SelectRenderingTest(FieldRendererTestSupport):
     def test_RenderSelectValue(self):
         def expect_render(valkey, vallabel, vallink, valchoices):
             valcont = "?continuation_url=test_cont"
-            valcont = ""
             if vallink and valkey in valchoices:
                 render_view = (
                     """<a href="%s">%s</a> """%
@@ -158,7 +157,7 @@ class SelectRenderingTest(FieldRendererTestSupport):
                 '''    <button type="submit" \n'''+
                 '''            name="%s__new_edit" \n'''+
                 '''            value="New"\n'''+
-                '''            title="Define new %s"\n'''+
+                '''            title="Define new or edit %s"\n'''+
                 '''    >\n'''+
                 '''      <span class="select-edit-button-text">+&#x270D;</span>\n'''+
                 '''    </button>\n'''+
@@ -215,3 +214,5 @@ if __name__ == "__main__":
     # tests  = getSuite(select=sel)
     # if tests: runner.run(tests)
     unittest.main()
+
+
