@@ -51,7 +51,7 @@ from entity_testtypedata    import (
 site_data_keys = (
     { '@id', '@type', '@context'
     , 'annal:type_id', 'annal:id', 'annal:type'
-    , 'annal:url', 'annal:comment'
+    , 'annal:url', 'annal:meta_comment'
     , 'rdfs:label', 'rdfs:comment'
     , 'collections', 'title'
     , 'annal:software_version'
@@ -86,7 +86,7 @@ class SiteTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     def test_SiteTest(self):
@@ -97,7 +97,6 @@ class SiteTest(AnnalistTestCase):
         s = Site(TestBaseUri, TestBaseDir)
         self.assertEqual(s._entitytype,     ANNAL.CURIE.Site)
         self.assertEqual(s._entityfile,     layout.SITE_META_FILE)
-        self.assertEqual(s._entityref,      layout.META_SITE_REF)
         self.assertEqual(s._entityid,       layout.SITEDATA_ID)
         self.assertEqual(s._entityurl,      TestBaseUri + "/")
         self.assertEqual(s._entitydir,      TestBaseDir + "/")
@@ -129,7 +128,7 @@ class SiteTest(AnnalistTestCase):
         self.assertEqual(ugp[RDFS.CURIE.label],               "Test User")
         self.assertEqual(ugp[RDFS.CURIE.comment],             "User user1: permissions for Test User in collection testcoll")
         self.assertEqual(ugp[ANNAL.CURIE.user_uri],           "mailto:testuser@example.org")
-        self.assertEqual(ugp[ANNAL.CURIE.user_permissions],   ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"])
+        self.assertEqual(ugp[ANNAL.CURIE.user_permission],    ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"])
         return
 
     def test_get_local_user_not_defined(self):
@@ -155,9 +154,8 @@ class SiteTest(AnnalistTestCase):
         self.assertEqual(ugp[ANNAL.CURIE.id],                 "_default_user_perms")
         self.assertEqual(ugp[ANNAL.CURIE.type_id],            "_user")
         self.assertEqual(ugp[RDFS.CURIE.label],               "Default permissions")
-        self.assertEqual(ugp[RDFS.CURIE.comment],             "Default permissions for authenticated user.")
         self.assertEqual(ugp[ANNAL.CURIE.user_uri],           "annal:User/_default_user_perms")
-        self.assertEqual(ugp[ANNAL.CURIE.user_permissions],   ["VIEW"])
+        self.assertEqual(ugp[ANNAL.CURIE.user_permission],    ["VIEW"])
         return
 
     # Collections
@@ -223,7 +221,7 @@ class SiteViewTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="all")
         return
 
     def test_SiteViewTest(self):
@@ -245,7 +243,7 @@ class SiteViewTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         # self.assertEqual(r.content, "???")
         self.assertContains(r, """<h3>Error</h3>""", html=True)
-        self.assertContains(r, """<p>Error presented</p>""", html=True)
+        self.assertContains(r, """<p class="messages">Error presented</p>""", html=True)
         return
 
     def test_get_info(self):
@@ -253,7 +251,7 @@ class SiteViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
         self.assertContains(r, """<h3>Information</h3>""", html=True)
-        self.assertContains(r, """<p>Information presented</p>""", html=True)
+        self.assertContains(r, """<p class="messages">Information presented</p>""", html=True)
         return
 
     def test_get_home(self):
@@ -284,8 +282,8 @@ class SiteViewTest(AnnalistTestCase):
         self.assertEqual(homelink.string,   "Home")
         self.assertEqual(homelink['href'],  self.uri)
         menuitems = s.find(class_="top-bar-section").find(class_="right").find_all("li")
-        self.assertEqual(menuitems[0].a.string,  "Login")
-        self.assertEqual(menuitems[0].a['href'], self.profileuri)
+        self.assertEqualIgnoreWS(menuitems[0].a.string,  "Login")
+        self.assertEqual(menuitems[0].a['href'],         self.profileuri)
         # Check displayed collections
         trows = s.form.find_all("div", class_="tbody")
         self.assertEqual(len(trows), len(init_collection_keys))
@@ -326,10 +324,10 @@ class SiteViewTest(AnnalistTestCase):
         self.assertEqual(homelink.string,   "Home")
         self.assertEqual(homelink['href'],  self.uri)
         menuitems = s.find(class_="top-bar-section").find(class_="right").find_all("li")
-        self.assertEqual(menuitems[0].a.string,     "User testuser")
-        self.assertEqual(menuitems[0].a['href'],    TestBasePath+"/profile/")
-        self.assertEqual(menuitems[1].a.string,     "Logout")
-        self.assertEqual(menuitems[1].a['href'],    TestBasePath+"/logout/")
+        self.assertEqualIgnoreWS(menuitems[0].a.string, "User testuser")
+        self.assertEqual(menuitems[0].a['href'],        TestBasePath+"/profile/")
+        self.assertEqualIgnoreWS(menuitems[1].a.string, "Logout")
+        self.assertEqual(menuitems[1].a['href'],        TestBasePath+"/logout/")
         # Displayed colllections and check-buttons
         # trows = s.form.find_all("div", class_="tbody")
         trows = s.select("form > div > div > div")
@@ -404,7 +402,7 @@ class SiteViewTest(AnnalistTestCase):
         self.assertEqual(testuser_perms[RDFS.CURIE.label],              "Test User")
         self.assertEqual(testuser_perms[RDFS.CURIE.comment],            expect_descr)
         self.assertEqual(testuser_perms[ANNAL.CURIE.user_uri],          "mailto:testuser@%s"%TestHost)
-        self.assertEqual(testuser_perms[ANNAL.CURIE.user_permissions],  expect_perms)
+        self.assertEqual(testuser_perms[ANNAL.CURIE.user_permission],   expect_perms)
         return
 
     def test_post_remove(self):
@@ -457,6 +455,12 @@ class SiteActionViewTests(AnnalistTestCase):
     def tearDown(self):
         return
 
+    @classmethod
+    def setUpClass(cls):
+        # Remove any collections left behind from previous tests
+        resetSitedata(scope="collections")
+        return
+
     def _conf_data(self, action="confirm"):
         action_values = (
             { 'confirm': "Confirm"
@@ -487,6 +491,11 @@ class SiteActionViewTests(AnnalistTestCase):
         # Confirm collections deleted
         r = self.client.get(TestBasePath+"/site/")
         colls = r.context['collections']
+        #@@ (diagnostic only)
+        if len(colls) != len(init_collection_keys)-2:
+            log.warning("@@ Collection count mismatch: %s != %d"%(len(colls), len(init_collection_keys)-2))
+            log.warning("@@ Collections seen %r"%(colls.keys(),))
+        #@@
         self.assertEqual(len(colls), len(init_collection_keys)-2)
         id = "coll2"
         self.assertEqual(colls[id]["annal:id"],   id)
