@@ -29,19 +29,23 @@ from annalist.models.recordview     import RecordView
 from annalist.models.recordfield    import RecordField
 from annalist.models.entitytypeinfo import EntityTypeInfo
 
-from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import (
+    init_annalist_test_site, init_annalist_test_coll, create_test_coll_inheriting, resetSitedata
+    )
 from entity_testutils       import (
     create_test_user,
     create_user_permissions,
+    context_field_map,
+    context_view_field,
     context_list_entities,
     context_list_head_fields,
     context_list_item_fields
     )
 from entity_testentitydata  import (
     entity_url, entitydata_edit_url, 
-    entitydata_default_view_context_data, entitydata_default_view_form_data,
+    entitydata_default_view_form_data,
     )
 
 #   -----------------------------------------------------------------------------
@@ -82,7 +86,7 @@ test_upload_file_view_create_values = (
     , 'rdfs:label':                 "test_upload_view label"
     , 'rdfs:comment':               "test_upload_view comment"
     , 'annal:record_type':          ""
-    , 'annal:add_field':            "yes"
+    # , 'annal:add_field':            "yes"
     , 'annal:view_fields':
       [ { 'annal:field_id':             "Entity_id"
         , 'annal:field_placement':      "small:0,12;medium:0,6"
@@ -104,7 +108,7 @@ test_upload_image_view_create_values = (
     , 'rdfs:label':                 "test_upload_image_view label"
     , 'rdfs:comment':               "test_upload_image_view comment"
     , 'annal:record_type':          ""
-    , 'annal:add_field':            "yes"
+    # , 'annal:add_field':            "yes"
     , 'annal:view_fields':
       [ { 'annal:field_id':             "Entity_id"
         , 'annal:field_placement':      "small:0,12;medium:0,6"
@@ -126,7 +130,7 @@ test_reference_view_create_values = (
     , 'rdfs:label':                 "test_reference_view label"
     , 'rdfs:comment':               "test_reference_view comment"
     , 'annal:record_type':          ""
-    , 'annal:add_field':            "yes"
+    # , 'annal:add_field':            "yes"
     , 'annal:view_fields':
       [ { 'annal:field_id':             "Entity_id"
         , 'annal:field_placement':      "small:0,12;medium:0,6"
@@ -148,7 +152,7 @@ test_image_ref_view_create_values = (
     , 'rdfs:label':                 "test_image_view label"
     , 'rdfs:comment':               "test_image_view comment"
     , 'annal:record_type':          ""
-    , 'annal:add_field':            "yes"
+    # , 'annal:add_field':            "yes"
     , 'annal:view_fields':
       [ { 'annal:field_id':             "Entity_id"
         , 'annal:field_placement':      "small:0,12;medium:0,6"
@@ -170,7 +174,7 @@ test_image_view_create_values = (
     , 'rdfs:label':                 "test_image_view label"
     , 'rdfs:comment':               "test_image_view comment"
     , 'annal:record_type':          ""
-    , 'annal:add_field':            "yes"
+    # , 'annal:add_field':            "yes"
     , 'annal:view_fields':
       [ { 'annal:field_id':             "Entity_id"
         , 'annal:field_placement':      "small:0,12;medium:0,6"
@@ -195,7 +199,7 @@ test_upload_file_field_create_values = (
     , 'annal:property_uri':             "test:upload"
     , 'annal:field_render_type':        "FileUpload"
     , 'annal:field_value_mode':         "Value_upload"
-    , 'annal:field_target_type':        "annal:Richtext"
+    , 'annal:field_value_type':        "annal:Richtext"
     , 'annal:placeholder':              "(File to upload)"
     , 'annal:default_value':            ""
     })
@@ -208,7 +212,7 @@ test_upload_image_field_create_values = (
     , 'annal:property_uri':             "test:upload"
     , 'annal:field_render_type':        "FileUpload"
     , 'annal:field_value_mode':         "Value_upload"
-    , 'annal:field_target_type':        "annal:Image"
+    , 'annal:field_value_type':        "annal:Image"
     , 'annal:placeholder':              "(Image to upload)"
     , 'annal:default_value':            ""
     })
@@ -221,7 +225,7 @@ test_reference_field_create_values = (
     , 'annal:property_uri':             "test:reference"
     , 'annal:field_render_type':        "URILink"
     , 'annal:field_value_mode':         "Value_field"
-    , 'annal:field_target_type':        "annal:Identifier"
+    , 'annal:field_value_type':        "annal:Identifier"
     , 'annal:field_ref_type':           "testupltype"
     , 'annal:field_ref_restriction':    "ALL"
     , 'annal:field_ref_field':          "test:upload"
@@ -237,7 +241,7 @@ test_image_ref_field_create_values = (
     , 'annal:property_uri':             "test:reference"
     , 'annal:field_render_type':        "RefImage"
     , 'annal:field_value_mode':         "Value_field"
-    , 'annal:field_target_type':        "annal:Identifier"
+    , 'annal:field_value_type':        "annal:Identifier"
     , 'annal:field_ref_type':           "testupltype"
     , 'annal:field_ref_restriction':    "ALL"
     , 'annal:field_ref_field':          "test:upload"
@@ -253,7 +257,7 @@ test_image_field_create_values = (
     , 'annal:property_uri':             "test:image"
     , 'annal:field_render_type':        "RefImage"
     , 'annal:field_value_mode':         "Value_upload"
-    , 'annal:field_target_type':        "annal:Image"
+    , 'annal:field_value_type':        "annal:Image"
     , 'annal:placeholder':              "(Image to upload)"
     , 'annal:default_value':            ""
     })
@@ -263,8 +267,8 @@ def test_upload_file_field_value():
         { "resource_name":              "upl_field.md"
         , "resource_type":              "text/markdown"
         , "upload_name":                "upl_field"
-        , "uploaded_size":              2957
-        , "uploaded_file":              "README.md"
+        , "uploaded_size":              137
+        , "uploaded_file":              "testdatafile.md"
         })
 
 def test_upload_image_field_value():
@@ -318,7 +322,7 @@ class UploadResourceTest(AnnalistTestCase):
     """
 
     def setUp(self):
-        self.filepath  = "%s/README.md"%TestBaseDir
+        self.filepath  = "%s/testdatafile.md"%TestBaseDir
         self.fileuri   = "file://"+self.filepath
         self.imagepath = "%s/test-image.jpg"%TestBaseDir
         self.imageuri  = "file://"+self.imagepath
@@ -377,21 +381,21 @@ class UploadResourceTest(AnnalistTestCase):
         test_entity_ids = ("test1", "test2")
         test_entity_ids = ("test1",)
         self.test_upl_type_info = EntityTypeInfo(
-            self.testsite, self.testcoll, "testupltype", create_typedata=True
+            self.testcoll, "testupltype", create_typedata=True
             )
         for entity_id in test_entity_ids:
             self.test_upl_type_info.create_entity(
                 entity_id, test_imp_entity_create_values(entity_id)
                 )
         self.test_ref_type_info = EntityTypeInfo(
-            self.testsite, self.testcoll, "testreftype", create_typedata=True
+            self.testcoll, "testreftype", create_typedata=True
             )
         for entity_id in test_entity_ids:
             self.test_ref_type_info.create_entity(
                 entity_id, test_ref_entity_create_values(entity_id)
                 )
         self.test_img_type_info = EntityTypeInfo(
-            self.testsite, self.testcoll, "testimgtype", create_typedata=True
+            self.testcoll, "testimgtype", create_typedata=True
             )
         for entity_id in test_entity_ids:
             self.test_img_type_info.create_entity(
@@ -410,7 +414,7 @@ class UploadResourceTest(AnnalistTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        resetSitedata()
+        resetSitedata(scope="collections")      # @@checkme@@
         return
 
     # Utility functions
@@ -444,7 +448,7 @@ class UploadResourceTest(AnnalistTestCase):
         resource_fileobj.close()
         testobj1.close()
         # Read back both and compare
-        siteobj = open(TestBaseDir+"/README.md", "rb")
+        siteobj = open(TestBaseDir+"/testdatafile.md", "rb")
         testobj = self.test_upl_type_info.get_fileobj(
             "test1", "test1res", "annal:Richtext", resource_type, "rb"
             )
@@ -454,7 +458,7 @@ class UploadResourceTest(AnnalistTestCase):
     def test_upload_file_resource(self):
         # See https://docs.djangoproject.com/en/1.7/topics/testing/tools/#django.test.Client.post
         with open(self.filepath) as fp:
-            f = entitydata_default_view_form_data(entity_id="test1", action="edit")
+            f = entitydata_default_view_form_data(type_id="testupltype", entity_id="test1", action="edit")
             f['upl_field'] = fp     # Upload file with submission
             u = entitydata_edit_url("edit", "testcoll", "testupltype", entity_id="test1", view_id="testuplfileview")
             r = self.client.post(u, f)
@@ -464,16 +468,16 @@ class UploadResourceTest(AnnalistTestCase):
         r = self.client.get(u)
         # Test context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        i = 3
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_upload_file")
-        self.assertDictionaryMatch(r.context['fields'][i].field_value, test_upload_file_field_value())
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
+        self.assertEqual(f3.field_id,     "Test_upload_file")
+        self.assertDictionaryMatch(f3.field_value, test_upload_file_field_value())
         # Read back and compare entity resource just created
         siteobj = open(self.filepath, "rb")
         testobj = self.test_upl_type_info.get_fileobj(
@@ -485,7 +489,7 @@ class UploadResourceTest(AnnalistTestCase):
     def test_reference_uploaded_resource(self):
         # Create uploaded resource (see previous test)
         with open(self.filepath) as fp:
-            f = entitydata_default_view_form_data(entity_id="test1", action="edit")
+            f = entitydata_default_view_form_data(type_id="testupltype", entity_id="test1", action="edit")
             f['upl_field'] = fp     # Upload file with submission
             u = entitydata_edit_url(
                 "view", "testcoll", "testupltype", entity_id="test1", view_id="testuplfileview"
@@ -502,33 +506,33 @@ class UploadResourceTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         # Check display context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        self.assertEqual(r.context['fields'][i].field_value,  "test_ref_entity test1 label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        self.assertEqual(r.context['fields'][i].field_value,  "test_ref_entity test1 comment")
-        i = 3
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        self.assertEqual(f1.field_value,  "test_ref_entity test1 label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        self.assertEqual(f2.field_value,  "test_ref_entity test1 comment")
+        f3 = context_view_field(r.context, 3, 0)
         basepath = TestBasePath + "/c/testcoll/d/testupltype/"
-        # print "\n*****\n"+repr(r.context['fields'][i].target_value)+"\n*****\n"
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_reference")
-        self.assertEqual(r.context['fields'][i].field_value,        "testupltype/test1")
-        self.assertEqual(r.context['fields'][i].field_value_link,   basepath+"test1/")
-        self.assertEqual(r.context['fields'][i].target_value['upload_name'],   "upl_field")
-        self.assertEqual(r.context['fields'][i].target_value['resource_name'], "upl_field.md")
-        self.assertEqual(r.context['fields'][i].target_value['resource_type'], "text/markdown")
-        self.assertEqual(r.context['fields'][i].target_value['uploaded_file'], "README.md")
-        self.assertEqual(r.context['fields'][i].target_value['uploaded_size'], 2957)
-        self.assertEqual(r.context['fields'][i].target_value_link,  basepath+"test1/upl_field.md")
+        # print "\n*****\n"+repr(context_view_field(r.context, i, 0).target_value)+"\n*****\n"
+        self.assertEqual(f3.field_id,     "Test_reference")
+        self.assertEqual(f3.field_value,        "testupltype/test1")
+        self.assertEqual(f3.field_value_link,   basepath+"test1/")
+        self.assertEqual(f3.target_value['upload_name'],   "upl_field")
+        self.assertEqual(f3.target_value['resource_name'], "upl_field.md")
+        self.assertEqual(f3.target_value['resource_type'], "text/markdown")
+        self.assertEqual(f3.target_value['uploaded_file'], "testdatafile.md")
+        self.assertEqual(f3.target_value['uploaded_size'], 137)
+        self.assertEqual(f3.target_value_link,  basepath+"test1/upl_field.md")
         return
 
     def test_upload_image_resource(self):
         # See https://docs.djangoproject.com/en/1.7/topics/testing/tools/#django.test.Client.post
         with open(self.imagepath) as fp:
-            f = entitydata_default_view_form_data(entity_id="test1", action="edit")
+            f = entitydata_default_view_form_data(type_id="testupltype", entity_id="test1", action="edit")
             f['upl_field'] = fp     # Upload file with submission
             u = entitydata_edit_url("edit", "testcoll", "testupltype", entity_id="test1", view_id="testuplimageview")
             r = self.client.post(u, f)
@@ -538,16 +542,16 @@ class UploadResourceTest(AnnalistTestCase):
         r = self.client.get(u)
         # Test context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        i = 3
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_upload_image")
-        self.assertDictionaryMatch(r.context['fields'][i].field_value, test_upload_image_field_value())
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
+        self.assertEqual(f3.field_id,     "Test_upload_image")
+        self.assertDictionaryMatch(f3.field_value, test_upload_image_field_value())
         # Read back and compare entity resource just created
         siteobj = open(self.imagepath, "rb")
         testobj = self.test_upl_type_info.get_fileobj(
@@ -567,27 +571,27 @@ class UploadResourceTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         # Check display context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        self.assertEqual(r.context['fields'][i].field_value,  "test_ref_entity test1 label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        self.assertEqual(r.context['fields'][i].field_value,  "test_ref_entity test1 comment")
-        i = 3
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        self.assertEqual(f1.field_value,  "test_ref_entity test1 label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        self.assertEqual(f2.field_value,  "test_ref_entity test1 comment")
+        f3 = context_view_field(r.context, 3, 0)
         basepath = TestBasePath + "/c/testcoll/d/testupltype/"
-        # print "\n*****\n"+repr(r.context['fields'][i].target_value)+"\n*****\n"
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_image_ref")
-        self.assertEqual(r.context['fields'][i].field_value,        "testupltype/test1")
-        self.assertEqual(r.context['fields'][i].field_value_link,   basepath+"test1/")
-        self.assertEqual(r.context['fields'][i].target_value['upload_name'],   "upl_field")
-        self.assertEqual(r.context['fields'][i].target_value['resource_name'], "upl_field.jpg")
-        self.assertEqual(r.context['fields'][i].target_value['resource_type'], "image/jpeg")
-        self.assertEqual(r.context['fields'][i].target_value['uploaded_file'], "test-image.jpg")
-        self.assertEqual(r.context['fields'][i].target_value['uploaded_size'], 1547926)
-        self.assertEqual(r.context['fields'][i].target_value_link,  basepath+"test1/upl_field.jpg")
+        # print "\n*****\n"+repr(context_view_field(r.context, i, 0).target_value)+"\n*****\n"
+        self.assertEqual(f3.field_id,     "Test_image_ref")
+        self.assertEqual(f3.field_value,        "testupltype/test1")
+        self.assertEqual(f3.field_value_link,   basepath+"test1/")
+        self.assertEqual(f3.target_value['upload_name'],   "upl_field")
+        self.assertEqual(f3.target_value['resource_name'], "upl_field.jpg")
+        self.assertEqual(f3.target_value['resource_type'], "image/jpeg")
+        self.assertEqual(f3.target_value['uploaded_file'], "test-image.jpg")
+        self.assertEqual(f3.target_value['uploaded_size'], 1547926)
+        self.assertEqual(f3.target_value_link,  basepath+"test1/upl_field.jpg")
         # Check for rendered image link
         # log.info(r.content)
         field_details = (
@@ -596,29 +600,30 @@ class UploadResourceTest(AnnalistTestCase):
             , "type_id":    "testupltype"
             , "entity_id":  "test1"
             , "field_id":   "upl_field"
+            , "tooltip":    ""
             })
-        img_element = (
-            """<div class="small-12 columns"> """+
-              """<div class="row view-value-row"> """+
-                """<div class="view-label small-12 medium-2 columns"> """+
-                  """<span>test_image_ref_field label</span> """+
-                """</div> """+
-                """<div class="view-value small-12 medium-10 columns"> """+
-                  """<a href="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" target="_blank"> """+
-                    """<img src="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" """+
-                    """     alt="Image at '%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg'" /> """+
-                  """</a> """+
-                """</div> """+
-              """</div> """+
-            """</div> """
-            )%field_details
+        img_element = """
+            <div class="small-12 columns" %(tooltip)s>
+              <div class="row view-value-row">
+                <div class="view-label small-12 medium-2 columns">
+                  <span>test_image_ref_field label</span>
+                </div>
+                <div class="view-value small-12 medium-10 columns">
+                  <a href="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" target="_blank">
+                    <img src="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg"
+                         alt="Image at '%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg'" />
+                  </a>
+                </div>
+              </div>
+            </div>
+            """%field_details
         self.assertContains(r, img_element, html=True)
         return
 
     def test_image_edit_field(self):
         # Upload to an image view field
         with open(self.imagepath) as fp:
-            f = entitydata_default_view_form_data(entity_id="test1", action="edit")
+            f = entitydata_default_view_form_data(type_id="testimgtype", entity_id="test1", action="edit")
             f['img_field'] = fp     # Upload file with submission
             u = entitydata_edit_url("edit", "testcoll", "testimgtype", entity_id="test1", view_id="testimgview")
             r = self.client.post(u, f)
@@ -635,17 +640,18 @@ class UploadResourceTest(AnnalistTestCase):
         # Retrieve updated form
         r = self.client.get(u)
         # Test context
+        # print "@@ "+context_field_map(r.context)
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        i = 3
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_image")
-        self.assertDictionaryMatch(r.context['fields'][i].field_value, test_image_field_value())
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
+        self.assertEqual(f3.field_id,     "Test_image")
+        self.assertDictionaryMatch(f3.field_value, test_image_field_value())
         return
 
     def test_image_view_field(self):
@@ -661,19 +667,19 @@ class UploadResourceTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "OK")
         # Check display context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        i = 3
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
         basepath = TestBasePath + "/c/testcoll/d/testimgtype/"
-        # print "\n*****\n"+repr(r.context['fields'][i].target_value)+"\n*****\n"
-        self.assertEqual(r.context['fields'][i].field_id,           "Test_image")
-        self.assertEqual(r.context['fields'][i].field_value,        test_image_field_value())
-        self.assertEqual(r.context['fields'][i].target_value_link,  basepath+"test1/img_field.jpg")
+        # print "\n*****\n"+repr(context_view_field(r.context, i, 0).target_value)+"\n*****\n"
+        self.assertEqual(f3.field_id,           "Test_image")
+        self.assertEqual(f3.field_value,        test_image_field_value())
+        self.assertEqual(f3.target_value_link,  basepath+"test1/img_field.jpg")
         # Check for rendered image link
         # log.info(r.content)
         field_details = (
@@ -682,22 +688,23 @@ class UploadResourceTest(AnnalistTestCase):
             , "type_id":    "testimgtype"
             , "entity_id":  "test1"
             , "field_id":   "img_field"
+            , "tooltip":    ""
             })
-        img_element = (
-            """<div class="small-12 columns"> """+
-              """<div class="row view-value-row"> """+
-                """<div class="view-label small-12 medium-2 columns"> """+
-                  """<span>test_image_field label</span> """+
-                """</div> """+
-                """<div class="view-value small-12 medium-10 columns"> """+
-                  """<a href="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" target="_blank"> """+
-                    """<img src="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" """+
-                    """     alt="Image at '%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg'" /> """+
-                  """</a> """+
-                """</div> """+
-              """</div> """+
-            """</div> """
-            )%field_details
+        img_element = """
+            <div class="small-12 columns" %(tooltip)s>
+              <div class="row view-value-row">
+                <div class="view-label small-12 medium-2 columns">
+                  <span>test_image_field label</span>
+                </div>
+                <div class="view-value small-12 medium-10 columns">
+                  <a href="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg" target="_blank">
+                    <img src="%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg"
+                         alt="Image at '%(basepath)s/c/%(coll_id)s/d/%(type_id)s/%(entity_id)s/%(field_id)s.jpg'" />
+                  </a>
+                </div>
+              </div>
+            </div>
+            """%field_details
         self.assertContains(r, img_element, html=True)
         return
 
@@ -708,7 +715,7 @@ class UploadResourceTest(AnnalistTestCase):
         self.test_image_edit_field()
 
         # Edit entity
-        f = entitydata_default_view_form_data(entity_id="test1", action="edit", update="Updated")
+        f = entitydata_default_view_form_data(type_id="testimgtype", entity_id="test1", action="edit", update="Updated")
         u = entitydata_edit_url("edit", "testcoll", "testimgtype", entity_id="test1", view_id="testimgview")
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   302)
@@ -718,17 +725,17 @@ class UploadResourceTest(AnnalistTestCase):
         r = self.client.get(u)
         # Test context
         self.assertEqual(len(r.context['fields']), 4)
-        i = 0
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_id")
-        self.assertEqual(r.context['fields'][i].field_value,  "test1")
-        i = 1
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_label")
-        self.assertEqual(r.context['fields'][i].field_value,  "Updated testcoll/testtype/test1")
-        i = 2
-        self.assertEqual(r.context['fields'][i].field_id,     "Entity_comment")
-        i = 3
-        self.assertEqual(r.context['fields'][i].field_id,     "Test_image")
-        self.assertDictionaryMatch(r.context['fields'][i].field_value, test_image_field_value())
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        self.assertEqual(f1.field_value,  "Updated testcoll/testimgtype/test1")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
+        self.assertEqual(f3.field_id,     "Test_image")
+        self.assertDictionaryMatch(f3.field_value, test_image_field_value())
 
         # Read back and compare entity resource
         siteobj = open(self.imagepath, "rb")
@@ -745,7 +752,7 @@ class UploadResourceTest(AnnalistTestCase):
         self.test_image_edit_field()
 
         # Rename entity
-        f = entitydata_default_view_form_data(orig_id="test1", entity_id="test_new", action="edit")
+        f = entitydata_default_view_form_data(type_id="testimgtype", orig_id="test1", entity_id="test_new", action="edit")
         u = entitydata_edit_url("edit", "testcoll", "testimgtype", entity_id="test1", view_id="testimgview")
         r = self.client.post(u, f)
         self.assertEqual(r.status_code,   302)
@@ -759,4 +766,82 @@ class UploadResourceTest(AnnalistTestCase):
         self.assertTrue(siteobj.read() == testobj.read(), "Renamed entity image != original")
         return
 
+    def test_inherited_image_edit(self):
+        # This tests that editing an inherited image creartes a new copy in the
+        # inheriting collection.
+
+        # Upload image
+        self.test_image_edit_field()
+
+        # Create collection inheriting uploaded image
+        testsubcoll = create_test_coll_inheriting(
+            base_coll_id="testcoll", coll_id="testsubcoll", type_id="testtype"
+            )
+        # create_test_user(testsubcoll, "testuser", "testpassword")
+        user_permissions = ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG"]
+        user_id          = "testuser"
+        user_perms = testsubcoll.create_user_permissions(
+            user_id, "mailto:%s@%s"%(user_id, TestHost),
+            "Test User",
+            "User %s: permissions for %s in collection %s"%(user_id, "Test User", testsubcoll.get_id()),
+            user_permissions)
+
+        # Get editing form
+        u = entitydata_edit_url(
+            "edit", "testsubcoll", "testimgtype", entity_id="test1", view_id="testimgview"
+            )
+        r = self.client.get(u)
+        self.assertEqual(r.status_code,   200)
+        self.assertEqual(r.reason_phrase, "OK")
+        # log.info(r.content)     #@@
+        hi1 = """<input type="hidden" name="orig_id"          value="test1" />"""
+        hi2 = """<input type="hidden" name="orig_type"        value="testimgtype" />"""
+        hi3 = """<input type="hidden" name="orig_coll"        value="testcoll" />"""
+        hi4 = """<input type="hidden" name="action"           value="edit" />"""
+        hi5 = """<input type="hidden" name="view_id"          value="testimgview" />"""
+        self.assertContains(r, hi1, html=True)
+        self.assertContains(r, hi2, html=True)
+        self.assertContains(r, hi3, html=True)
+        self.assertContains(r, hi4, html=True)
+        self.assertContains(r, hi5, html=True)
+
+        # Edit entity
+        f = entitydata_default_view_form_data(
+            coll_id="testsubcoll", type_id="testimgtype", entity_id="test1", 
+            orig_coll="testcoll", action="edit", 
+            update="Updated"
+            )
+        u = entitydata_edit_url(
+            "edit", "testsubcoll", "testimgtype", entity_id="test1", view_id="testimgview"
+            )
+        r = self.client.post(u, f)
+        self.assertEqual(r.status_code,   302)
+        self.assertEqual(r.reason_phrase, "FOUND")
+
+        # Retrieve updated form
+        r = self.client.get(u)
+        # Test context
+        self.assertEqual(len(r.context['fields']), 4)
+        f0 = context_view_field(r.context, 0, 0)
+        self.assertEqual(f0.field_id,     "Entity_id")
+        self.assertEqual(f0.field_value,  "test1")
+        f1 = context_view_field(r.context, 1, 0)
+        self.assertEqual(f1.field_id,     "Entity_label")
+        self.assertEqual(f1.field_value,  "Updated testsubcoll/testimgtype/test1")
+        f2 = context_view_field(r.context, 2, 0)
+        self.assertEqual(f2.field_id,     "Entity_comment")
+        f3 = context_view_field(r.context, 3, 0)
+        self.assertEqual(f3.field_id,     "Test_image")
+        self.assertDictionaryMatch(f3.field_value, test_image_field_value())
+
+        # Read back and compare entity resource
+        inherited_test_img_type_info = EntityTypeInfo(
+            testsubcoll, "testimgtype", create_typedata=True
+            )
+        siteobj = open(self.imagepath, "rb")
+        testobj = inherited_test_img_type_info.get_fileobj(
+            "test1", "img_field", "annal:Image", "image/jpeg", "rb"
+            )
+        self.assertTrue(siteobj.read() == testobj.read(), "Edited entity image != original")
+
 # End.
