@@ -15,11 +15,13 @@ import os.path
 
 from django.conf import settings
 
+import utils
+
 from annalist.layout import Layout
 
 import annalist.util
 
-import annalist.views.fields.render_utils
+import annalist.views.fields.find_renderers
 import annalist.views.fields.render_placement
 
 test_layout     = Layout(settings.BASE_DATA_DIR)    # e.g. ".../sampledata/data/"
@@ -36,11 +38,13 @@ def load_tests(loader, tests, ignore):
     if os.name == "posix":
         # The doctest stuff doesn't seem to work on Windows
         # (These add a total of 12 tests to the overall test)
+        tests.addTests(doctest.DocTestSuite(utils.uri_builder))
         tests.addTests(doctest.DocTestSuite(annalist.util))
         tests.addTests(doctest.DocTestSuite(annalist.identifiers))
         tests.addTests(doctest.DocTestSuite(annalist.views.uri_builder))
+        tests.addTests(doctest.DocTestSuite(annalist.views.displayinfo))
         tests.addTests(doctest.DocTestSuite(annalist.views.form_utils.fieldchoice))
-        tests.addTests(doctest.DocTestSuite(annalist.views.fields.render_utils))
+        tests.addTests(doctest.DocTestSuite(annalist.views.fields.find_renderers))
         tests.addTests(doctest.DocTestSuite(annalist.views.fields.bound_field))
         tests.addTests(doctest.DocTestSuite(annalist.views.fields.render_placement))
         tests.addTests(doctest.DocTestSuite(annalist.models.entityfinder))
