@@ -26,13 +26,13 @@ from django.conf import settings
 
 from annalist.identifiers import ANNAL
 
-def valid_id(id):
+def valid_id(id, reserved_ok=False):
     """
     Checks the supplied id is valid as an Annalist identifier.
 
     The main requirement is that it is valid as a URI path segment, so it can be used
     in the creation of URIs for Annalist resources.  Also, filters out reserved Annalist
-    identifiers.
+    identifiers unless reserved_ok=True parameter is provided.
 
     >>> valid_id("abcdef_1234")
     True
@@ -40,15 +40,18 @@ def valid_id(id):
     False
     >>> valid_id("_annalist_collection")
     False
+    >>> valid_id("_annalist_collection", reserved_ok=True)
+    True
     >>> valid_id("")
     False
     """
     reserved = (
         [ "_annalist_collection"
+        # , "_annalist_site"
         ])
     # cf. urls.py:
-    if id and re.match(r"\w{1,32}$", id):
-        return id not in reserved
+    if id and re.match(r"\w{1,128}$", id):
+        return reserved_ok or (id not in reserved)
     # log.warning("util.valid_id: id %s"%(id))
     return False
 
@@ -87,7 +90,7 @@ def extract_entity_id(eid):
 
 def fill_type_entity_id(eid, default_type_id=None):
     """
-    Assemble a type+entity comopisite identifier based on a supplied id string.
+    Assemble a type+entity composite identifier based on a supplied id string.
     If the string does not already include a type_id value, the supplied default
     is used.
     """
@@ -119,6 +122,17 @@ def make_entity_base_url(url):
     """
     return url if url.endswith("/") else url + "/"
 
+def label_from_id(id):
+    """
+    Returns a label string constructed from the suppliued Id string
+
+    Underscore characters in the Id are replaced by spaces.
+    The first character may be capirtalized.
+    """
+    temp  = id.replace('_', ' ').strip()
+    label = temp[0].upper() + temp[1:] 
+    return label
+
 def slug_from_name(filename):
     """
     Extracts a slug (id) value from a filename
@@ -198,7 +212,7 @@ def entity_dir_path(base_dir, path, filename):
                 from which data is to be read
     path        is a relative path or list of path segments within the site or 
                 resource for the site or collection from which data is to be read.
-                This value may be absent.
+                This value may be absent (an empty list).
     filename    is a file name for the data resource to be read.
 
     Returns a pair containing the full directory and path names for the entity file.
@@ -292,6 +306,36 @@ def entity_url_path(baseuri, entityref):
     uri = urlparse.urljoin(baseuri, entityref)
     return urlparse.urlparse(uri).path
 
+def make_resource_url(baseuri, entityref, resourceref):
+    """
+    Build a URL for an entity resource that is based on a supplied base URI
+    and entity reference, but including the supplied resource name reference
+    (i.e. filename)
+
+    This function preserves any query parameters from the supplied entityurl.
+
+    >>> make_resource_url("http://example.org/foo/", "/bar/stuff", "entity.ref")
+    'http://example.org/bar/entity.ref'
+    >>> make_resource_url("http://example.org/foo/", "/bar/stuff?query=val", "entity.ref")
+    'http://example.org/bar/entity.ref?query=val'
+    """
+    url          = urlparse.urljoin(baseuri, entityref)
+    resource_url = urlparse.urljoin(url, make_resource_ref_query(entityref, resourceref))
+    return resource_url
+
+def make_resource_ref_query(entityref, resourceref):
+    """
+    Returns `resourceref` with the query component (if any) from entityref.
+
+    This is used to generate a resource reference that is the supplied resource ref
+    treated as relative to the supplied entityref, including any query parameters
+    included in entityref.
+    """
+    query = urlparse.urlsplit(entityref).query
+    if query != "":
+        query = "?" + query
+    return urlparse.urljoin(resourceref, query)
+
 def strip_comments(f):
     """
     Returns a file-like object that returns content from the supplied file-like
@@ -398,6 +442,8 @@ def replacetree(src, tgt):
     Work-around for python problem with shutils tree copy functions on Windows.
     See: http://stackoverflow.com/questions/23924223/
     """
+    # print "@@ replacetree src %s"%(src,)
+    # print "@@ replacetree tgt %s"%(tgt,)
     if os.path.exists(tgt):
         removetree(tgt)
     shutil.copytree(src, tgt)
@@ -464,7 +510,7 @@ def download_url_to_fileobj(url, fileobj=None):
         r.close()
     return (resource_url, resource_type)
 
-# Update MIME typesreturned by open_url when opening a file
+# Update MIME types returned by open_url when opening a file
 # @@TODO: unify logic with resourcetypes module, and do all MIME type wrangling there
 import mimetypes
 mimetypes.init()
