@@ -10,49 +10,42 @@ import logging
 log = logging.getLogger(__name__)
 
 from django.conf import settings
-from django.http                import HttpResponse
-from django.http                import HttpResponseRedirect
-from django.core.urlresolvers   import resolve, reverse
+from django.http                    import HttpResponse
+from django.http                    import HttpResponseRedirect
+from django.core.urlresolvers       import resolve, reverse
 
-from annalist                   import message
-from annalist.exceptions        import Annalist_Error
+from annalist                       import message
+from annalist                       import layout
+from annalist.identifiers           import ANNAL, RDFS
+from annalist.exceptions            import Annalist_Error
 
-import annalist.models.entitytypeinfo as entitytypeinfo
-from annalist.models.site       import Site
-from annalist.models.collection import Collection
-from annalist.models.recordtype import RecordType
-from annalist.models.recordview import RecordView
-from annalist.models.recordlist import RecordList
+from annalist.models                import entitytypeinfo
+from annalist.models.site           import Site
+from annalist.models.collection     import Collection
+from annalist.models.recordtype     import RecordType
+from annalist.models.recordview     import RecordView
+from annalist.models.recordlist     import RecordList
+from annalist.models.collectiondata import migrate_coll_data, migrate_coll_config_dirs
 
-from annalist.views.uri_builder import uri_with_params
-from annalist.views.displayinfo import DisplayInfo
-from annalist.views.generic     import AnnalistGenericView
-from annalist.views.confirm     import ConfirmView
+from annalist.views.uri_builder     import uri_with_params
+from annalist.views.uri_builder     import (
+    uri_param_dict,
+    scope_params, continuation_params,
+    uri_params, uri_with_params,
+    )
+from annalist.views.displayinfo     import DisplayInfo
+from annalist.views.generic         import AnnalistGenericView
+from annalist.views.confirm         import ConfirmView
 
 
 class CollectionView(AnnalistGenericView):
     """
     View class to handle requests to display an Annalist collection.
 
-    Redirects to default list view.
+    Redirects to a default view or list view.
     """
     def __init__(self):
         super(CollectionView, self).__init__()
-        return
-
-    def get(self, request, coll_id):
-        """
-        Form for displaying the current collection 
-        """
-        return HttpResponseRedirect(self.view_uri("AnnalistEntityDefaultListAll", coll_id=coll_id))
-
-
-class CollectionEditView(AnnalistGenericView):
-    """
-    View class to handle requests to an Annalist collection edit URI
-    """
-    def __init__(self):
-        super(CollectionEditView, self).__init__()
         self.default_continuation = self.view_uri("AnnalistSiteView")
         return
 
@@ -63,7 +56,60 @@ class CollectionEditView(AnnalistGenericView):
         viewinfo = DisplayInfo(self, action, request_dict, self.default_continuation)
         viewinfo.get_site_info(self.get_request_host())
         viewinfo.get_coll_info(coll_id)
-        viewinfo.get_type_info(entitytypeinfo.COLL_ID)
+        viewinfo.check_authorization(action)
+        self.default_continuation = self.view_uri("AnnalistCollectionView", coll_id=coll_id)
+        if not viewinfo.http_response:
+            errs = migrate_coll_config_dirs(viewinfo.collection)
+            if errs:
+                viewinfo.report_error("\n".join(errs))
+        return viewinfo
+
+    def get(self, request, coll_id):
+        """
+        Display the specified collection 
+        """
+        viewinfo = self.collection_view_setup(coll_id, "view", request.GET.dict())
+        if viewinfo.http_response:
+            log.debug(
+                "CollectionView.get: response %d: %s"%
+                (viewinfo.http_response.status_code, viewinfo.http_response.reason_phrase)
+                )
+            return viewinfo.http_response
+        # Flush caches and regenerate JSON-LD context when invoking top-level view of collection
+        viewinfo.flush_collection_caches()
+        viewinfo.collection.generate_coll_jsonld_context()
+        # Select and display view of collection
+        default_view, default_type, default_entity = viewinfo.get_default_view_type_entity()
+        if default_view and default_type and default_entity:
+            redirect_uri = self.view_uri(
+                "AnnalistEntityDataView",
+                coll_id=coll_id, 
+                view_id=default_view, type_id=default_type, entity_id=default_entity
+                )
+        else:
+            redirect_uri = self.view_uri(
+                "AnnalistEntityDefaultListAll", 
+                coll_id=coll_id
+                )
+        return HttpResponseRedirect(redirect_uri)
+
+class CollectionEditView(AnnalistGenericView):
+    """
+    View class to handle requests to an Annalist collection edit URI
+    """
+    def __init__(self):
+        super(CollectionEditView, self).__init__()
+        self.default_continuation = self.view_uri("AnnalistSiteView")
+        return
+
+    def collection_edit_setup(self, coll_id, action, request_dict):
+        """
+        Assemble display information for collection view request handler
+        """
+        viewinfo = DisplayInfo(self, action, request_dict, self.default_continuation)
+        viewinfo.get_site_info(self.get_request_host())
+        viewinfo.get_coll_info(coll_id)
+        viewinfo.get_request_type_info(entitytypeinfo.COLL_ID)
         viewinfo.check_authorization(action)
         self.default_continuation = self.view_uri("AnnalistCollectionView", coll_id=coll_id)
         return viewinfo
@@ -80,17 +126,22 @@ class CollectionEditView(AnnalistGenericView):
             coll = viewinfo.collection
             context = (
                 { 'continuation_url':   viewinfo.get_continuation_url() or ""
+                , 'continuation_param': viewinfo.get_continuation_param()
                 , 'types':              sorted(coll.types(altscope=None), key=get_id)
                 , 'lists':              sorted(coll.lists(altscope=None), key=get_id)
                 , 'views':              sorted(coll.views(altscope=None), key=get_id)
                 , 'select_rows':        "6"
                 })
             context.update(viewinfo.context_data())
+            context['heading'] = "Customize collection: %(coll_label)s"%context
             return context
         continuation_url = None
-        viewinfo = self.collection_view_setup(coll_id, "view", request.GET.dict())
+        # View permission only to display form, 
+        # as it presents useful information even when not editing.
+        viewinfo = self.collection_edit_setup(coll_id, "view", request.GET.dict())
         if viewinfo.http_response:
             return viewinfo.http_response
+        self.help_markdown = viewinfo.collection.get(RDFS.CURIE.comment, None)
         return (
             self.render_html(resultdata(viewinfo), 'annalist_collection_edit.html') or 
             self.error(self.error406values())
@@ -109,11 +160,38 @@ class CollectionEditView(AnnalistGenericView):
         #       URI-wrangling?
         redirect_uri  = None
         http_response = None
-        viewinfo = self.collection_view_setup(coll_id, "config", request.POST.dict())
+        viewinfo = self.collection_edit_setup(coll_id, "config", request.POST.dict())
         if viewinfo.http_response:
             return viewinfo.http_response
         if "close" in request.POST:
             redirect_uri = viewinfo.get_continuation_next()
+        if "migrate" in request.POST:
+            msgs = migrate_coll_data(viewinfo.collection)
+            msg_vals = {'id': coll_id}
+            if msgs:
+                for msg in msgs:
+                    log.warning(msg)
+                err = message.MIGRATE_COLLECTION_ERROR%msg_vals
+                msg = "\n".join([err]+msgs)
+                log.error(msg)
+                http_response = self.error(dict(self.error500values(),message=msg))
+            else:
+                # Redisplay current page with completion message
+                viewuri = self.get_request_path()
+                http_response = self.redirect_info(
+                    self.get_request_path(),
+                    view_params=continuation_params(uri_param_dict(viewuri)),
+                    info_message=message.MIGRATED_COLLECTION_DATA%msg_vals
+                    )
+            return http_response
+        # Edit collection metadata
+        if "metadata" in request.POST:
+            redirect_uri = self.item_edit_uri(
+                layout.SITEDATA_ID, "_coll", "Collection_view", coll_id, 
+                message.NO_COLLECTION_METADATA, 
+                viewinfo.get_continuation_here(),
+                viewinfo.get_continuation_url()
+                )
         # Record types
         type_id = request.POST.get('typelist', None)
         if "type_new" in request.POST:
@@ -136,12 +214,15 @@ class CollectionEditView(AnnalistGenericView):
                 viewinfo.get_continuation_url()
                 )
         if "type_delete" in request.POST:
-            redirect_uri, http_response = self.item_delete_response(
-                coll_id, type_id, 
-                message.NO_TYPE_FOR_DELETE, 
-                message.REMOVE_RECORD_TYPE, 
-                "AnnalistRecordTypeDeleteView",
-                viewinfo.get_continuation_url()
+            http_response = viewinfo.confirm_delete_entity_response(
+                layout.TYPE_TYPEID, type_id, 
+                self.view_uri("AnnalistRecordTypeDeleteView", coll_id=coll_id),
+                form_action_field="type_delete",
+                form_value_field="typelist",
+                response_messages=
+                    { "no_entity":          message.NO_TYPE_FOR_DELETE
+                    , "confirm_completion": message.REMOVE_RECORD_TYPE
+                    }
                 )
         # List views
         list_id = request.POST.get('listlist', None)
@@ -165,12 +246,15 @@ class CollectionEditView(AnnalistGenericView):
                 viewinfo.get_continuation_url()
                 )
         if "list_delete" in request.POST:
-            redirect_uri, http_response = self.item_delete_response(
-                coll_id, list_id, 
-                message.NO_LIST_FOR_DELETE, 
-                message.REMOVE_RECORD_LIST, 
-                "AnnalistRecordListDeleteView",
-                viewinfo.get_continuation_url()
+            http_response = viewinfo.confirm_delete_entity_response(
+                layout.LIST_TYPEID, list_id, 
+                self.view_uri("AnnalistRecordListDeleteView", coll_id=coll_id),
+                form_action_field="list_delete",
+                form_value_field="listlist",
+                response_messages=
+                    { "no_entity":          message.NO_LIST_FOR_DELETE
+                    , "confirm_completion": message.REMOVE_RECORD_LIST
+                    }
                 )
         # Record views
         view_id = request.POST.get('viewlist', None)
@@ -194,19 +278,29 @@ class CollectionEditView(AnnalistGenericView):
                 viewinfo.get_continuation_url()
                 )
         if "view_delete" in request.POST:
-            redirect_uri, http_response = self.item_delete_response(
-                coll_id, view_id, 
-                message.NO_VIEW_FOR_DELETE, 
-                message.REMOVE_RECORD_VIEW, 
-                "AnnalistRecordViewDeleteView",
-                viewinfo.get_continuation_url()
+            http_response = viewinfo.confirm_delete_entity_response(
+                layout.VIEW_TYPEID, view_id, 
+                self.view_uri("AnnalistRecordViewDeleteView", coll_id=coll_id),
+                form_action_field="view_delete",
+                form_value_field="viewlist",
+                response_messages=
+                    { "no_entity":          message.NO_VIEW_FOR_DELETE
+                    , "confirm_completion": message.REMOVE_RECORD_VIEW
+                    }
                 )
         # Invoke selected view and/or render status response
         if redirect_uri:
             http_response = http_response or HttpResponseRedirect(redirect_uri)
         if http_response:
             return http_response
-        raise Annalist_Error(request.POST, "Unexpected values in POST to "+self.get_request_path())
+        e = Annalist_Error(request.POST, "Unexpected values in POST to "+self.get_request_path())
+        log.exception(str(e))
+        return self.error(
+            dict(self.error500values(),
+                message=str(e)+" - see server log for details"
+                )
+            )
+        # raise Annalist_Error(request.POST, "Unexpected values in POST to "+self.get_request_path())
 
     # POST helper methods
 
@@ -227,12 +321,16 @@ class CollectionEditView(AnnalistGenericView):
         #       and is used as part of the URL used to redisplay the current 
         #       page with an error message.
         # @@TODO:  pass in viewinfo rather than continuation URLs
-        redirect_uri = (
-            self.check_value_supplied(
-                entity_id, no_entity_msg, 
-                continuation_url
+        if not entity_id:
+            continuation_url_dict = {}
+            if continuation_url:
+                continuation_url_dict = {'continuation_url': continuation_url}
+            return uri_with_params(
+                self.get_request_path(), 
+                self.error_params(no_entity_msg),
+                continuation_url_dict
                 )
-            or
+        redirect_uri = (
             uri_with_params(
                 self.view_uri("AnnalistEntityEditView", action=action, 
                     coll_id=coll_id, view_id=view_id, type_id=type_id, entity_id=entity_id
@@ -268,36 +366,4 @@ class CollectionEditView(AnnalistGenericView):
             "edit")
         return redirect_uri
 
-    def item_delete_response(self, 
-            coll_id, entity_id, 
-            no_entity_msg, 
-            confirm_msg, 
-            complete_action_view,
-            continuation_url):
-        redirect_uri  = None
-        http_response = None
-        if entity_id:
-            # Get user to confirm action before actually doing it
-            confirmed_action_uri = self.view_uri(
-                complete_action_view, coll_id=coll_id
-                )
-            message_vals = {'id': entity_id, 'coll_id': coll_id}
-            http_response = (
-                ConfirmView.render_form(self.request,
-                    action_description=     confirm_msg%message_vals,
-                    confirmed_action_uri=   confirmed_action_uri,
-                    action_params=          self.request.POST,
-                    cancel_action_uri=      self.get_request_path(),
-                    title=                  self.site_data()["title"]
-                    )
-                )
-        else:
-            redirect_uri = (
-                self.check_value_supplied(
-                    entity_id, no_entity_msg,
-                    continuation_url
-                    )
-                )
-        return redirect_uri, http_response
-
 # End.
