@@ -47,18 +47,20 @@ class EntityDeleteConfirmedBaseView(AnnalistGenericView):
         viewinfo = DisplayInfo(self, "delete", request_params, continuation_url)
         viewinfo.get_site_info(self.get_request_host())
         viewinfo.get_coll_info(coll_id)
-        viewinfo.get_type_info(type_id)
+        viewinfo.get_request_type_info(type_id)
         viewinfo.check_authorization("delete")
         if viewinfo.http_response:
             return viewinfo.http_response
-        typeinfo     = viewinfo.entitytypeinfo
+        typeinfo     = viewinfo.curr_typeinfo
         message_vals = {'id': entity_id, 'type_id': type_id, 'coll_id': coll_id}
         messages     = (
             { 'entity_removed': typeinfo.entitymessages['entity_removed']%message_vals
             })
         err = typeinfo.entityclass.remove(typeinfo.entityparent, entity_id)
         if err:
-            return self.redirect_error(continuation_url, continuation_url_params, error_message=str(err))
+            return self.redirect_error(
+                continuation_url, continuation_url_params, error_message=str(err)
+                )
         return self.redirect_info(
             continuation_url, continuation_url_params, 
             info_message=messages['entity_removed']
