@@ -27,6 +27,13 @@ from annalist.views.form_utils.fieldchoice      import FieldChoice
 #
 #   ----------------------------------------------------------------------------
 
+# These symbols don't display will with some languages/fonts (specifically Chinese)
+view_is_occupied  = "&block;"
+view_not_occupied = "&blk14;"
+
+option_is_occupied  = "#"
+option_not_occupied = "."
+
 # Enumerated placement options.
 #
 # '.' and '#' here are placeholders for symbols that will be used to show
@@ -49,29 +56,29 @@ placement_occupancy = OrderedDict(
     , ("small:0,12;medium:3,3", "...###...... (3/3)")
     , ("small:0,12;medium:6,3", "......###... (6/3)")
     , ("small:0,12;medium:9,3", ".........### (9/3)")
-    , ("small:0,12;medium:6,6right", "......###### (6/6 right)")
-    , ("small:0,12;medium:8,4right", "........#### (8/4 right)")
-    , ("small:0,12;medium:9,3right", ".........### (9/3 right)")
-    , ("small:0,9", "#########... (0/9 column)")
-    , ("small:3,9", "...######### (3/9 column)")
-    , ("small:0,8", "########.... (0/8 column)")
-    , ("small:4,8", "....######## (4/8 column)")
-    , ("small:0,6", "######...... (0/6 column)")
-    , ("small:3,6", "...######... (3/6 column)")
-    , ("small:6,6", "......###### (6/6 column)")
-    , ("small:0,4", "####........ (0/4 column)")
-    , ("small:4,4", "....####.... (4/4 column)")
-    , ("small:8,4", "........#### (8/4 column)")
-    , ("small:0,3", "###......... (0/3 column)")
-    , ("small:3,3", "...###...... (3/3 column)")
-    , ("small:6,3", "......###... (6/3 column)")
-    , ("small:9,3", ".........### (9/3 column)")
+    , ("small:0,12;medium:6,6right", "......###### (6/6R)")
+    , ("small:0,12;medium:8,4right", "........#### (8/4R)")
+    , ("small:0,12;medium:9,3right", ".........### (9/3R)")
+    , ("small:0,9", "#########... (0/9col)")
+    , ("small:3,9", "...######### (3/9col)")
+    , ("small:0,8", "########.... (0/8col)")
+    , ("small:4,8", "....######## (4/8col)")
+    , ("small:0,6", "######...... (0/6col)")
+    , ("small:3,6", "...######... (3/6col)")
+    , ("small:6,6", "......###### (6/6col)")
+    , ("small:0,4", "####........ (0/4col)")
+    , ("small:4,4", "....####.... (4/4col)")
+    , ("small:8,4", "........#### (8/4col)")
+    , ("small:0,3", "###......... (0/3col)")
+    , ("small:3,3", "...###...... (3/3col)")
+    , ("small:6,3", "......###... (6/3col)")
+    , ("small:9,3", ".........### (9/3col)")
     ])
 
 def option_symbol(occupied):
     return (
-        "&block;" if occupied == "#" else 
-        "&blk14;" if occupied == "." else occupied
+        option_is_occupied  if occupied == "#" else 
+        option_not_occupied if occupied == "." else occupied
         )
 
 def option_body(occupancy):
@@ -80,6 +87,19 @@ def option_body(occupancy):
     """
     return "".join([ option_symbol(c) for c in occupancy ])
 
+
+def view_symbol(occupied):
+    return (
+        view_is_occupied  if occupied == "#" else 
+        view_not_occupied if occupied == "." else occupied
+        )
+
+def view_body(occupancy):
+    """
+    Returns an option body string corresponding to a supplied occupancy string
+    """
+    return "".join([ view_symbol(c) for c in occupancy ])
+
 def get_placement_options():
     return [ FieldChoice(o, label=option_body(placement_occupancy[o])) 
              for o in placement_occupancy 
@@ -93,7 +113,7 @@ def get_placement_value_option_dict():
 def get_placement_option_value_dict():
     return { option_body(placement_occupancy[o]) : o for o in placement_occupancy }
 
-def placement_display_text(placement, placeholder="(select...)"):
+def placement_opton_text(placement, placeholder="(select...)"):
     if placement in placement_occupancy:
         display_text = option_body(placement_occupancy[placement])
     elif placement == "":
@@ -102,14 +122,8 @@ def placement_display_text(placement, placeholder="(select...)"):
         display_text = placement
     return display_text
 
-def placement_display_span(placement):
-    return (
-        '''<span class="placement-text">%s</span>'''%
-        placement_display_text(placement)
-        )
-
 def placement_option(placement, placeholder, placement_selected="False"):
-    body_text = placement_display_text(placement, placeholder=placeholder)
+    body_text = placement_opton_text(placement, placeholder=placeholder)
     if placement_selected:
         selected = ''' selected="selected"'''
     else:
@@ -119,6 +133,15 @@ def placement_option(placement, placeholder, placement_selected="False"):
         (placement, selected, body_text)
         )
 
+def placement_display_span(placement):
+    if placement in placement_occupancy:
+        display_text = view_body(placement_occupancy[placement])
+    elif placement == "":
+        display_text = placeholder
+    else:
+        display_text = placement
+    return '''<span class="placement-text">%s</span>'''%display_text
+
 #   ----------------------------------------------------------------------------
 #
 #   Field placement field renderers
@@ -178,7 +201,7 @@ def get_field_placement_renderer():
     """
     Return field renderer object for field placement values
     """
-    return RenderFieldValue(
+    return RenderFieldValue("placement",
         view_renderer=placement_view_renderer(), 
         edit_renderer=placement_edit_renderer()
         )
@@ -189,11 +212,13 @@ def get_field_placement_renderer():
 #
 #   ----------------------------------------------------------------------------
 
-Placement = namedtuple("Placement", ['field', 'label', 'value'])
+LayoutOptions = namedtuple("LayoutOptions", ["s", "m", "l"])
+
+Placement = namedtuple("Placement", ['width', 'offset', 'display', 'field', 'label', 'value'])
 
 def get_placement_classes(placement):
     """
-    Returns placement classes corresponding to placement string provided.
+    Returns Placement classes corresponding to placement string provided.
 
     >>> get_placement_classes("small:0,12").field
     'small-12 columns'
@@ -201,35 +226,88 @@ def get_placement_classes(placement):
     'small-12 medium-2 columns'
     >>> get_placement_classes("small:0,12").value
     'small-12 medium-10 columns'
-    >>> get_placement_classes("medium:0,12")
-    Placement(field='small-12 columns', label='small-12 medium-2 columns', value='small-12 medium-10 columns')
-    >>> get_placement_classes("large:0,12")
-    Placement(field='small-12 columns', label='small-12 medium-2 columns', value='small-12 medium-10 columns')
+    >>> get_placement_classes("medium:0,12")                        # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=12, l=12), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-12 columns', \
+        label='small-12 medium-2 columns', \
+        value='small-12 medium-10 columns')
+    >>> get_placement_classes("large:0,12")                         # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=12, l=12), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-12 columns', \
+        label='small-12 medium-2 columns', \
+        value='small-12 medium-10 columns')
 
-    >>> get_placement_classes("small:0,12;medium:0,4")
-    Placement(field='small-12 medium-4 columns', label='small-12 medium-6 columns', value='small-12 medium-6 columns')
-    >>> get_placement_classes("small:0,12; medium:0,4")
-    Placement(field='small-12 medium-4 columns', label='small-12 medium-6 columns', value='small-12 medium-6 columns')
+    >>> get_placement_classes("small:0,12;medium:0,4")              # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=4, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-12 medium-4 columns', \
+        label='small-12 medium-6 columns', \
+        value='small-12 medium-6 columns')
+    >>> get_placement_classes("small:0,12; medium:0,4")             # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=4, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-12 medium-4 columns', \
+        label='small-12 medium-6 columns', \
+        value='small-12 medium-6 columns')
 
-    >>> get_placement_classes("small:0,12;medium:0,6;large:0,4")
-    Placement(field='small-12 medium-6 large-4 columns', label='small-12 medium-4 large-6 columns', value='small-12 medium-8 large-6 columns')
-    >>> get_placement_classes("small:0,6;medium:0,4")
-    Placement(field='small-6 medium-4 columns', label='small-12 medium-6 columns', value='small-12 medium-6 columns')
-    >>> get_placement_classes("small:0,6;medium:0,4,right")
-    Placement(field='small-6 medium-4 columns right', label='small-12 medium-6 columns', value='small-12 medium-6 columns')
-    >>> get_placement_classes("small:0,6")
-    Placement(field='small-6 columns', label='small-12 medium-4 columns', value='small-12 medium-8 columns')
+    >>> get_placement_classes("small:0,12;medium:0,6;large:0,4")    # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=6, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-12 medium-6 large-4 columns', \
+        label='small-12 medium-4 large-6 columns', \
+        value='small-12 medium-8 large-6 columns')
 
-    >>> get_placement_classes("small:0,12,hide;medium:0,4")
-    Placement(field='small-12 medium-4 columns show-for-medium-up', label='small-12 medium-6 columns', value='small-12 medium-6 columns')
+    >>> get_placement_classes("small:0,6;medium:0,4")               # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=6, m=4, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-6 medium-4 columns', \
+        label='small-12 medium-6 columns', \
+        value='small-12 medium-6 columns')
+
+    >>> get_placement_classes("small:0,6;medium:0,4,right")         # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=6, m=4, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-6 medium-4 columns right', \
+        label='small-12 medium-6 columns', \
+        value='small-12 medium-6 columns')
+
+    >>> get_placement_classes("small:0,6")                          # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=6, m=6, l=6), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=True, m=True, l=True), \
+        field='small-6 columns', \
+        label='small-12 medium-4 columns', \
+        value='small-12 medium-8 columns')
+
+    >>> get_placement_classes("small:0,12,hide;medium:0,4")         # doctest: +NORMALIZE_WHITESPACE
+    Placement(width=LayoutOptions(s=12, m=4, l=4), \
+        offset=LayoutOptions(s=0, m=0, l=0), \
+        display=LayoutOptions(s=False, m=True, l=True), \
+        field='small-12 medium-4 columns show-for-medium-up', \
+        label='small-12 medium-6 columns', \
+        value='small-12 medium-6 columns')
     """
     def set_field_width(pmmode, pmwidth):
-        field_width[pmmode] = pmwidth
-        label_width[pmmode] = labelw[pmmode]*(12 // pmwidth)
-        value_width[pmmode] = 12 - label_width[pmmode]
-        if label_width[pmmode] >= 12:
-            label_width[pmmode] = 12
-            value_width[pmmode] = 12
+        if pmwidth == 0:
+            field_width[pmmode] = 0
+            label_width[pmmode] = 0
+            value_width[pmmode] = 0
+        else:
+            field_width[pmmode] = pmwidth
+            label_width[pmmode] = labelw[pmmode]*(12 // pmwidth)
+            value_width[pmmode] = 12 - label_width[pmmode]
+            if label_width[pmmode] >= 12:
+                label_width[pmmode] = 12
+                value_width[pmmode] = 12
         return
     def format_class(cd, right="", show=""):
         prev = cd.get("small", None)
@@ -250,8 +328,11 @@ def get_placement_classes(placement):
     value_width = OrderedDict()
     pmright     = ""
     pmshow      = ""
-    set_field_width("small", 12)        # Default small-12 columns (may be overridden)
+    set_field_width("small",  12)       # Default small-12  columns (may be overridden)
     set_field_width("medium", 12)       # Default medium-12 columns (may be overridden)
+    set_field_width("large",  12)       # Default large-12  columns (may be overridden)
+    field_offset  = {'small': 0, 'medium': 0, 'large': 0}
+    field_display = {'small': True, 'medium': True, 'large': True}
     # Process each placement sub-expression
     for p in ps:
         pm = ppr.match(p)
@@ -266,9 +347,25 @@ def get_placement_classes(placement):
             pmshow = {'small': "show-for-medium-up", 'medium': "show-for-large-up", 'large': ""}[pmmode]
             # print "pmhide %s, pmmode %s, pmshow %s"%(pmhide, pmmode, pmshow)
         set_field_width(pmmode, pmwidth)
+        field_offset[pmmode]  = pmoffset
+        if pmhide:
+            field_display[pmmode] = False
         if pmmode == "small":
             set_field_width("medium", pmwidth)
+        field_offset["medium"] = pmoffset
+        if pmmode in ["small", "medium"]:
+            set_field_width("large", pmwidth)
+            field_offset["large"] = pmoffset
     c = Placement(
+            width=make_field_width(
+                sw=field_width["small"], mw=field_width["medium"], lw=field_width["large"]
+                ),
+            offset=make_field_offset(
+                so=field_offset['small'], mo=field_offset['medium'],lo=field_offset['large']
+                ),
+            display=make_field_display(
+                sd=field_display['small'], md=field_display['medium'],ld=field_display['large']
+                ),
             field=format_class(field_width, pmright, pmshow),
             label=format_class(label_width),
             value=format_class(value_width)
@@ -276,6 +373,15 @@ def get_placement_classes(placement):
     # log.debug("get_placement_class %s, returns %s"%(placement,c))
     return c
 
+def make_field_width(sw=12, mw=12, lw=12):
+    return LayoutOptions(s=sw, m=mw, l=lw)
+
+def make_field_offset(so=0, mo=0, lo=0):
+    return LayoutOptions(s=so, m=mo, l=lo)
+
+def make_field_display(sd=True, md=True, ld=True):
+    return LayoutOptions(s=sd, m=md, l=ld)
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
