@@ -14,12 +14,24 @@ log = logging.getLogger(__name__)
 from django.http        import HttpResponse
 from django.template    import Template, Context
 
+from annalist.exceptions                import Annalist_Error
+
 from annalist.views.fields.bound_field  import bound_field
 
+from render_fieldvalue  import (
+    RenderFieldValue,
+    TemplateWrapValueRenderer, 
+    ModeWrapValueRenderer
+    )
+
+#   ------------------------------------------------------------
+#   Local data values
+#   ------------------------------------------------------------
+
 view_group = (
     { 'head':
         """
-        <!-- views.fields.render_repeatgroup.view_group -->
+        <!-- views.fields.render_repeatgroup.view_group (head) -->
         <div class="small-12 columns">
           <div class="row">
             <div class="group-label small-2 columns">
@@ -40,7 +52,7 @@ view_group = (
             <div class="small-10 columns">
               {% for f in repeat_bound_fields %}
                 <div class="view-group row">
-                  {% include f.field_render_label_view with field=f %}
+                  {% include f.render.label_view with field=f %}
                 </div>
               {% endfor %}
             </div>
@@ -53,16 +65,20 @@ view_group = (
 
 edit_group = (
     { 'head':
-        """<!-- views.fields.render_repeatgroup.edit_group -->
-        <div class="group-label small-2 columns">
-          <span>{{field.field_label}}</span>
-        </div>
-        <div class="group-placeholder small-10 columns">
-          {{field.field_placeholder}}
+        """<!-- views.fields.render_repeatgroup.edit_group (head) -->
+        <div class="small-12 columns"{{field.field_tooltip_attr|safe}}>
+          <div class="row">
+            <div class="group-label small-2 columns">
+              <span>{{field.field_label}}</span>
+            </div>
+            <div class="group-placeholder small-10 columns">
+              {{field.field_placeholder}}
+            </div>
+          </div>
         </div>"""
     , 'body':
         """
-        <div class="group-row small-12 columns">
+        <div class="group-row small-12 columns"{{field.field_tooltip_attr|safe}}>
           <div class="row selectable">
             <div class="small-2 columns checkbox-in-edit-padding">
               {% if auth_config %}
@@ -73,7 +89,7 @@ edit_group = (
             <div class="small-10 columns">
               {% for f in repeat_bound_fields %}
                 <div class="edit-group row">
-                  {% include f.field_render_label_edit with field=f %}
+                  {% include f.render.label_edit with field=f %}
                 </div>
               {% endfor %}
             </div>
@@ -103,9 +119,9 @@ edit_group = (
 view_grouprow = (
     { 'head':
         """
-        <!-- views.fields.render_repeatgroup.view_grouprow -->
+        <!-- views.fields.render_repeatgroup.view_grouprow (head) -->
         <div class="small-12 columns">
-          <div class="row">
+          <div class="grouprow row">
             <div class="group-label small-12 medium-2 columns">
               <span>{{field.field_label}}</span>
             </div>
@@ -113,8 +129,13 @@ view_grouprow = (
               <div class="row">
                 <div class="small-12 columns">
                   <div class="view-grouprow col-head row">
-                    {% for f in field.group_field_descs %}
-                    {% include f.field_render_colhead_view with field=f %}
+                    {% for f in group_head_fields %}
+                    <!-- ===== renderer {{f.render}} -->
+                    <!-- ===== {{f.field_render_type}}.{{f.field_value_mode}}/{{f.field_id}}:{{f.field_label}} ({{render_mode}}) -->
+                    <!-- ===== f.render.col_head_view... -->
+                    <!-- {_ include f.field_render_colhead_view with field=f %} -->
+                    {% include f.render.col_head_view with field=f %}
+                    <!-- ===== f.render.col_head_view end -->
                     {% endfor %}
                   </div>
                 </div>
@@ -127,12 +148,12 @@ view_grouprow = (
         """
         <!-- views.fields.render_repeatgroup.view_grouprow (empty list) -->
         <div class="small-12 columns">
-          <div class="row">
+          <div class="grouprow row">
             <div class="group-label small-12 medium-2 columns">
               <span>{{field.field_label}}</span>
             </div>
             <div class="group-placeholder small-12 medium-10 columns">
-              <span>(None)</span>
+              <span>(None specified)</span>
             </div>
           </div>
         </div>
@@ -140,7 +161,7 @@ view_grouprow = (
     , 'body':
         """
         <div class="small-12 columns">
-          <div class="row">
+          <div class="grouprow row">
             <div class="small-12 medium-2 columns">
               &nbsp;
             </div>
@@ -149,7 +170,7 @@ view_grouprow = (
                 <div class="small-12 columns">
                   <div class="view-grouprow row">
                     {% for f in repeat_bound_fields %}
-                    {% include f.field_render_colview with field=f %}
+                    {% include f.render.col_view with field=f %}
                     {% endfor %}
                   </div>
                 </div>
@@ -166,9 +187,9 @@ view_grouprow = (
 edit_grouprow = (
     { 'head':
         """
-        <!-- views.fields.render_repeatgroup.edit_grouprow -->
-        <div class="small-12 columns">
-          <div class="row">
+        <!-- views.fields.render_repeatgroup.edit_grouprow (head) -->
+        <div class="small-12 columns"{{field.field_tooltip_attr|safe}}>
+          <div class="grouprow row">
             <div class="group-label small-12 medium-2 columns">
               <span>{{field.field_label}}</span>
             </div>
@@ -179,8 +200,8 @@ edit_grouprow = (
                 </div>
                 <div class="small-11 columns">
                   <div class="edit-grouprow col-head row">
-                    {% for f in field.group_field_descs %}
-                    {% include f.field_render_colhead_edit with field=f %}
+                    {% for f in group_head_fields %}
+                    {% include f.render.col_head_edit with field=f %}
                     {% endfor %}
                   </div>
                 </div>
@@ -192,7 +213,7 @@ edit_grouprow = (
     , 'body':
         """
         <div class="small-12 columns">
-          <div class="row">
+          <div class="grouprow row">
             <div class="small-12 medium-2 columns hide-for-small-only">
               &nbsp;
             </div>
@@ -206,7 +227,7 @@ edit_grouprow = (
                 <div class="small-11 columns">
                   <div class="edit-grouprow row">
                     {% for f in repeat_bound_fields %}
-                    {% include f.field_render_coledit with field=f %}
+                    {% include f.render.col_edit with field=f %}
                     {% endfor %}
                   </div>
                 </div>
@@ -218,7 +239,7 @@ edit_grouprow = (
     , 'tail':
         """
         <div class="small-12 columns">
-          <div class="row">
+          <div class="grouprow row">
             <div class="small-12 medium-2 columns">
               &nbsp;
             </div>
@@ -247,15 +268,15 @@ edit_grouprow = (
 view_listrow = (
     { 'head':
         """
-        <!-- views.fields.render_repeatgroup.view_listrow -->
+        <!-- views.fields.render_repeatgroup.view_listrow (head) -->
         <div class="thead row">
           <div class="small-1 columns">
             &nbsp;
           </div>
           <div class="small-11 columns">
             <div class="view-listrow col-head row">
-              {% for f in field.group_field_descs %}
-              {% include f.field_render_colhead with field=f %}
+              {% for f in group_head_fields %}
+              {% include f.render.col_head with field=f %}
               {% endfor %}
             </div>
           </div>
@@ -271,7 +292,7 @@ view_listrow = (
           <div class="small-11 columns">
             <div class="view-listrow row">
               {% for f in repeat_bound_fields %}
-              {% include f.field_render_view with field=f %}
+              {% include f.render.view with field=f %}
               {% endfor %}
             </div>
           </div>
@@ -279,63 +300,9 @@ view_listrow = (
         """
     })
 
-edit_listrow_unused = (
-    { 'head':
-        """
-        <!-- views.fields.render_repeatgroup.edit_listrow -->
-        <div class="thead row">
-          <div class="small-12 columns">
-            <div class="row">
-              <div class="group-label small-12 columns">
-                <span>{{field.field_label}}</span>
-              </div>
-            </div>
-            <div class="row">
-              <div class="small-1 columns">
-                &nbsp;
-              </div>
-              <div class="small-11 columns">
-                <div class="edit-listrow col-head row">
-                  {% for f in field.group_field_descs %}
-                  {% include f.field_render_colhead with field=f %}
-                  {% endfor %}
-                </div>
-              </div>
-            </div>
-          </div>
-        </div>
-        """
-    , 'body':
-        """
-        <div class="tbody row select-row">
-          <div class="small-1 columns checkbox-in-edit-padding">
-            <input type="checkbox" class="select-box right" name="entity_select" 
-                   value="{{repeat_entity.entity_type_id}}/{{repeat_entity.entity_id}}" />
-          </div>
-          <div class="small-11 columns">
-            <div class="edit-listbody row">
-              {% for f in repeat_bound_fields %}
-              {% include f.field_render_coledit with field=f %}
-              {% endfor %}
-            </div>
-          </div>
-        </div>"""
-    , 'tail':
-        """
-        <div class="row">
-          <div class="small-12 columns">
-            <input type="submit" name="{{field.group_id}}__remove" 
-                   value="{{field.group_delete_label}}" />
-            <input type="submit" name="{{field.group_id}}__add"    
-                   value="{{field.group_add_label}}" />
-            <input type="submit" name="{{field.group_id}}__up"
-                   value="Move up" />
-            <input type="submit" name="{{field.group_id}}__down"
-                   value="Move down" />
-          </div>
-        </div>"""
-    })
-
+#   ------------------------------------------------------------
+#   Repeat group render class
+#   ------------------------------------------------------------
 
 class RenderRepeatGroup(object):
     """
@@ -343,10 +310,10 @@ class RenderRepeatGroup(object):
     """
 
     def __init__(self, templates=None):
+        """
+        Creates a renderer object for a repeating group field
+        """
         # Later, may introduce a template_file= option to read from templates directory
-        """
-        Creates a renderer object for a simple text field
-        """
         # log.info("RenderRepeatGroup: __init__ %r"%(templates))
         super(RenderRepeatGroup, self).__init__()
         assert templates is not None, "RenderRepeatGroup template must be supplied (.edit, .view or .item)"
@@ -382,32 +349,34 @@ class RenderRepeatGroup(object):
         try:
             # log.info("RenderRepeatGroup.render field: %r"%(context['field'],))
             # log.info("RenderRepeatGroup.render descs: %r"%(context['field']['group_field_descs'],))
-            value_list     = context['field']['field_value']
-            if len(value_list) > 0:
-                response_parts = [self._template_head.render(context)]
-                repeat_index = 0
-                extras       = context['field']['context_extra_values']
-                for g in value_list:
-                    log.debug("RenderRepeatGroup.render field_val: %r"%(g))
-                    r = [ bound_field(f, g, context_extra_values=extras) 
-                          for f in context['field']['group_field_descs'] ]
-                    repeat_id = context.get('repeat_prefix', "") + context['field']['group_id']
-                    repeat_dict = (
-                        { 'repeat_id':            repeat_id
-                        , 'repeat_index':         str(repeat_index)
-                        , 'repeat_prefix':        repeat_id+("__%d__"%repeat_index)
-                        , 'repeat_bound_fields':  r
-                        , 'repeat_entity':        g
-                        })
-                    # log.info("RenderRepeatGroup.render repeat_dict: %r"%(repeat_dict))
-                    with context.push(repeat_dict):
-                        response_parts.append(self._template_body.render(context))
-                    repeat_index += 1
-                response_parts.append(self._template_tail.render(context))
-            else:
-                # Empty list
-                response_parts = [self._template_empty.render(context)]
-                response_parts.append(self._template_tail.render(context))
+            h = [ bound_field(f, {}) for f in context['field']['group_field_descs'] ]
+            with context.push({ 'group_head_fields': h }):
+                value_list     = context['field']['field_value']
+                if len(value_list) > 0:
+                    response_parts = [self._template_head.render(context)]
+                    repeat_index = 0
+                    extras       = context['field']['context_extra_values']
+                    for g in value_list:
+                        # log.debug("RenderRepeatGroup.render field_val: %r"%(g))
+                        r = [ bound_field(f, g, context_extra_values=extras) 
+                              for f in context['field']['group_field_descs'] ]
+                        repeat_id = context.get('repeat_prefix', "") + context['field']['group_id']
+                        repeat_dict = (
+                            { 'repeat_id':            repeat_id
+                            , 'repeat_index':         str(repeat_index)
+                            , 'repeat_prefix':        repeat_id+("__%d__"%repeat_index)
+                            , 'repeat_bound_fields':  r
+                            , 'repeat_entity':        g
+                            })
+                        # log.info("RenderRepeatGroup.render repeat_dict: %r"%(repeat_dict))
+                        with context.push(repeat_dict):
+                            response_parts.append(self._template_body.render(context))
+                        repeat_index += 1
+                    response_parts.append(self._template_tail.render(context))
+                else:
+                    # Empty list
+                    response_parts = [self._template_empty.render(context)]
+                    response_parts.append(self._template_tail.render(context))
         except Exception as e:
             log.exception("Exception in RenderRepeatGroup.render")
             ex_type, ex, tb = sys.exc_info()
@@ -421,13 +390,93 @@ class RenderRepeatGroup(object):
             del tb
         return "".join(response_parts)
 
-    def render_mode(self):
-        """
-        Returns a renderer object that renders whatever is required for the 
-        current value of "render_mode" in the view context.
 
-        In the case of a repeat group, invokes the repeat render object.
+#   ------------------------------------------------------------
+#   Repeat group renderer factory class and functions
+#   ------------------------------------------------------------
+
+class RenderGroupFieldValue(RenderFieldValue):
+    """
+    Return field render factory for a repeated group based on 
+    the supplied render value objects.
+    """
+    def __init__(self, render_type, view_renderer=None, edit_renderer=None):
         """
-        return self
+        Creates a renderer factory for a repeating value field.
+
+        view_renderer   is a render object that formats values for viewing
+        edit_renderer   is a render object that formats value editing widgets
+
+        Methods provided return composed renderers for a variety of contexts.
+        """
+        super(RenderGroupFieldValue, self).__init__(render_type,
+            view_renderer=view_renderer, 
+            edit_renderer=edit_renderer
+            )
+        # Override view/edit renderers to not use wrapper.
+        self._render_view       = ModeWrapValueRenderer("view", self._view_renderer)
+        self._render_edit       = ModeWrapValueRenderer("edit", self._edit_renderer)
+        #@@TODO: remove labelling from templates instead?
+        self._render_label_view = ModeWrapValueRenderer("view", self._view_renderer)
+        self._render_label_edit = ModeWrapValueRenderer("edit", self._edit_renderer)
+        self._render_col_view   = ModeWrapValueRenderer("view", self._view_renderer)
+        self._render_col_edit   = ModeWrapValueRenderer("edit", self._edit_renderer)
+        return
+
+    # def label_view(self):
+    #     log.warning("RepeatGroup renderer has no label_view method")
+    #     raise Annalist_Error("RepeatGroup renderer has no label_view method")
+
+    # def label_edit(self):
+    #     log.warning("RepeatGroup renderer has no label_edit method")
+    #     raise Annalist_Error("RepeatGroup renderer has no label_edit method")
+
+    # def col_head(self):
+    #     log.warning("RepeatGroup renderer has no col_head method")
+    #     raise Annalist_Error("RepeatGroup renderer has no col_head method")
+
+    # def col_head_view(self):
+    #     log.warning("RepeatGroup renderer has no col_head_view method")
+    #     raise Annalist_Error("RepeatGroup renderer has no col_head_view method")
+
+    # def col_head_edit(self):
+    #     log.warning("RepeatGroup renderer has no col_head_edit method")
+    #     raise Annalist_Error("RepeatGroup renderer has no col_head_edit method")
+
+    # def col_view(self):
+    #     log.warning("RepeatGroup renderer has no col_view method")
+    #     raise Annalist_Error("RepeatGroup renderer has no col_view method")
+
+    # def col_edit(self):
+    #     log.warning("RepeatGroup renderer has no col_edit method")
+    #     raise Annalist_Error("RepeatGroup renderer has no col_edit method")
+
+
+def get_repeatgroup_renderer():
+    """
+    Return field renderer object for RepeatGroup (labeled fields)
+    """
+    return RenderGroupFieldValue("repeatgroup",
+        view_renderer=RenderRepeatGroup(view_group),
+        edit_renderer=RenderRepeatGroup(edit_group)
+        )
+
+def get_repeatgrouprow_renderer():
+    """
+    Return field renderer object for RepeatGroup as row (col headier labels)
+    """
+    return RenderGroupFieldValue("repeatgrouprow",
+        view_renderer=RenderRepeatGroup(view_grouprow),
+        edit_renderer=RenderRepeatGroup(edit_grouprow)
+        )
+
+def get_repeatlistrow_renderer():
+    """
+    Return field renderer object for RepeatGroup as list (col header labels)
+    """
+    return RenderGroupFieldValue("repeatlistrow",
+        view_renderer=RenderRepeatGroup(view_listrow),
+        edit_renderer=Template("@@repeatlistrow_renderer cannot be used for editing@@")
+        )
 
 # End.
