@@ -22,7 +22,7 @@ from annalist.views.fields.render_fieldvalue    import (
     get_field_edit_value,
     get_field_view_value
     )
-from annalist.views.form_utils.fieldchoice      import FieldChoice
+from annalist.views.form_utils.fieldchoice      import FieldChoice, update_choice_labels
 
 from django.template        import Template, Context
 
@@ -42,13 +42,13 @@ edit_options = (
         '''{% if opt.value == "" %} '''+
           '''<option value="" selected="selected">{{field.field_placeholder}}</option>\n'''+
         '''{% else %} '''+
-          '''<option value="{{opt.value}}" selected="selected">{{opt.label}}&nbsp;&nbsp;&nbsp;({{opt.value}})</option>\n'''+
+          '''<option value="{{opt.value}}" selected="selected">{{opt.choice_html}}</option>\n'''+
         '''{% endif %} '''+
       '''{% else %} '''+
         '''{% if opt.value == "" %} '''+
           '''<option value="">{{field.field_placeholder}}</option>\n'''+
         '''{% else %} '''+
-          '''<option value="{{opt.value}}">{{opt.label}}&nbsp;&nbsp;&nbsp;({{opt.value}})</option>\n'''+
+          '''<option value="{{opt.value}}">{{opt.choice_html}}</option>\n'''+
         '''{% endif %} '''+
       '''{% endif %} '''+
     '''{% endfor %} '''
@@ -57,7 +57,7 @@ edit_options = (
 view_select = (
     """<!-- fields.render_select.view_select -->
     {% if field_linkval %}
-      <a href="{{field_linkval}}">{{field_labelval}}</a>
+      <a href="{{field_linkval}}{{field_continuation_param}}">{{field_labelval}}</a>
     {% elif field_textval and field_textval != "" %}
       <span class="value-missing">{{field_labelval}}</span>
     {% else %}
@@ -81,7 +81,7 @@ edit_select = (
         <button type="submit" 
                 name="{{repeat_prefix}}{{field.field_name}}__new_edit" 
                 value="New"
-                title="Define new {{field.field_label}}"
+                title="Define new or edit {{field.field_label}}"
         >
           <span class="select-edit-button-text">+&#x270D;</span>
         </button>
@@ -92,7 +92,7 @@ edit_select = (
 view_choice = (
     """<!-- fields.render_select.view_choice -->
     {% if field_linkval %}
-      <a href="{{field_linkval}}">{{field_labelval}}</a>
+      <a href="{{field_linkval}}{{field_continuation_param}}">{{field_labelval}}</a>
     {% elif field_textval and field_textval != "" %}
       <span class="value-missing">{{field_labelval}}</span>
     {% else %}
@@ -114,7 +114,7 @@ edit_choice = (
 view_entitytype = (
     """<!-- fields.render_select.view_entitytype -->
     {% if field_linkval %}
-      <a href="{{field_linkval}}">{{field_labelval}}</a>
+      <a href="{{field_linkval}}{{field_continuation_param}}">{{field_labelval}}</a>
     {% elif field_textval and field_textval != "" %}
       <span class="value-missing">{{field_labelval}}</span>
     {% else %}
@@ -245,8 +245,7 @@ class Select_edit_renderer(object):
             # Use refer-to type if value does not include type..
             typval  = fill_type_entity_id(val, context['field']['field_ref_type'])
             textval = SelectValueMapper.encode(typval)
-            options = context['field']['options']
-            # options is a list of FieldChoice values
+            options = update_choice_labels(context['field']['options'])
             # print repr(options)
             if textval not in [ o.value for o in options ]:
                 options = list(options)                   # clone
@@ -274,7 +273,7 @@ def get_select_renderer():
     """
     Return field renderer object for value selector (with '+' button)
     """
-    return RenderFieldValue(
+    return RenderFieldValue("select",
         view_renderer=Select_view_renderer(view_select),
         edit_renderer=Select_edit_renderer(edit_select),
         )
@@ -283,7 +282,7 @@ def get_choice_renderer():
     """
     Return field renderer object for value selector (without '+' button)
     """
-    return RenderFieldValue(
+    return RenderFieldValue("choice",
         view_renderer=Select_view_renderer(view_choice),
         edit_renderer=Select_edit_renderer(edit_choice),
         )
@@ -292,16 +291,19 @@ def get_entitytype_renderer():
     """
     Return field renderer object for entitytype
     """
-    return RenderFieldValue(
+    return RenderFieldValue("entitytype",
         view_renderer=Select_view_renderer(view_entitytype),
         edit_renderer=Select_edit_renderer(edit_entitytype),
         )
 
 def get_view_choice_renderer():
     """
-    Return field renderer object for entitytype
+    Return field renderer object for "view choice" controlm, which combines
+    a regular selection box with a "Show view" button.  This option is used
+    by the view template/renderer, and is not available as a field choice 
+    within the view.
     """
-    return RenderFieldValue(
+    return RenderFieldValue("view_choice",
         view_renderer=Select_view_renderer(view_view_choice),
         edit_renderer=Select_edit_renderer(edit_view_choice),
         )
