@@ -3,7 +3,7 @@ Renderer and value mapper for text value rendered as Markdown.
 """
 
 __author__      = "Graham Klyne (GK@ACM.ORG)"
-__copyright__   = "Copyright 2014, G. Klyne"
+__copyright__   = "Copyright 2015, G. Klyne"
 __license__     = "MIT (http://opensource.org/licenses/MIT)"
 
 import logging
@@ -11,6 +11,8 @@ log = logging.getLogger(__name__)
 
 import markdown
 
+from annalist.views.displayinfo                 import apply_substitutions
+
 from annalist.views.fields.render_base          import RenderBase
 from annalist.views.fields.render_fieldvalue    import (
     RenderFieldValue,
@@ -51,6 +53,7 @@ class text_markdown_view_renderer(object):
         Render Markdown text for viewing.
         """
         textval = TextMarkdownValueMapper.encode(get_field_view_value(context, None))
+        textval = apply_substitutions(context, textval)
         htmlval = markdown.markdown(textval)
         return """<span class="markdown">%s</span>"""%htmlval
 
@@ -77,11 +80,20 @@ class text_markdown_edit_renderer(object):
 
 def get_text_markdown_renderer():
     """
-    Return field renderer object for Markdoiwn text
+    Return field renderer object for Markdown text
     """
-    return RenderFieldValue(
+    return RenderFieldValue("markdown",
         view_renderer=text_markdown_view_renderer(), 
         edit_renderer=text_markdown_edit_renderer(),
         )
 
+def get_show_markdown_renderer():
+    """
+    Return field renderer object for display-only (no input) of Markdown text
+    """
+    return RenderFieldValue("show_markdown",
+        view_renderer=text_markdown_view_renderer(), 
+        edit_renderer=text_markdown_view_renderer(),
+        )
+
 # End.
