@@ -8,6 +8,7 @@ __author__      = "Graham Klyne (GK@ACM.ORG)"
 __copyright__   = "Copyright 2014, G. Klyne"
 __license__     = "MIT (http://opensource.org/licenses/MIT)"
 
+import traceback
 import logging
 log = logging.getLogger(__name__)
 
@@ -52,6 +53,7 @@ class TokenSetValueMapper(RenderBase):
         if isinstance(field_value,(str,unicode)):
             return field_value.split()
         log.warning("TokenSetValueMapper.decode: %r"%(field_value,))
+        # log.info("\n".join(traceback.format_stack()))
         return [field_value]
 
 #   ----------------------------------------------------------------------------
@@ -89,7 +91,7 @@ def get_field_tokenset_renderer():
     """
     Return field renderer object for token list values
     """
-    return RenderFieldValue(
+    return RenderFieldValue("tokenset",
         view_renderer=tokenset_view_renderer(), 
         edit_renderer=tokenset_edit_renderer(),
         )
