@@ -80,14 +80,14 @@ class EntityValueMap(object):
             context.update(kval)
         return context
 
-    def map_form_data_to_values(self, form_data, entityvals, **kwargs):
+    def map_form_data_to_values(self, form_data, entityvals):
         """
         Map data from form response to entity data.  
 
         Returns a deep copy of the supplied `entityvals` updated with values from
-        then form.  Values not mentioned in the form data are not updated.
+        the form.  Values not mentioned in the form data are not updated.
         """
-        log.debug("map_form_data_to_values: form_data %r, entityvals %r"%(form_data, entityvals))
+        # log.debug("map_form_data_to_values: form_data %r, entityvals %r"%(form_data, entityvals))
         values = copy.deepcopy(entityvals) or {}
         for kmap in self._map:
             kmap.map_form_to_entity(form_data, values)
