@@ -38,7 +38,7 @@ class SiteView(AnnalistGenericView):
     """
     def __init__(self):
         super(SiteView, self).__init__()
-        self.help = "site-help"
+        self.help_markdown = None
         return
 
     # GET
@@ -51,10 +51,11 @@ class SiteView(AnnalistGenericView):
         viewinfo = DisplayInfo(self, "view", {}, None)    # No continuation
         viewinfo.get_site_info(self.get_request_host())
         viewinfo.get_coll_info(layout.SITEDATA_ID)
-        viewinfo.get_type_info(entitytypeinfo.COLL_ID)
+        viewinfo.get_request_type_info(entitytypeinfo.COLL_ID)
         viewinfo.check_authorization("view")
         if viewinfo.http_response:
             return viewinfo.http_response
+        self.help_markdown = viewinfo.collection.get(RDFS.CURIE.comment, None)
         resultdata = viewinfo.sitedata
         resultdata.update(viewinfo.context_data())
         # log.info("SiteView.get: site_data %r"%(self.site_data()))
