@@ -20,6 +20,12 @@ unreserved  = "-._~"
 # subset of above safe in query string (no "?", "&" or #")
 query_safe  = re.sub('[?&#]', '', gen_delims + sub_delims + unreserved)
 
+def uri_quote_param(pval):
+    """
+    Apply escaping to a supplied query parameter value for inclusion in a URI.
+    """
+    return urllib.quote(pval, query_safe)
+
 def uri_base(uri):
     """
     Get the base URI from the supplied URI by removing any parameters and/or fragments.
@@ -55,8 +61,9 @@ def uri_param_dict(uri):
 def build_dict(*param_dicts, **param_dict):
     merged_param_dict = param_dict.copy()
     for d in param_dicts:
-        # log.info("param_dicts %r"%(d,))
-        merged_param_dict.update(d)
+        if d is not None:
+            # log.info("param_dicts %r"%(d,))
+            merged_param_dict.update(d)
     return merged_param_dict
 
 def uri_params(*param_dicts, **param_dict):
@@ -71,17 +78,52 @@ def uri_params(*param_dicts, **param_dict):
         pval = uri_param_dict[pnam]
         if pval:
             # log.info("pnam %s, pval %s, uri_param_dict %r"%(pnam, pval, uri_param_dict))
-            uri_param_str += next_sep + pnam + "=" + urllib.quote(pval, query_safe)
+            uri_param_str += next_sep + pnam + "=" + uri_quote_param(pval)
             next_sep = "&"
     return uri_param_str
 
 def uri_with_params(base_uri, *param_dicts, **param_dict):
     """
     Construct a URI from the supplied base URI (with any parameters and/or fragment removed)
-    and URI paramneters created using the supplied dictionary values.
+    and URI parameters created using the supplied dictionary values.
     """
     return uri_base(base_uri) + uri_params(*param_dicts, **param_dict)
 
+def scope_params(*param_dicts, **param_dict):
+    """
+    Return URI parameters from the supplied dictionary specifically used for entity selection,
+    ignoring all others.  These are the parameters which, in conjunction with a base URI, 
+    represent a resource or set of resources to be returned.
+
+    Preserves the following query params from original request:
+        scope
+        search
+
+    Query parameters not preserved (among others):
+        continuation_url
+        info_head
+        info_message
+        error_head
+        error_message
+        add_field
+        type
+    """
+    uri_param_dict = build_dict(*param_dicts, **param_dict)
+    return (
+        { 'search':           uri_param_dict.get('search_for')       or 
+                              uri_param_dict.get('search')           or None
+        , 'scope':            uri_param_dict.get('scope')            or None
+        })
+
+def _unused_scope_params_url(base_url, type=None):
+    """
+    Takes a supplied URL and returns a corresponding continuation URL with all
+    but scope parameters removed (c.f. scope_params above).
+    """
+    url_params = uri_param_dict(base_url)
+    scope_url  = uri_with_params(base_url, scope_params(url_params))
+    return scope_url
+
 def continuation_params(*param_dicts, **param_dict):
     """
     Return URI parameters from the supplied dictionary specifically needed for a continuation 
@@ -93,8 +135,18 @@ def continuation_params(*param_dicts, **param_dict):
         { 'continuation_url': uri_param_dict.get('continuation_url') or None
         , 'search':           uri_param_dict.get('search_for')       or 
                               uri_param_dict.get('search')           or None
+        , 'scope':            uri_param_dict.get('scope')            or None
         })
 
+def continuation_params_url(base_url):
+    """
+    Takes a supplied URL and returns a corresponding continuation URL with transient 
+    query parameters removed (cf. continuation_params above).
+    """
+    url_params = uri_param_dict(base_url)
+    cont_url   = uri_with_params(base_url, continuation_params(url_params))
+    return cont_url
+
 def continuation_url_chain(continuation_url):
     """
     Disects a supplied continuation URL into its components going back up the return chain.
