@@ -30,14 +30,22 @@ import annalist
 # from annalist_manager       import am_errors
 import am_errors
 from am_runtests            import am_runtests
-from am_runserver           import am_runserver, am_serverlog, am_sitedirectory, am_version
 from am_initialize          import am_initialize
+from am_createsite          import am_createsite, am_updatesite
+from am_runserver           import (
+    am_runserver, am_serverlog, am_sitedirectory, 
+    am_settingsmodule, am_settingsfile, am_settingsdir, 
+    am_version
+    )
 from am_createuser          import (
-    am_createadminuser, am_defaultadminuser, am_updateadminuser, 
-    am_setdefaultpermissions, am_setpublicpermissions,
+    am_createlocaluser, am_createadminuser, am_defaultadminuser, am_updateadminuser, 
+    am_setuserpermissions, am_setdefaultpermissions, am_setpublicpermissions,
     am_deleteuser
     )
-from am_createsite          import am_createsite, am_updatesite
+from am_managecollections   import (
+    am_installcollection, am_copycollection,
+    am_migrationreport, am_migratecollection, am_migrateallcollections
+    )
 from am_help                import am_help, command_summary_help
 
 VERSION = annalist.__version__
@@ -82,12 +90,14 @@ def parseCommandArgs(argv):
     parser.add_argument("-f", "--force",
                         action='store_true',
                         dest="force",
-                        help="Force overwrite of existing site data.")
+                        help="Force overwrite of existing site data or collection.")
     parser.add_argument("--debug",
                         action="store_true", 
                         dest="debug", 
                         default=False,
-                        help="Run with full debug output enabled")
+                        help="Run with full debug output enabled.  "+
+                             "Also creates log file 'annalist-manager.log' in the working directory"
+                        )
     parser.add_argument("command", metavar="COMMAND",
                         nargs=None,
                         help="sub-command, one of the options listed below."
@@ -116,12 +126,16 @@ def run(userhome, userconfig, options, progname):
         return am_runtests(annroot, options)
     if options.command.startswith("init"):                  # initialize (intsllaation, django database)
         return am_initialize(annroot, userhome, userconfig, options)
+    if options.command.startswith("createl"):               # createlocaluser
+        return am_createlocaluser(annroot, userhome, options)
     if options.command.startswith("createa"):               # createadminuser
         return am_createadminuser(annroot, userhome, options)
     if options.command.startswith("defaulta"):              # defaultadminuser
         return am_defaultadminuser(annroot, userhome, options)
     if options.command.startswith("updatea"):               # updateadminuser
         return am_updateadminuser(annroot, userhome, options)
+    if options.command.startswith("setuse"):                # setuserpermissions
+        return am_setuserpermissions(annroot, userhome, options)
     if options.command.startswith("setdef"):                # setdefaultpermissions
         return am_setdefaultpermissions(annroot, userhome, options)
     if options.command.startswith("setpub"):                # setpublicpermissions
@@ -132,12 +146,28 @@ def run(userhome, userconfig, options, progname):
         return am_createsite(annroot, userhome, options)
     if options.command.startswith("updates"):               # updatesitedata
         return am_updatesite(annroot, userhome, options)
+    if options.command.startswith("installc"):              # installcollection
+        return am_installcollection(annroot, userhome, options)
+    if options.command.startswith("copyc"):                 # copycollection
+        return am_copycollection(annroot, userhome, options)
+    if options.command.startswith("migrationr"):            # migrationreport
+        return am_migrationreport(annroot, userhome, options)
+    if options.command.startswith("migratec"):              # migratecollection
+        return am_migratecollection(annroot, userhome, options)
+    if options.command.startswith("migratea"):              # migrateallcollections
+        return am_migrateallcollections(annroot, userhome, options)
     if options.command.startswith("runs"):                  # runserver
         return am_runserver(annroot, userhome, options)
     if options.command.startswith("serv"):                  # serverlog
         return am_serverlog(annroot, userhome, options)
     if options.command.startswith("site"):                  # sitedir
         return am_sitedirectory(annroot, userhome, options)
+    if options.command.startswith("settingsm"):             # settingsmodule
+        return am_settingsmodule(annroot, userhome, options)
+    if options.command.startswith("settingsf"):             # settingsfile
+        return am_settingsfile(annroot, userhome, options)
+    if options.command.startswith("settingsd"):             # settingsdir
+        return am_settingsdir(annroot, userhome, options)
     if options.command.startswith("ver"):                   # version
         return am_version(annroot, userhome, options)
     if options.command.startswith("help"):
@@ -156,10 +186,11 @@ def runCommand(userhome, userconfig, argv):
     Returns exit status.
     """
     options = parseCommandArgs(argv[1:])
-    if options and options.debug:
-        logging.basicConfig(level=logging.DEBUG)
-    else:
-        logging.basicConfig(level=logging.INFO)
+    # if options and options.debug:
+    #     # This doesn't work as expected - some kind of interference with Django log settings?
+    #     logging.basicConfig(level=logging.DEBUG, filename="annalist-manager.log", filemode="w")
+    # else:
+    #     logging.basicConfig(level=logging.INFO)
     # log.debug("runCommand: configbase %s, filebase %s, argv %s"%(configbase, filebase, repr(argv)))
     # log.debug("Options: %s"%(repr(options)))
     # else:
@@ -192,6 +223,7 @@ if __name__ == "__main__":
 #
 # python am_main.py runtests
 # python am_main.py --config=runtests initialize
+# python am_main.py --config=runtests createlocaluser testtestuser  testestuser@localhost
 # python am_main.py --config=runtests createadminuser testtestadmin testestadmin@localhost
 # python am_main.py --config=runtests createadminuser testtestadmin testestadmin@localhost
 # python am_main.py --config=runtests deleteuser testtestadmin
