@@ -16,6 +16,7 @@ import os
 import django
 import sys
 import logging
+import logging.handlers
 from annalist import __version__
 from annalist import layout
 
@@ -25,6 +26,26 @@ SITE_CONFIG_DIR = os.path.dirname(SETTINGS_DIR)                         # src/an
 SITE_SRC_ROOT   = os.path.dirname(SITE_CONFIG_DIR)                      # src/annalist_root
 SAMPLEDATA_DIR  = SITE_SRC_ROOT+"/sampledata"                           # src/annalist_root/sampledata
 
+class RotatingNewFileHandler(logging.handlers.RotatingFileHandler):
+    """
+    Define a rotating file logging handler that additionally forces a new file 
+    the first time it is instantiated in a run of the containing program.
+
+    NOTE: if multiple file hanfdlers are used with in an application, only the
+    first one instantiated will be allocated a new file at startup.  The
+    class variable '_newfile' might be replaced with a dictionary
+    indexed by the (fully expanded) filename.
+    """
+
+    _newfile = False
+
+    def __init__(self, *args, **kwargs):
+        super(RotatingNewFileHandler, self).__init__(*args, **kwargs)
+        if not RotatingNewFileHandler._newfile:
+            self.doRollover()
+            RotatingNewFileHandler._newfile = True
+        return
+
 # Quick-start development settings - unsuitable for production
 # See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/
 
@@ -40,7 +61,7 @@ TEMPLATE_DEBUG = False
 # for tracing field values displayed in Annalist edit/view forms.
 # Suggested use is to raise level to logging.INFO when running a single named
 # test, when trying to understand how values end up in a form.
-TRACE_FIELD_VALUE   = logging.DEBUG
+TRACE_FIELD_VALUE   = logging.INFO
 
 ALLOWED_HOSTS = []
 
@@ -54,7 +75,7 @@ INSTALLED_APPS = (
     'django.contrib.messages',
     'django.contrib.staticfiles',
     'annalist',
-    'oauth2',
+    'login',
 )
 
 MIDDLEWARE_CLASSES = (
@@ -73,7 +94,7 @@ WSGI_APPLICATION = 'annalist_site.wsgi.application'
 # Customize authentication backends
 AUTHENTICATION_BACKENDS = (
     'django.contrib.auth.backends.ModelBackend',    # default
-    'oauth2.OAuth2CheckBackend.OAuth2CheckBackend'
+    'login.OAuth2CheckBackend.OAuth2CheckBackend'
     )
 
 # Database
@@ -109,6 +130,7 @@ STATIC_URL = '/static/'
 STATICFILES_DIRS = (
     # SITE_SRC_ROOT+"/static/",
     SITE_SRC_ROOT+"/annalist/data/static/",
+    SITE_SRC_ROOT+"/annalist/data/identity_providers/",
 )
 
 ANNALIST_VERSION = __version__
