@@ -21,7 +21,7 @@ ALLOWED_HOSTS   = ['*']     # Insecure: use e.g. ['.annalist.net']
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': os.path.join(BASE_DATA_DIR, 'annalist_site/_annalist_site/db.sqlite3'),
+        'NAME': os.path.join(BASE_DATA_DIR, 'annalist_site/db.sqlite3'),
     }
 }
 
@@ -53,10 +53,14 @@ LOGGING = {
         },
         # Log to a text file that can be rotated by logrotate
         'logfile': {
-            'class': 'logging.handlers.WatchedFileHandler',
-            'level': 'INFO',
-            'filename': LOGGING_FILE,
-            'formatter': 'timed'
+            # 'class': 'logging.handlers.WatchedFileHandler',
+            # 'class':        'logging.handlers.RotatingFileHandler',
+            'class':        'annalist_site.settings.common.RotatingNewFileHandler',
+            'filename':     LOGGING_FILE,
+            'maxBytes':     2*1024*1024,            # 2Mb
+            'backupCount':  9,                      # Keep 9 files
+            'level':        TRACE_FIELD_VALUE,
+            'formatter':    'timed'
         },
     },
     'loggers': {
@@ -77,6 +81,11 @@ LOGGING = {
             'level': 'INFO',
             'propagate': False,
         },
+        'annalist_root': {
+            'handlers': ['logfile'],
+            'level': 'INFO', # Or maybe INFO or DEBUG
+            'propagate': False
+        },
         'annalist_site': {
             'handlers': ['logfile'],
             'level': 'INFO', # Or maybe INFO or DEBUG
@@ -84,10 +93,10 @@ LOGGING = {
         },
         'annalist': {
             'handlers': ['logfile'],
-            'level': 'INFO', # Or maybe INFO or DEBUG
+            'level': TRACE_FIELD_VALUE, # Or maybe INFO or DEBUG
             'propagate': False
         },
-        'oauth2': {
+        'login': {
             'handlers': ['logfile'],
             'level': 'INFO', # Or maybe INFO or DEBUG
             'propagate': False
@@ -97,6 +106,15 @@ LOGGING = {
 
 import logging
 log = logging.getLogger(__name__)
+log.info("Annalist starting...")
+
+# Force new log files for any rotating file log handlers
+for h in log.handlers:
+    log.info("@@ log handler %r"%(h,))
+    if isinstance(h, logging.handlers.RotatingFileHandler):
+        log.info("@@ log rollover")        
+        h.doRollover()
+
 # log.info("Annalist version %s (personal configuration)"%(ANNALIST_VERSION))
 log.info(ANNALIST_VERSION_MSG)
 log.debug("SETTINGS_MODULE:  "+SETTINGS_MODULE)
