@@ -12,8 +12,8 @@ from django.conf.urls           import patterns, include, url
 from django.contrib             import admin
 admin.autodiscover()
 
-from annalist.views.home        import AnnalistHomeView
-from annalist.views.statichack  import serve_static
+from annalist.views.home_redirects  import AnnalistHomeView
+from annalist.views.statichack      import serve_static
 
 urlpatterns = patterns('',
     url(r'^$',         AnnalistHomeView.as_view(), name='AnnalistHomeView'),
@@ -27,3 +27,5 @@ if not settings.DEBUG:
     urlpatterns += patterns('',
         url(r'^static/(?P<path>.*)$', serve_static),
         )
+
+# End.
