@@ -20,15 +20,18 @@ class ContentNegotiationView(generic.View):
     def accept_types(types):
         """
         Decorator to use associated function to render the indicated content types 
+
+        Invokes decorated method and returns its result if accept content type matches,
+         otherwise returns None.
         """
         def decorator(func):
-            def guard(self, *values):
+            def guard(self, *values, **kwargs):
                 accept_header = self.request.META.get('HTTP_ACCEPT', "*/*")
                 accept_types  = [ a.split(';')[0].strip().lower() 
                                   for a in accept_header.split(',') ]
                 for t in types:
                     if t in accept_types:
-                        return func(self, *values)
+                        return func(self, *values, **kwargs)
                 return None
             return guard
         return decorator
@@ -36,13 +39,17 @@ class ContentNegotiationView(generic.View):
     @staticmethod
     def content_types(types):
         """
-        Decorator to use associated function when supplied with the indicated content types 
+        Decorator to use associated function when supplied with (one of)
+        the indicated content types in a POST request.
+
+        Invokes decorated method and returns its result if content type matches, 
+        otherwise returns None.
         """
         def decorator(func):
-            def guard(self, *values):
+            def guard(self, *values, **kwargs):
                 content_type = self.request.META.get('CONTENT_TYPE', "application/octet-stream")
                 if content_type.split(';')[0].strip().lower() in types:
-                    return func(self, *values)
+                    return func(self, *values, **kwargs)
                 return None
             return guard
         return decorator
